(function () {
    'use strict';

    async function prepareMockActions({ connection: { appId, backend: { emails, mockBrowser } }, }) {
        return {
            async asMockPerson(email, action) {
                const browser = await mockBrowser();
                const tab = await browser.mockAppWindow({
                    appId,
                    apiLink: window.location.href,
                    windowLink: window.location.href,
                });
                const { accessModel } = tab.models;
                const { loginService } = tab.remote.auth.users;
                emails.disableEmails();
                await loginService.sendLoginLink({ email });
                const { loginToken } = emails.recallLatestLoginEmail();
                await accessModel.login(loginToken);
                await action(tab);
                await accessModel.logout();
                emails.enableEmails();
            },
        };
    }

    async function demoQuestions(connection) {
        const { asMockPerson } = await prepareMockActions({ connection });
        let questionId;
        await asMockPerson("a@xiome.io", async (tab) => {
            const boardModel = tab.models.questionsModel.makeBoardModel("default");
            const question = await boardModel.postQuestion({
                content: "How much wood would a woodchuck chuck if a woodchuck could chuck wood?",
            });
            questionId = question.questionId;
            await boardModel.likeQuestion(questionId, true);
        });
        await asMockPerson("creative@xiome.io", async (tab) => {
            const boardModel = tab.models.questionsModel.makeBoardModel("default");
            await boardModel.loadQuestions();
            await boardModel.likeQuestion(questionId, true);
            await boardModel.postAnswer(questionId, {
                content: "Researchers say,\nApproximately 700 pounds of wood.",
            });
        });
    }

    const nap = (milliseconds) => new Promise(resolve => setTimeout(resolve, milliseconds));

    function hitchie(func, process) {
        return ((...args) => process(func, ...args));
    }

    function applyMockHacks({ connection, frontend }) {
        const { loginService } = connection.remote.auth.users;
        loginService.sendLoginLink = hitchie(loginService.sendLoginLink, async (func, ...args) => {
            await func(...args);
            const details = connection.backend.emails.recallLatestLoginEmail();
            console.log("mock: logging in...");
            await nap(1000);
            await frontend.models.accessModel.login(details.loginToken);
            console.log(`mock: logged in as ${details.to}`);
        });
        connection.popups.triggerBankPopup = hitchie(connection.popups.triggerBankPopup, async (...args) => {
            console.log("mock: bank popup", args);
        });
    }

    function memoryFlexStorage() {
        const map = new Map();
        return {
            async read(key) {
                return map.get(key);
            },
            async write(key, data) {
                map.set(key, data);
            },
            async delete(key) {
                map.delete(key);
            },
        };
    }

    /**
     * Convert a camel-case name into a dashed name
     * - for example
     *       dashify("BigManStyle")
     *       //> "big-man-style"
     */
    function dashify(camel) {
        return camel.replace(/([a-zA-Z])(?=[A-Z])/g, '$1-').toLowerCase();
    }

    function registerComponents(components) {
        for (const [name, component] of Object.entries(components))
            customElements.define(dashify(name), component);
    }

    const apiOrigin = "https://api.xiome.io";

    function mockPopups({ mockStripeOperations }) {
        return {
            triggerBankPopup: (async ({ stripeAccountId }) => {
                await mockStripeOperations.linkBankWithExistingStripeAccount(stripeAccountId);
            }),
        };
    }

    async function mockRegisterApp({ apiLink, ownerEmail, adminEmail, appOrigins, backend }) {
        const mockBrowserForPlatform = await backend.mockBrowser();
        const mockWindowForPlatform = await mockBrowserForPlatform.mockAppWindow({
            apiLink,
            appId: backend.platformAppId,
            windowLink: window.location.href,
        });
        const { accessModel, appsModel } = mockWindowForPlatform.models;
        await accessModel.sendLoginLink(ownerEmail);
        await accessModel.login(backend.emails.recallLatestLoginEmail().loginToken);
        const { appId } = await appsModel.registerApp({
            label: "Mock App",
            home: window.location.href,
            origins: appOrigins,
        });
        console.log("mock: register app", appId);
        await appsModel.appEditService.assignAdmin({
            appId,
            email: adminEmail,
        });
        // TODO reactivate store
        // // link bank account with stripe
        // await mockWindowForPlatform.models.storeModel.shares.bank.setupStripeAccount(appId)
        // const stripeDetails = await mockWindowForPlatform.models.bankModel.getStripeAccountDetails(appId)
        // console.log("mock: app stripe details", stripeDetails)
        return appId;
    }

    function objectMap$1(input, mapper) {
        const output = {};
        for (const [key, value] of Object.entries(input))
            output[key] = mapper(value, key);
        return output;
    }

    function isObject(value) {
        return value !== undefined
            && value !== null
            && typeof value === "object";
    }

    const _meta = Symbol();

    function isShapeContext(shapeContext) {
        return isObject(shapeContext) && !!shapeContext[_meta];
    }

    function generateRemote({ link, shape, specpath = [], requester, }) {
        return objectMap$1(shape, (value, key) => {
            if (isShapeContext(value)) {
                const getMeta = value[_meta];
                function recurseOverContext(shapeContext, subpath = []) {
                    return objectMap$1(shapeContext, (value2, key2) => {
                        if (value2 === true) {
                            return async (...args) => {
                                const specifier = [...specpath, ...subpath, key, key2].join(".");
                                const meta = await getMeta();
                                const result = await requester({
                                    link,
                                    args,
                                    meta,
                                    specifier,
                                });
                                return result;
                            };
                        }
                        else if (isObject(value2)) {
                            return recurseOverContext(value2, [...subpath, key2]);
                        }
                    });
                }
                return recurseOverContext(value);
            }
            else if (isObject(value)) {
                return generateRemote({
                    link,
                    shape: value,
                    specpath: [...specpath, key],
                    requester,
                });
            }
        });
    }

    const contentTypeText = "text/plain; charset=utf-8";

    function jsonHttpRequest({ link, meta, args, specifier, requestOrigin, headers: moreHeaders, }) {
        return {
            method: "post",
            path: new URL(link).pathname,
            headers: {
                // json requests are sent as plain text
                // in order to avoid any cors 'options' preflight requests
                // by qualifying as a cors "simple request"
                "content-type": contentTypeText,
                ...requestOrigin
                    ? { "origin": requestOrigin }
                    : {},
                ...moreHeaders,
            },
            body: JSON.stringify({
                jsonrpc: "2.0",
                method: specifier,
                params: [meta, ...args],
            }),
        };
    }

    function parseJsonResponse(response) {
        const json = JSON.parse(response.body);
        if (response.status !== 200 || json.error) {
            const message = json.error?.message;
            throw new Error(message
                ? `${response.status}: ${message}`
                : `unknown error`);
        }
        else {
            return json.result;
        }
    }

    function loopbackJsonRemote({ link, shape, headers, servelet }) {
        return generateRemote({
            link,
            shape,
            requester: async ({ args, link, meta, specifier }) => {
                const request = jsonHttpRequest({
                    link,
                    args,
                    meta,
                    headers,
                    specifier,
                });
                const response = await servelet(request);
                return parseJsonResponse(response);
            },
        });
    }

    function noServeletLogger() {
        return {
            logRequest() { },
            logApiError() { },
            logUnexpectedError() { },
        };
    }

    function obtain(specifier, object) {
        return specifier.split(".").reduce((x, y) => x && x[y] || undefined, object);
    }

    class ApiError extends Error {
        code;
        constructor(code, message) {
            super(message);
            this.code = code;
        }
    }

    function isApiError(error) {
        return !!error && error instanceof ApiError;
    }

    function stopwatch() {
        const start = Date.now();
        return function stop() {
            const elapsed = Date.now() - start;
            return elapsed;
        };
    }

    function serveletLoggerCore(logger, detailed) {
        return {
            logRequest({ specifier, meta, args, times, result }) {
                if (detailed)
                    logger.log(`🛎️ ${specifier}()`, `\n 👥`, meta, `\n 📜`, args, `\n ⏱️ ${times.total}ms (${times.auth}ms + ${times.procedure}ms)` +
                        `\n 🎁`, result);
                else
                    logger.log(`🛎️ ${specifier}() ${times.total}ms (${times.auth}ms + ${times.procedure}ms)`);
            },
            logApiError(error) {
                logger.warn(`⚠️ ${error.code} ApiError: ${error.message}`);
            },
            logUnexpectedError(error) {
                logger.error(`🚨 unexpected error!\n`, error);
            },
        };
    }

    function timestampedLogger(logger) {
        function timestamp(time) {
            const date = new Date(time);
            const year = date.getUTCFullYear().toString().padStart(4, "0");
            const month = (date.getUTCMonth() + 1).toString().padStart(2, "0");
            const day = date.getUTCDate().toString().padStart(2, "0");
            const calendar = `${year}-${month}-${day}`;
            const hour = date.getUTCHours().toString().padStart(2, "0");
            const minute = date.getUTCMinutes().toString().padStart(2, "0");
            const second = date.getUTCSeconds().toString().padStart(2, "0");
            const milliseconds = date.getUTCMilliseconds().toString().padStart(3, "0");
            const clock = `${hour}:${minute}:${second}.${milliseconds}`;
            return `[${calendar}::${clock}]`;
        }
        function stamp(log) {
            return (...data) => log(timestamp(Date.now()), ...data);
        }
        return {
            log: stamp(logger.log),
            warn: stamp(logger.warn),
            error: stamp(logger.error),
        };
    }

    function doubleSpaceLogger(logger) {
        function double(log) {
            return (...data) => {
                if (data.length > 0 && typeof data[0] === "string") {
                    const [first, ...rest] = data;
                    log("\n" + first, ...rest);
                }
                else {
                    log("\n", ...data);
                }
            };
        }
        return {
            log: double(logger.log),
            warn: double(logger.warn),
            error: double(logger.error),
        };
    }

    function serveletLoggerPlain(logger, detailed = false) {
        const maybeDoubleSpaced = detailed
            ? doubleSpaceLogger(logger)
            : logger;
        return serveletLoggerCore(timestampedLogger(maybeDoubleSpaced), detailed);
    }

    function makeServelet({ expose, responder, serveletLogger = serveletLoggerPlain(console), parseRequest, }) {
        return async function execute(request) {
            let errorRequestId;
            try {
                const totalClock = stopwatch();
                const { requestId, specifier, meta, args } = await parseRequest(request);
                errorRequestId = requestId;
                const { func, policy } = obtain(specifier, expose);
                const times = {
                    auth: -1,
                    procedure: -1,
                    total: -1,
                };
                const authClock = stopwatch();
                const auth = await policy(meta, request);
                times.auth = authClock();
                const procedureClock = stopwatch();
                const result = await func(auth, ...args);
                times.procedure = procedureClock();
                const response = responder.resultResponse(requestId, result);
                times.total = totalClock();
                serveletLogger.logRequest({ specifier, meta, args, times, result });
                return response;
            }
            catch (error) {
                if (isApiError(error)) {
                    serveletLogger.logApiError(error);
                    return responder.errorResponse(errorRequestId, error);
                }
                else {
                    serveletLogger.logUnexpectedError(error);
                    return responder.errorResponse(errorRequestId, new ApiError(500, "error"));
                }
            }
        };
    }

    async function parseJsonRequest(request) {
        const { id, method, params } = JSON.parse(request.body);
        const specifier = method;
        const [meta, ...args] = params;
        return {
            requestId: id,
            specifier,
            meta,
            args,
        };
    }

    const contentTypeJson = "application/json; charset=utf-8";

    const jsonrpc = "2.0";
    const errorCode = -32000;
    const makeJsonHttpResponder = ({ headers }) => ({
        resultResponse: (requestId, result) => ({
            status: 200,
            headers: {
                "Content-Type": contentTypeJson,
                ...headers,
            },
            body: JSON.stringify({
                jsonrpc,
                id: requestId,
                result,
            }),
        }),
        errorResponse: (requestId, error) => ({
            status: error.code,
            headers: {
                "Content-Type": contentTypeJson,
                ...headers,
            },
            body: JSON.stringify({
                jsonrpc,
                id: requestId,
                error: {
                    code: errorCode,
                    message: error.message,
                },
            }),
        }),
    });

    function makeJsonHttpServelet(expose, serveletLogger) {
        return makeServelet({
            expose,
            serveletLogger,
            responder: makeJsonHttpResponder({ headers: {} }),
            parseRequest: parseJsonRequest,
        });
    }

    function asShape(shape) {
        return shape;
    }

    /**
     * create a pub/sub context
     */
    function pubsub() {
        let listeners = [];
        return {
            publish: (async (...args) => {
                const operations = listeners.map(listener => listener(...args));
                await Promise.all(operations);
            }),
            subscribe(func) {
                listeners.push(func);
                return () => {
                    listeners = listeners.filter(listener => listener !== func);
                };
            },
            dispose: () => {
                listeners = [];
            },
        };
    }
    /**
     * create a separated group of publish and subscribe functions
     */
    function pubsubs(obj) {
        const publishers = {};
        const subscribers = {};
        for (const [key, original] of Object.entries(obj)) {
            publishers[key] = original.publish;
            subscribers[key] = original.subscribe;
        }
        return { publishers, subscribers };
    }

    const browser = typeof atob === "function";
    function encode(text) {
        return browser
            ? btoa(text)
            : Buffer.from(text).toString("base64");
    }
    function decode(base64) {
        return browser
            ? atob(base64)
            : Buffer.from(base64, "base64").toString("binary");
    }
    const url$1 = Object.freeze({
        encode: (text) => {
            const base64 = encode(text);
            const base64url = addEqualsPadding(base64
                .replace(/=/g, "")
                .replace(/\+/g, "-")
                .replace(/\//g, "_"));
            return base64url;
        },
        decode: (base64url) => {
            const base64 = base64url
                .replace(/-/g, "+")
                .replace(/_/g, "/");
            return decode(base64);
        },
    });
    function addEqualsPadding(base64) {
        return base64 + Array((4 - base64.length % 4) % 4 + 1).join("=");
    }

    const tokenDecode = function decode(token) {
        const [headerPart, dataPart] = token.split(".");
        const header = decodePart(headerPart);
        const data = decodePart(dataPart);
        return { header, data };
    };
    function decodePart(part) {
        return JSON.parse(decodeURIComponent(url$1.decode(part)
            .split("")
            .map(c => "%" + ("00" + c.charCodeAt(0).toString(16)).slice(-2))
            .join("")));
    }

    const expiryRenewalCushion = 60 * 1000;
    function isTokenTimingExpired(exp) {
        const expiry = exp * 1000;
        return Date.now() > (expiry - expiryRenewalCushion);
    }

    function isTokenValid(token) {
        return !!token
            && !isTokenTimingExpired(tokenDecode(token).data.exp);
    }

    const decodeAccessToken = (accessToken) => {
        return tokenDecode(accessToken).data.payload;
    };

    function makeAuthMediator({ appId, storage, greenService, }) {
        const tokenChangeEvent = pubsub();
        const accessEvent = pubsub();
        const key = `auth-tokens-${appId}`;
        const emptyTokens = () => ({ accessToken: undefined, refreshToken: undefined });
        const getTokens = async () => await storage.read(key) ?? emptyTokens();
        const setTokens = async (tokens) => storage.write(key, tokens);
        async function commitTokens(tokens) {
            const access = decodeAccessToken(tokens.accessToken);
            await setTokens(tokens);
            tokenChangeEvent.publish();
            await accessEvent.publish(access);
            return access;
        }
        async function authorize(refreshToken) {
            const accessToken = await greenService.authorize({
                appId,
                refreshToken,
                scope: { core: true },
            });
            const access = await commitTokens({ accessToken, refreshToken });
            return { access, accessToken };
        }
        const obtainAccessAndReauthorizeIfNecessary = (async () => {
            const { accessToken, refreshToken } = await getTokens();
            return isTokenValid(accessToken)
                ? { accessToken, access: decodeAccessToken(accessToken) }
                : isTokenValid(refreshToken)
                    ? authorize(refreshToken)
                    : authorize(undefined);
        });
        return {
            subscribeToAccessChange: accessEvent.subscribe,
            subscribeToTokenChange: tokenChangeEvent.subscribe,
            async initialize() {
                const { access } = await obtainAccessAndReauthorizeIfNecessary();
                await accessEvent.publish(access);
                return access;
            },
            async getValidAccess() {
                return (await obtainAccessAndReauthorizeIfNecessary()).access;
            },
            async getValidAccessToken() {
                return (await obtainAccessAndReauthorizeIfNecessary()).accessToken;
            },
            async login(tokens) {
                return commitTokens(tokens);
            },
            async logout() {
                await setTokens(emptyTokens());
                return (await obtainAccessAndReauthorizeIfNecessary()).access;
            },
            async reauthorize() {
                const tokens = await getTokens();
                await setTokens({ ...tokens, accessToken: undefined });
                return (await obtainAccessAndReauthorizeIfNecessary()).access;
            },
        };
    }

    function prepareApiShape({ appId, storage }) {
        let authMediator;
        const standardMeta = async () => ({
            accessToken: await authMediator.getValidAccessToken(),
        });
        const shape = asShape({
            auth: {
                apps: {
                    appService: {
                        [_meta]: standardMeta,
                        listApps: true,
                        registerApp: true,
                    },
                    appEditService: {
                        [_meta]: standardMeta,
                        deleteApp: true,
                        updateApp: true,
                        listAdmins: true,
                        revokeAdmin: true,
                        assignAdmin: true,
                        assignPlatformUserAsAdmin: true,
                    },
                },
                permissions: {
                    permissionsService: {
                        [_meta]: standardMeta,
                        assignPrivilege: true,
                        createRole: true,
                        deleteRole: true,
                        fetchPermissions: true,
                        unassignPrivilege: true,
                        createPrivilege: true,
                        deletePrivilege: true,
                    },
                },
                users: {
                    greenService: {
                        [_meta]: async () => undefined,
                        authorize: true,
                    },
                    loginService: {
                        [_meta]: standardMeta,
                        authenticateViaLoginToken: true,
                        sendLoginLink: true,
                    },
                    personalService: {
                        [_meta]: standardMeta,
                        setProfile: true,
                    },
                    userService: {
                        [_meta]: standardMeta,
                        getUser: true,
                    },
                },
            },
            example: {
                exampleService: {
                    [_meta]: standardMeta,
                    exampleFunction: true,
                },
            },
            administrative: {
                roleAssignmentService: {
                    [_meta]: standardMeta,
                    assignRoleToUser: true,
                    fetchPermissions: true,
                    revokeRoleFromUser: true,
                    searchUsers: true,
                },
            },
            questions: {
                questionsModerationService: {
                    [_meta]: standardMeta,
                    archiveBoard: true,
                    fetchReportedQuestions: true,
                },
                questionsPostingService: {
                    [_meta]: standardMeta,
                    likeQuestion: true,
                    postQuestion: true,
                    reportQuestion: true,
                    archiveQuestion: true,
                },
                questionsReadingService: {
                    [_meta]: standardMeta,
                    fetchQuestions: true,
                },
                questionsAnsweringService: {
                    [_meta]: standardMeta,
                    archiveAnswer: true,
                    likeAnswer: true,
                    postAnswer: true,
                    reportAnswer: true,
                },
            },
            livestream: {
                livestreamViewingService: {
                    [_meta]: standardMeta,
                    getShows: true,
                },
                livestreamModerationService: {
                    [_meta]: standardMeta,
                    setShow: true,
                },
            },
            videos: {
                dacastService: {
                    [_meta]: standardMeta,
                    clearLink: true,
                    getLink: true,
                    setLink: true,
                },
                contentService: {
                    [_meta]: standardMeta,
                    fetchModerationData: true,
                    deleteView: true,
                    getShows: true,
                    writeView: true,
                },
            },
        });
        function installAuthMediator({ greenService }) {
            authMediator = makeAuthMediator({
                appId,
                storage,
                greenService,
            });
            return authMediator;
        }
        return { shape, installAuthMediator };
    }

    function mockRemote({ api, appId, origin, apiLink, storage, }) {
        const { shape, installAuthMediator } = prepareApiShape({
            appId,
            storage,
        });
        const remote = loopbackJsonRemote({
            shape,
            link: apiLink,
            headers: { origin },
            servelet: makeJsonHttpServelet(api, noServeletLogger()),
        });
        // const remote = logAllCalls<typeof api>({
        // 	// logger: new DisabledLogger(),
        // 	logger: console,
        // 	fullyDebug: false,
        // 	remote: addMockLatency({
        // 		latency,
        // 		remote: loopbackJsonRemote<typeof api>({
        // 			shape,
        // 			link: apiLink,
        // 			headers: {origin},
        // 			servelet: makeJsonHttpServelet(api, noServeletLogger()),
        // 		}),
        // 	}),
        // })
        const authMediator = installAuthMediator({
            greenService: remote.auth.users.greenService,
        });
        return { remote, authMediator };
    }

    (function(global) {
        var channels = [];

        function BroadcastChannel(channel) {
            var $this = this;
            channel = String(channel);

            var id = '$BroadcastChannel$' + channel + '$';

            channels[id] = channels[id] || [];
            channels[id].push(this);

            this._name = channel;
            this._id = id;
            this._closed = false;
            this._mc = new MessageChannel();
            this._mc.port1.start();
            this._mc.port2.start();

            global.addEventListener('storage', function(e) {
                if (e.storageArea !== global.localStorage) return;
                if (e.newValue == null || e.newValue === '') return;
                if (e.key.substring(0, id.length) !== id) return;
                var data = JSON.parse(e.newValue);
                $this._mc.port2.postMessage(data);
            });
        }

        BroadcastChannel.prototype = {
            // BroadcastChannel API
            get name() {
                return this._name;
            },
            postMessage: function(message) {
                var $this = this;
                if (this._closed) {
                    var e = new Error();
                    e.name = 'InvalidStateError';
                    throw e;
                }
                var value = JSON.stringify(message);

                // Broadcast to other contexts via storage events...
                var key = this._id + String(Date.now()) + '$' + String(Math.random());
                global.localStorage.setItem(key, value);
                setTimeout(function() {
                    global.localStorage.removeItem(key);
                }, 500);

                // Broadcast to current context via ports
                channels[this._id].forEach(function(bc) {
                    if (bc === $this) return;
                    bc._mc.port2.postMessage(JSON.parse(value));
                });
            },
            close: function() {
                if (this._closed) return;
                this._closed = true;
                this._mc.port1.close();
                this._mc.port2.close();

                var index = channels[this._id].indexOf(this);
                channels[this._id].splice(index, 1);
            },

            // EventTarget API
            get onmessage() {
                return this._mc.port1.onmessage;
            },
            set onmessage(value) {
                this._mc.port1.onmessage = value;
            },
            addEventListener: function(/*type, listener , useCapture*/) {
                return this._mc.port1.addEventListener.apply(this._mc.port1, arguments);
            },
            removeEventListener: function(/*type, listener , useCapture*/) {
                return this._mc.port1.removeEventListener.apply(this._mc.port1, arguments);
            },
            dispatchEvent: function(/*event*/) {
                return this._mc.port1.dispatchEvent.apply(this._mc.port1, arguments);
            },
        };

        global.BroadcastChannel = global.BroadcastChannel || BroadcastChannel;
    })(self);

    function wireMediatorBroadcastChannel({ appId, authMediator }) {
        const channel = new BroadcastChannel("tokenChangeEvent");
        authMediator.subscribeToTokenChange(() => channel.postMessage({ appId }));
        channel.onmessage = (event) => {
            if (event.data.appId === appId)
                authMediator.initialize();
        };
    }

    async function mockWiredRemote({ apiLink, appId, storage, appWindowLink, backend, }) {
        const { remote, authMediator } = mockRemote({
            appId,
            apiLink,
            storage,
            api: backend.api,
            origin: new URL(appWindowLink).origin,
        });
        wireMediatorBroadcastChannel({ appId, authMediator });
        return { remote, authMediator };
    }

    const second = 1000;
    const minute = 60 * second;
    const hour = 60 * minute;
    const day = 24 * hour;
    const year = 365 * day;
    const month = year / 12;

    const mockConfig = ({ platformHome, platformOrigins }) => ({
        server: {
            port: 4999,
            detailedLogs: true,
        },
        platform: {
            legalLink: "https://xiome.io/legal",
            technician: {
                email: "chasemoskal@gmail.com",
            },
            appDetails: {
                appId: "27a9f4e61286d562c85df1862e88c50e6b5c0707cc84e3e1df4aeb9d9c236814",
                label: "Xiome Cloud",
                home: platformHome,
                origins: platformOrigins,
            },
        },
        dacast: "mock-mode",
        email: "mock-console",
        database: "mock-storage",
        stripe: "mock-mode",
        crypto: {
            keys: "mock-mode",
            tokenLifespans: {
                login: 5 * minute,
                refresh: 30 * day,
                access: 5 * minute,
                external: 5 * minute,
            },
        },
    });

    const isNode = (typeof process !== "undefined")
        && (process?.release?.name === "node");

    function encodeHex(binary) {
        let result = "";
        for (const byte of new Uint8Array(binary))
            result += ("0" + (byte & 0xFF).toString(16)).slice(-2);
        return result;
    }
    const hexabet = "0123456789abcdef";
    function isStrictHex(text) {
        for (const character of text)
            if (!hexabet.includes(character))
                return false;
        return true;
    }
    function isHex(text) {
        return isStrictHex(text.toLowerCase());
    }
    function decodeHex(text) {
        const hex = text.toLowerCase();
        if (!isStrictHex(hex))
            throw new Error("decodeHex: invalid hex");
        const result = [];
        for (let i = 0; i < hex.length; i += 2)
            result.push(parseInt(hex.substr(i, 2), 16));
        return new Uint8Array(result).buffer;
    }

    var __classPrivateFieldSet$9 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var __classPrivateFieldGet$b = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var _DamnId_binary, _DamnId_string;
    class DamnId {
        constructor(binary) {
            _DamnId_binary.set(this, void 0);
            _DamnId_string.set(this, void 0);
            __classPrivateFieldSet$9(this, _DamnId_binary, binary, "f");
            __classPrivateFieldSet$9(this, _DamnId_string, encodeHex(binary), "f");
        }
        static fromBinary(binary) {
            return new DamnId(binary);
        }
        static fromString(text) {
            const binary = decodeHex(text);
            return new DamnId(binary);
        }
        static isId(text) {
            return text.length === 64 && isHex(text);
        }
        get binary() { return __classPrivateFieldGet$b(this, _DamnId_binary, "f"); }
        get string() { return __classPrivateFieldGet$b(this, _DamnId_string, "f"); }
        toBinary() { return __classPrivateFieldGet$b(this, _DamnId_binary, "f"); }
        toString() { return __classPrivateFieldGet$b(this, _DamnId_string, "f"); }
    }
    _DamnId_binary = new WeakMap(), _DamnId_string = new WeakMap();

    async function getRando({ isNode: isNode$1 = isNode } = {}) {
        const randomBuffer = isNode$1
            ? await (async () => {
                const crypto = await import('crypto');
                return (bytes) => crypto.randomBytes(bytes).buffer;
            })()
            : (bytes) => crypto.getRandomValues(new Uint8Array(bytes)).buffer;
        const compare = isNode$1
            ? await (async () => {
                const crypto = await import('crypto');
                return (a, b) => crypto.timingSafeEqual(Buffer.from(a, "utf8"), Buffer.from(b, "utf8"));
            })()
            : (a, b) => {
                console.warn("insecure compare");
                return a === b;
            };
        function random() {
            const buffer = randomBuffer(8);
            const ints = new Int8Array(buffer);
            ints[7] = 63;
            ints[6] |= 0xf0;
            const view = new DataView(buffer);
            return view.getFloat64(0, true) - 1;
        }
        // function randomId_old() {
        // 	const buffer = randomBuffer(32)
        // 	return identifierFromBinary(buffer)
        // }
        function randomId() {
            const buffer = randomBuffer(32);
            return new DamnId(buffer);
        }
        function randomSample(palette) {
            return palette[Math.floor(random() * palette.length)];
        }
        function randomSequence(length, palette) {
            const results = [];
            while (results.length < length) {
                const sample = randomSample(palette);
                results.push(sample);
            }
            return results.join("");
        }
        return {
            random,
            compare,
            randomId,
            randomBuffer,
            randomSample,
            randomSequence,
        };
    }

    function subbies() {
        const memory = new Map();
        return {
            subscribe(listener) {
                const symbol = Symbol();
                memory.set(symbol, listener);
                return () => memory.delete(symbol);
            },
            publish: (context => {
                for (const [, listener] of memory)
                    listener(context);
            }),
            clear() {
                memory.clear();
            },
        };
    }

    function debounce(delay, action) {
        let latestArgs;
        let timeout;
        let promise;
        let resolve;
        let reject;
        function reset() {
            latestArgs = undefined;
            if (timeout)
                clearTimeout(timeout);
            timeout = undefined;
            promise = new Promise((res, rej) => {
                resolve = res;
                reject = rej;
            });
        }
        reset();
        return ((...args) => {
            latestArgs = args;
            if (timeout)
                clearTimeout(timeout);
            timeout = setTimeout(() => {
                Promise.resolve(action(...latestArgs))
                    .then(r => resolve(r))
                    .catch(err => reject(err))
                    .finally(() => reset());
            }, delay);
            return promise;
        });
    }

    const debounceDelay = 1;

    class SnapstateError extends Error {
        constructor() {
            super(...arguments);
            this.name = this.constructor.name;
        }
    }
    class SnapstateReadonlyError extends SnapstateError {
    }
    class SnapstateCircularError extends SnapstateError {
    }

    function trackingMechanics() {
        let activeWatch;
        const sessions = new Map();
        function unsubscribe(identifier) {
            for (const [, record] of sessions)
                record.delete(identifier);
        }
        let waiter = Promise.resolve();
        return {
            get wait() { return waiter; },
            track(state, observer, reaction) {
                const identifier = Symbol();
                activeWatch = { identifier, session: { observer, reaction } };
                observer(state);
                activeWatch = undefined;
                return () => unsubscribe(identifier);
            },
            avoidCircular(key) {
                if (activeWatch)
                    throw new SnapstateCircularError(`cannot set any properties within a reaction, not even "${key}"`);
            },
            triggerReactions: (() => {
                const reactionsQueue = new Set();
                const fireReactionsQueue = debounce(debounceDelay, (state) => {
                    for (const key of reactionsQueue) {
                        for (const [, { observer, reaction }] of sessions.get(key) ?? []) {
                            if (reaction)
                                reaction(observer(state));
                            else
                                observer(state);
                        }
                    }
                    reactionsQueue.clear();
                });
                return (state, key) => {
                    reactionsQueue.add(key);
                    const promise = fireReactionsQueue(state);
                    waiter = waiter.then(() => promise);
                };
            })(),
            reactionRegistration(key) {
                if (activeWatch) {
                    let record = sessions.get(key);
                    if (!record) {
                        record = new Map();
                        sessions.set(key, record);
                    }
                    record.set(activeWatch.identifier, activeWatch.session);
                }
            },
        };
    }

    function snapstate(actual) {
        const tracking = trackingMechanics();
        function get(t, key) {
            tracking.reactionRegistration(key);
            return actual[key];
        }
        const readable = new Proxy(actual, {
            get,
            set(t, key) {
                throw new SnapstateReadonlyError(`readonly state property "${key}"`);
            },
        });
        const { publish: rawPublish, subscribe } = subbies();
        const publishReadable = debounce(debounceDelay, () => rawPublish(readable));
        let waiter = Promise.resolve();
        const writable = new Proxy(actual, {
            get,
            set(t, key, value) {
                tracking.avoidCircular(key);
                actual[key] = value;
                tracking.triggerReactions(readable, key);
                waiter = publishReadable();
                return true;
            },
        });
        return {
            readable,
            writable,
            subscribe,
            track(observer, reaction) {
                return tracking.track(readable, observer, reaction);
            },
            async wait() {
                await Promise.all([waiter, tracking.wait]);
            },
        };
    }

    function makeExampleModel({ getAccessOp }) {
        const state = snapstate({
            accessOp: getAccessOp(),
        });
        return {
            state: state.readable,
            subscribe: state.subscribe,
            updateAccessOp: (op) => {
                state.writable.accessOp = op;
            },
        };
    }

    var Ops;
    (function (Ops) {
        (function (Mode) {
            Mode[Mode["None"] = 0] = "None";
            Mode[Mode["Loading"] = 1] = "Loading";
            Mode[Mode["Error"] = 2] = "Error";
            Mode[Mode["Ready"] = 3] = "Ready";
        })(Ops.Mode || (Ops.Mode = {}));
    })(Ops || (Ops = {}));
    const ops = {
        none: () => ({
            mode: Ops.Mode.None,
        }),
        loading: () => ({
            mode: Ops.Mode.Loading,
        }),
        error: (reason) => ({
            mode: Ops.Mode.Error,
            reason,
        }),
        ready: (value) => ({
            mode: Ops.Mode.Ready,
            value,
        }),
        replaceValue(op, value) {
            return op.mode === Ops.Mode.Ready
                ? { ...op, value }
                : op;
        },
        isNone: (op) => op.mode === Ops.Mode.Ready,
        isLoading: (op) => op.mode === Ops.Mode.Loading,
        isError: (op) => op.mode === Ops.Mode.Error,
        isReady: (op) => op.mode === Ops.Mode.Ready,
        value(op) {
            return op.mode === Ops.Mode.Ready
                ? op.value
                : undefined;
        },
        select(op, options) {
            switch (op.mode) {
                case Ops.Mode.None: return options.none();
                case Ops.Mode.Loading: return options.loading();
                case Ops.Mode.Error: return options.error(op.reason);
                case Ops.Mode.Ready: return options.ready(op.value);
            }
        },
        async operation({ promise, errorReason = "an error occurred", setOp, }) {
            setOp(ops.loading());
            try {
                const value = await promise;
                setOp(ops.ready(value));
                return value;
            }
            catch (error) {
                setOp(ops.error(errorReason));
                throw error;
            }
        },
        mode(op) {
            return ops.select(op, {
                none: () => "none",
                loading: () => "loading",
                error: () => "error",
                ready: () => "ready",
            });
        },
        combine(...ops) {
            const isAnyError = !!ops.find(op => op.mode === Ops.Mode.Error);
            const isAnyLoading = !!ops.find(op => op.mode === Ops.Mode.Loading);
            const isAnyNotReady = !!ops.find(op => op.mode !== Ops.Mode.Ready);
            return isAnyError
                ? { mode: Ops.Mode.Error, reason: "error" }
                : isAnyLoading
                    ? { mode: Ops.Mode.Loading }
                    : isAnyNotReady
                        ? { mode: Ops.Mode.None }
                        : { mode: Ops.Mode.Ready, value: undefined };
        },
        debug(op) {
            return ops.select(op, {
                none: () => [`<op {mode: None}>`],
                loading: () => [`<op {mode: Loading}>`],
                error: reason => [`<op {mode: Error, reason: "${reason}"}>`],
                ready: value => [`<op {mode: Ready, value: `, value, `}>`],
            });
        },
    };

    function makeAppsModel({ appService, appEditService, getValidAccess, }) {
        const { readable, writable, subscribe } = snapstate({
            active: false,
            appRecords: ops.none(),
            addingNewApp: ops.none(),
        });
        const actions = {
            setAppRecords(op) {
                const appList = ops.value(op) ?? [];
                let records = {};
                for (const app of appList)
                    records[app.appId] = ops.ready(app);
                writable.appRecords = ops.replaceValue(op, records);
            },
            setIndividualAppRecord(appId, op) {
                if (ops.isLoading(writable.appRecords))
                    throw new Error("cannot set individual app while apps are loading");
                const apps = ops.value(writable.appRecords) ?? {};
                writable.appRecords = ops.ready({ ...apps, [appId]: op });
            },
            deleteIndividualAppRecord(appId) {
                if (ops.isReady(writable.appRecords)) {
                    const existingRecords = ops.value(writable.appRecords);
                    const records = {};
                    for (const [key, value] of Object.entries(existingRecords)) {
                        if (key !== appId)
                            records[key] = value;
                    }
                    writable.appRecords = ops.ready(records);
                }
                else
                    throw new Error("cannot delete individual app while apps are loading");
            },
        };
        async function getUserId() {
            const access = await getValidAccess();
            return access?.user?.userId;
        }
        async function loadApps() {
            writable.active = true;
            const appsPromise = ops.operation({
                promise: (async () => {
                    const userId = await getUserId();
                    return userId
                        ? appService.listApps({ ownerUserId: userId })
                        : [];
                })(),
                setOp: op => actions.setAppRecords(op),
            });
            // actions.setLoadingPromise(appsPromise.then(() => undefined))
            return appsPromise;
        }
        async function registerApp(appDraft) {
            const userId = await getUserId();
            const result = await ops.operation({
                errorReason: "failed to register app",
                promise: (async () => {
                    const result = await appService.registerApp({
                        appDraft,
                        ownerUserId: userId,
                    });
                    await appEditService.assignPlatformUserAsAdmin({
                        appId: result.appId,
                        platformUserId: userId,
                    });
                    return result;
                })(),
                setOp: op => {
                    writable.addingNewApp = ops.replaceValue(op, null);
                    if (ops.isReady(op)) {
                        const newApp = { ...ops.value(op), ...appDraft };
                        actions.setIndividualAppRecord(newApp.appId, ops.ready(newApp));
                    }
                },
            });
            return result;
        }
        async function updateApp(appId, appDraft) {
            const records = ops.value(writable.appRecords);
            if (!records)
                throw new Error("cannot update app while loading records");
            const existingApp = ops.value(records[appId]);
            if (!existingApp)
                throw new Error("cannot update app not present in records");
            return ops.operation({
                promise: appEditService.updateApp({ appId, appDraft }),
                setOp: op => actions.setIndividualAppRecord(appId, ops.replaceValue(op, { ...existingApp, ...appDraft })),
            });
        }
        function getApp(appId) {
            const records = ops.value(writable.appRecords);
            return records
                ? ops.value(records[appId])
                : undefined;
        }
        async function deleteApp(appId) {
            await ops.operation({
                promise: appEditService.deleteApp({ appId }),
                setOp: op => {
                    actions.setIndividualAppRecord(appId, ops.replaceValue(op, getApp(appId)));
                },
            });
            actions.deleteIndividualAppRecord(appId);
        }
        return {
            state: readable,
            subscribe,
            appEditService,
            getApp(appId) {
                const records = ops.value(readable.appRecords);
                return records
                    ? ops.value(records[appId])
                    : undefined;
            },
            loadApps,
            deleteApp,
            updateApp,
            registerApp,
            async updateAccessOp(op) {
                if (readable.active)
                    await loadApps();
            },
        };
    }

    function merge(newItems, oldItems, compare) {
        const result = [...newItems];
        for (const oldItem of oldItems) {
            const included = !!result.find(newItem => compare(newItem, oldItem));
            if (!included)
                result.push(oldItem);
        }
        return result;
    }

    function deepClone(obj) {
        return JSON.parse(JSON.stringify(obj));
    }
    function deepFreeze(object) {
        Object.freeze(object);
        for (const value of Object.values(object)) {
            if (value && typeof value === "object")
                deepFreeze(value);
        }
        return object;
    }
    const isSet = (a) => (a !== null && a !== undefined);
    function deepEqual(a, b) {
        if (!isSet(a) || !isSet(b))
            return a === b;
        for (const [key, aValue] of Object.entries(a)) {
            if (!b.hasOwnProperty(key))
                return false;
            const bValue = b[key];
            switch (typeof aValue) {
                case "object":
                    if (!deepEqual(aValue, bValue))
                        return false;
                    break;
                case "function":
                    if (!isSet(bValue) || aValue.toString() !== bValue.toString())
                        return false;
                    break;
                default:
                    if (aValue !== bValue)
                        return false;
            }
        }
        for (const [key] of Object.entries(a))
            if (!b.hasOwnProperty(key))
                return false;
        for (const [key] of Object.entries(b))
            if (!a.hasOwnProperty(key))
                return false;
        return true;
    }

    function happystate({ state: realState, actions: makeActions, }) {
        let frozenState = deepFreeze(deepClone(realState));
        const actions = makeActions(realState);
        const { publish, subscribe: onStateChange, dispose: clearStateListeners, } = pubsub();
        const change = debounce(1, publish);
        const augmentedActions = {};
        for (const [key, action] of Object.entries(actions)) {
            augmentedActions[key] = (...args) => {
                action(...args);
                frozenState = deepFreeze(deepClone(realState));
                change();
            };
        }
        return {
            actions: augmentedActions,
            onStateChange,
            clearStateListeners,
            getState: () => frozenState,
        };
    }

    function makeQuestionsModelHappy() {
        return happystate({
            state: {
                access: undefined,
                users: [],
                questions: [],
                boardOps: {},
                postingOp: ops.ready(undefined),
            },
            actions: state => ({
                setAccess(access) {
                    state.access = access;
                },
                setBoardOp(board, op) {
                    state.boardOps = { ...state.boardOps, [board]: op };
                },
                setPostingOp(op) {
                    state.postingOp = op;
                },
                addUsers(newUsers) {
                    state.users = [...merge(newUsers.filter(u => !!u), state.users, (a, b) => a.userId === b.userId)];
                },
                addQuestions(newQuestions) {
                    state.questions = [...merge(newQuestions, state.questions, (a, b) => a.questionId === b.questionId)];
                },
                addAnswer(newAnswer) {
                    const question = state.questions
                        .find(q => q.questionId === newAnswer.questionId);
                    if (!question)
                        throw new Error(`can't find question to add answer, question id: "${newAnswer.questionId}"`);
                    const newQuestion = {
                        ...question,
                        answers: [...question.answers, newAnswer],
                    };
                    state.questions = [...merge([newQuestion], state.questions, (a, b) => a.questionId === b.questionId)];
                },
                setQuestionLike(questionId, like) {
                    state.questions = state.questions.map(question => question.questionId === questionId
                        ? {
                            ...question,
                            liked: like,
                            likes: question.liked === like
                                ? question.likes
                                : like
                                    ? question.likes + 1
                                    : question.likes - 1
                        }
                        : { ...question });
                },
                setAnswerLike(questionId, answerId, status) {
                    state.questions = state.questions.map(question => question.questionId === questionId
                        ? {
                            ...question,
                            answers: question.answers.map(answer => answer.answerId === answerId
                                ? {
                                    ...answer,
                                    liked: status,
                                    likes: answer.liked === status
                                        ? answer.likes
                                        : status
                                            ? answer.likes + 1
                                            : answer.likes - 1
                                }
                                : { ...answer })
                        }
                        : { ...question });
                },
                setAnswerReport(questionId, answerId, status) {
                    state.questions = state.questions.map(question => question.questionId === questionId
                        ? {
                            ...question,
                            answers: question.answers.map(answer => answer.answerId === answerId
                                ? {
                                    ...answer,
                                    reported: status,
                                    reports: answer.reported === status
                                        ? answer.reports
                                        : status
                                            ? answer.reports + 1
                                            : answer.reports - 1
                                }
                                : { ...answer })
                        }
                        : { ...question });
                },
                setQuestionReport(questionId, report) {
                    state.questions = state.questions.map(question => question.questionId === questionId
                        ? {
                            ...question,
                            reported: report,
                            reports: question.reported === report
                                ? question.reports
                                : report
                                    ? question.reports + 1
                                    : question.reports - 1
                        }
                        : { ...question });
                },
                setQuestionArchive(questionId, archive) {
                    state.questions = state.questions.map(question => question.questionId === questionId
                        ? { ...question, archive }
                        : { ...question });
                },
                setAnswerArchive(questionId, answerId, archive) {
                    state.questions = state.questions.map(question => question.questionId === questionId
                        ? {
                            ...question,
                            answers: question.answers.map(answer => answer.answerId === answerId
                                ? { ...answer, archive }
                                : { ...answer })
                        }
                        : { ...question });
                },
            }),
        });
    }

    const videoPowerPrivileges = {
        "view all videos": "22f380eb0c5c7463b8a16fcdfc2f5e58257d38f94972924e70d27a08fabb5506",
        "moderate videos": "1b19013344a59cb49bfd9bddc8dd817a9e4cfa29003437021df24bdc832ec246",
    };
    const videoPrivileges = { ...videoPowerPrivileges };

    function asPermissions(permissions) {
        return permissions;
    }
    function has(privileges, hasPrivilege) {
        const hasPrivileges = {};
        for (const key of Object.keys(privileges))
            hasPrivileges[key] = hasPrivilege;
        return hasPrivileges;
    }
    function mutable(active, privileges) {
        return has(privileges, { active, immutable: false });
    }
    function immutable(active, privileges) {
        return has(privileges, { active, immutable: true });
    }

    const universalPrivilege = "c1cb5ac1000af6081ff47e80804de7542b57a1425c3faac8153e708aaf79271a";
    const commonPrivileges = {
        "read questions": "5b19b929648cbbe26c052a5016892b370f048eead1e6a303a50b7f5234ec5161",
        "post questions": "aeaae6139d05f77193ca7c272a65977c70cb188f3070359866237a5ade84ba15",
        "like questions": "5490898c541d733956b28664e7536673842e73b289fc45ab9caf143138c14a3e",
        "report questions": "f2f89d433f398b267e320e0e3b9929eb3934af5c02f7e3fb2e032fd19399ad21",
        "view livestream": "be713689a187ecf01a8faa7a1d64f9b1a1a3726701387c1c83a7aaec815ec412",
    };
    const commonPowerPrivileges = {
        "edit any profile": "4240ca0331e8679a444c6c4c6a803d8a12893f1554460af426064b980164c8ee",
        "customize permissions": "f380d9b4d1001da8dc86a6e0b6a0721b92ed8060c52cd4ba433e76cbd9efe6b5",
        "answer questions": "afaa00f7bc9ab09d3bbc61a26f6d9bc666cd9bb7ac270ebbdeba3cf4dd3ab9a9",
        "moderate questions": "f6a3152a13f313a60f84d82ba8f7765afb19f70c04ef14688d802237a5d2ab5e",
        "view stats": "092866ae6378833e43ec7001188aa1829d4cf2858e42da0ad3f44a2fedbcb07b",
        "administrate user roles": "a072e9dd003a762e7d693658d8ac724be352b18da240843f691595c89020e350",
        "moderate livestream": "a59d6ce7dd46e0bd7710d11cbc609cf5849e32deb13753eec8150e6fb40a43e9",
        ...videoPowerPrivileges,
    };
    const platformPowerPrivileges = {
        "edit any app": "e5fadb4b03badd6699b355ec43e1d150d6f4df81ecc291b5b742f06ebe62a0f8",
        "view platform stats": "eec7d2ddc20df6080c4166a3dea0b2643b483ce61326cbefad95e45824810692",
    };
    const appPowerPrivileges = {
        "manage store": "c0ffc302f2a431a02fef22592ff2b4321944dfbfe29e7f6f83e13b6332e54b46",
        "give away freebies": "38a7f04980f1de72b975cb5ae4be7b850aac2ca96b545ba6894daaa8b5d7c9cf",
    };
    const active = true;
    const INACTIVE = false;
    const universalPermissions = asPermissions({
        privileges: {
            ...commonPrivileges,
            ...commonPowerPrivileges,
            "universal": universalPrivilege,
            "banned": "ffff9ebcbe8f69f3520ec574f5a1489a70bf04521d7a85fcbd25c54cc22802f1",
        },
        roles: {
            "everybody": {
                roleId: "aaaa777dc7d7e9b5e9e0386abf0cf2f059cda4f71779cc910b85e8f504452b23",
                public: true,
                assignable: false,
                hasPrivileges: {
                    ...immutable(INACTIVE, commonPrivileges),
                    ...immutable(INACTIVE, commonPowerPrivileges),
                    "universal": { active: true, immutable: true },
                    "banned": { active: false, immutable: true },
                    "read questions": { active: true, immutable: false },
                },
            },
            "authenticated": {
                roleId: "bbbb642d549d06113953a01fb52a8e010150edc502b929395bfb32d250d44b1c",
                public: false,
                assignable: false,
                hasPrivileges: {
                    ...mutable(active, commonPrivileges),
                    ...immutable(INACTIVE, commonPowerPrivileges),
                    "universal": { active: false, immutable: true },
                    "banned": { active: false, immutable: true },
                },
            },
            "banned": {
                roleId: "dead633608412f8eefbb40f4678ef2bafaeb7883d09a6094dafe5f69dd6a8684",
                public: true,
                assignable: true,
                hasPrivileges: {
                    ...immutable(INACTIVE, commonPrivileges),
                    ...immutable(INACTIVE, commonPowerPrivileges),
                    "universal": { active: false, immutable: true },
                    "banned": { active: true, immutable: true },
                },
            },
            "technician": {
                roleId: "9999d97119a83ea3853273a93e24ae0a1f2a994eccb21dacacab6ff860d8d869",
                public: true,
                assignable: false,
                hasPrivileges: {
                    ...immutable(active, commonPrivileges),
                    ...immutable(active, commonPowerPrivileges),
                    "universal": { active: false, immutable: true },
                    "banned": { active: false, immutable: true },
                },
            },
        },
    });
    const platformPermissions = asPermissions({
        privileges: {
            ...universalPermissions.privileges,
            ...platformPowerPrivileges,
        },
        roles: {
            ...universalPermissions.roles,
            "everybody": {
                ...universalPermissions.roles.everybody,
                hasPrivileges: {
                    ...universalPermissions.roles.everybody.hasPrivileges,
                    ...immutable(INACTIVE, platformPowerPrivileges),
                }
            },
            "authenticated": {
                ...universalPermissions.roles.authenticated,
                hasPrivileges: {
                    ...universalPermissions.roles.authenticated.hasPrivileges,
                    ...immutable(INACTIVE, platformPowerPrivileges),
                }
            },
            "banned": {
                ...universalPermissions.roles.banned,
                hasPrivileges: {
                    ...universalPermissions.roles.banned.hasPrivileges,
                    ...immutable(INACTIVE, platformPowerPrivileges),
                }
            },
            "technician": {
                ...universalPermissions.roles.technician,
                hasPrivileges: {
                    ...universalPermissions.roles.technician.hasPrivileges,
                    ...immutable(active, commonPrivileges),
                    ...immutable(active, platformPowerPrivileges),
                },
            },
        },
    });
    const appPermissions = asPermissions({
        privileges: {
            ...universalPermissions.privileges,
            ...appPowerPrivileges,
        },
        roles: {
            ...universalPermissions.roles,
            "everybody": {
                ...universalPermissions.roles.everybody,
                hasPrivileges: {
                    ...universalPermissions.roles.everybody.hasPrivileges,
                    ...immutable(INACTIVE, appPowerPrivileges),
                }
            },
            "authenticated": {
                ...universalPermissions.roles.authenticated,
                hasPrivileges: {
                    ...universalPermissions.roles.authenticated.hasPrivileges,
                    ...immutable(INACTIVE, appPowerPrivileges),
                }
            },
            "banned": {
                ...universalPermissions.roles.banned,
                hasPrivileges: {
                    ...universalPermissions.roles.banned.hasPrivileges,
                    ...immutable(INACTIVE, appPowerPrivileges),
                }
            },
            "admin": {
                roleId: "d21e66058b95b3b2e80c1a00c82abd56b8ec7a4b20f0424c2a00cc6ba352efeb",
                public: true,
                assignable: false,
                hasPrivileges: {
                    ...immutable(active, appPowerPrivileges),
                    ...immutable(active, commonPrivileges),
                    ...immutable(active, commonPowerPrivileges),
                    "universal": { active: false, immutable: true },
                    "banned": { active: false, immutable: true },
                },
            },
            "technician": {
                ...universalPermissions.roles.technician,
                hasPrivileges: {
                    ...universalPermissions.roles.technician.hasPrivileges,
                    ...immutable(active, appPowerPrivileges),
                },
            },
        },
    });

    function prepareQuestionsBoardModelGetter({ happy, questionsPostingService, questionsAnsweringService, questionsModerationService, getAccess, loadQuestionsForBoard, }) {
        const { actions, getState } = happy;
        return (board) => ({
            getPermissions() {
                const { access } = getState();
                return {
                    "read questions": access
                        ? access.permit.privileges.includes(appPermissions.privileges["read questions"])
                        : false,
                    "post questions": access
                        ? access.permit.privileges.includes(appPermissions.privileges["post questions"]) && !access.permit.privileges.includes(appPermissions.privileges["banned"])
                        : false,
                    "moderate questions": access
                        ? access.permit.privileges.includes(appPermissions.privileges["moderate questions"])
                        : false,
                    "answer questions": access
                        ? access.permit.privileges.includes(appPermissions.privileges["answer questions"])
                        : false,
                    "like questions": access
                        ? access.permit.privileges.includes(appPermissions.privileges["like questions"])
                        : false,
                    "report questions": access
                        ? access.permit.privileges.includes(appPermissions.privileges["report questions"])
                        : false,
                };
            },
            getBoardName() {
                return board;
            },
            getAccess() {
                return getState().access;
            },
            getBoardOp() {
                return getState().boardOps[board];
            },
            getPostingOp() {
                return getState().postingOp;
            },
            getQuestions() {
                return getState().questions
                    .filter(question => question.board === board)
                    .filter(question => question.archive === false)
                    .map(question => ({
                    ...question,
                    answers: question.answers
                        .filter(answer => answer.archive === false)
                }));
            },
            getUser(userId) {
                return getState().users.find(user => user.userId === userId);
            },
            async loadQuestions() {
                await loadQuestionsForBoard(board);
            },
            async postQuestion({ content }) {
                const question = await ops.operation({
                    promise: questionsPostingService.postQuestion({
                        questionDraft: { board, content }
                    }),
                    setOp: op => actions.setPostingOp(ops.replaceValue(op, undefined)),
                });
                actions.addQuestions([question]);
                const access = ops.value(getAccess());
                actions.addUsers([access.user]);
                return question;
            },
            async postAnswer(questionId, answerDraft) {
                const answer = await ops.operation({
                    promise: questionsAnsweringService.postAnswer({
                        questionId,
                        answerDraft,
                    }),
                    setOp: op => actions.setPostingOp(ops.replaceValue(op, undefined)),
                });
                actions.addAnswer(answer);
                const access = ops.value(getAccess());
                actions.addUsers([access.user]);
                return answer;
            },
            async likeQuestion(questionId, like) {
                await questionsPostingService.likeQuestion({
                    like: like,
                    questionId,
                });
                actions.setQuestionLike(questionId, like);
            },
            async likeAnswer(questionId, answerId, like) {
                await questionsAnsweringService.likeAnswer({ answerId, like });
                actions.setAnswerLike(questionId, answerId, like);
            },
            async reportQuestion(questionId, report) {
                await questionsPostingService.reportQuestion({
                    report,
                    questionId,
                });
                actions.setQuestionReport(questionId, report);
            },
            async reportAnswer(questionId, answerId, report) {
                await questionsAnsweringService.reportAnswer({ answerId, report });
                actions.setAnswerReport(questionId, answerId, report);
            },
            async archiveQuestion(questionId, archive) {
                await questionsPostingService.archiveQuestion({
                    archive,
                    questionId,
                });
                actions.setQuestionArchive(questionId, archive);
            },
            async archiveAnswer(questionId, answerId, archive) {
                await questionsAnsweringService.archiveAnswer({ answerId, archive });
                actions.setAnswerArchive(questionId, answerId, archive);
            },
            async archiveBoard() {
                await questionsModerationService.archiveBoard({ board });
                for (const question of getState().questions)
                    actions.setQuestionArchive(question.questionId, true);
            },
        });
    }

    function makeQuestionsModel(options) {
        const { questionsReadingService } = options;
        const happy = makeQuestionsModelHappy();
        const { actions, getState, onStateChange } = happy;
        async function loadQuestionsForBoard(board) {
            await ops.operation({
                promise: (async () => {
                    const { users, questions } = await questionsReadingService
                        .fetchQuestions({ board });
                    actions.addUsers(users);
                    actions.addQuestions(questions);
                })(),
                setOp: op => actions.setBoardOp(board, op),
            });
        }
        async function refreshAllBoards() {
            const state = getState();
            await Promise.all(Object.keys(state.boardOps).map(loadQuestionsForBoard));
        }
        return {
            onStateChange,
            makeBoardModel: prepareQuestionsBoardModelGetter({
                happy,
                loadQuestionsForBoard,
                ...options,
            }),
            accessChange: (access) => {
                actions.setAccess(access);
                if (access?.user)
                    actions.addUsers([access.user]);
                refreshAllBoards();
            },
        };
    }

    class AccessError extends Error {
    }
    class AccessLoginExpiredError extends AccessError {
        constructor() {
            super("login token has expired");
        }
    }

    function makeAccessModel({ authMediator, loginService }) {
        const state = snapstate({
            accessOp: ops.none(),
        });
        authMediator.subscribeToAccessChange(access => {
            state.writable.accessOp = ops.ready(access);
        });
        async function accessOperation(promise) {
            return ops.operation({
                promise,
                setOp: op => state.writable.accessOp = op,
            });
        }
        const loginFacilities = {
            async useExistingLogin() {
                await accessOperation(authMediator.initialize());
            },
            async sendLoginLink(email) {
                return loginService.sendLoginLink({ email });
            },
            async login(loginToken) {
                try {
                    if (isTokenValid(loginToken))
                        await accessOperation(loginService
                            .authenticateViaLoginToken({ loginToken })
                            .then(tokens => authMediator.login(tokens)));
                    else
                        throw new AccessLoginExpiredError();
                }
                catch (error) {
                    console.error(error);
                    state.writable.accessOp = ops.none();
                    await accessOperation(authMediator.initialize());
                    throw error;
                }
            },
            async logout() {
                await ops.operation({
                    promise: authMediator.logout(),
                    setOp: op => state.writable.accessOp = op,
                });
            },
            async reauthorize() {
                await accessOperation(authMediator.reauthorize());
            },
        };
        return {
            readable: state.readable,
            track: state.track,
            subscribe: state.subscribe,
            ...loginFacilities,
            getAccessOp() {
                return state.readable.accessOp;
            },
            getAccess() {
                return ops.value(state.readable.accessOp);
            },
            getValidAccess() {
                return authMediator.getValidAccess();
            },
        };
    }

    var LivestreamRights;
    (function (LivestreamRights) {
        LivestreamRights[LivestreamRights["Forbidden"] = 0] = "Forbidden";
        LivestreamRights[LivestreamRights["Viewer"] = 1] = "Viewer";
        LivestreamRights[LivestreamRights["Moderator"] = 2] = "Moderator";
    })(LivestreamRights || (LivestreamRights = {}));

    function objectMap(input, mapper) {
        const output = {};
        for (const [key, value] of Object.entries(input))
            output[key] = mapper(value, key);
        return output;
    }
    function objectMap2(input, mapper) {
        const output = {};
        for (const [key, value] of Object.entries(input))
            output[key] = mapper(value, key);
        return output;
    }

    // https://vimeo.com/122311493
    function makeLivestreamModel({ livestreamViewingService, livestreamModerationService, getAccessOp, }) {
        const state = snapstate({
            rights: LivestreamRights.Forbidden,
            accessOp: ops.none(),
            shows: {},
        });
        const stateActions = {
            addShow(label, show) {
                state.writable.shows = {
                    ...state.writable.shows,
                    [label]: show,
                };
            },
            clearShows() {
                state.writable.shows = objectMap(state.writable.shows, () => ops.none());
            },
        };
        function getShow(label) {
            return state.readable.shows[label]
                ?? ops.ready({ label, vimeoId: null });
        }
        async function commitShow(show) {
            return await ops.operation({
                promise: livestreamModerationService.setShow(show)
                    .then(() => show),
                setOp: op => stateActions.addShow(show.label, op)
            });
        }
        async function loadShow(label) {
            if (state.readable.rights !== LivestreamRights.Forbidden) {
                await ops.operation({
                    promise: livestreamViewingService.getShows({ labels: [label] })
                        .then(shows => shows[0]),
                    setOp: op => stateActions.addShow(label, op),
                });
            }
            else {
                stateActions.addShow(label, ops.none());
            }
        }
        async function refreshAllShows() {
            if (state.readable.rights !== LivestreamRights.Forbidden) {
                const labels = Object.keys(state.readable.shows);
                if (labels.length > 0)
                    await ops.operation({
                        promise: livestreamViewingService.getShows({ labels }),
                        setOp: op => {
                            labels.forEach((label, index) => {
                                const show = ops.isReady(op)
                                    ? ops.value(op)[index]
                                    : undefined;
                                stateActions.addShow(label, ops.replaceValue(op, show));
                            });
                        },
                    });
            }
            else
                stateActions.clearShows();
        }
        return {
            state: state.readable,
            subscribe: state.subscribe,
            getShow,
            commitShow,
            loadShow: debounce(200, loadShow),
            async updateAccessOp(op) {
                state.writable.accessOp = op;
                {
                    const access = ops.value(getAccessOp());
                    const permitted = (priv) => access.permit.privileges.includes(appPermissions.privileges[priv]);
                    state.writable.rights = !access
                        ? LivestreamRights.Forbidden
                        : permitted("moderate livestream")
                            ? LivestreamRights.Moderator
                            : permitted("view livestream")
                                ? LivestreamRights.Viewer
                                : LivestreamRights.Forbidden;
                }
                await refreshAllShows();
            },
        };
    }

    function makePersonalModel({ personalService, getAccessOp, reauthorize, }) {
        const { readable, writable, track, subscribe } = snapstate({
            accessOp: getAccessOp(),
            submitDraftOp: ops.ready(undefined),
        });
        return {
            track,
            subscribe,
            readable,
            async saveProfile(profileDraft) {
                await ops.operation({
                    promise: (async () => {
                        const { user: { userId } } = ops.value(readable.accessOp);
                        await personalService.setProfile({ userId, profileDraft });
                        await reauthorize();
                    })(),
                    errorReason: "error saving profile",
                    setOp(op) { writable.submitDraftOp = op; },
                });
            },
            updateAccessOp(op) {
                writable.accessOp = op;
            },
        };
    }

    function makeAllowanceChecker(access, privileges) {
        const allowed = {};
        for (const [key, privilegeId] of Object.entries(privileges))
            allowed[key] = access
                ? access.permit.privileges.includes(privilegeId)
                : false;
        return (privilege) => allowed[privilege];
    }

    function makeAdministrativeModel({ roleAssignmentService, reauthorize, }) {
        const state = snapstate({
            accessOp: ops.none(),
            permissionsOp: ops.none(),
        });
        function getAccess() {
            return ops.value(state.readable.accessOp);
        }
        let initializedInDom = false;
        function allowanceChecker() {
            return makeAllowanceChecker(getAccess(), appPermissions.privileges);
        }
        async function loadPermissions() {
            if (initializedInDom && allowanceChecker()("administrate user roles"))
                await ops.operation({
                    promise: roleAssignmentService.fetchPermissions(),
                    setOp: op => state.writable.permissionsOp = op,
                });
        }
        async function initialize() {
            initializedInDom = true;
            await loadPermissions();
        }
        return {
            state: state.readable,
            subscribe: state.subscribe,
            getAccess,
            initialize,
            get isAllowed() {
                return allowanceChecker();
            },
            reauthorize,
            searchUsers: roleAssignmentService.searchUsers,
            assignRoleToUser: roleAssignmentService.assignRoleToUser,
            revokeRoleFromUser: roleAssignmentService.revokeRoleFromUser,
            async updateAccessOp(op) {
                state.writable.accessOp = op;
                loadPermissions();
            },
        };
    }

    function makePermissionsModel({ permissionsService, reauthorize, }) {
        const { readable, writable, subscribe, track } = snapstate({
            active: false,
            accessOp: ops.none(),
            permissionsDisplay: ops.none(),
        });
        function getAccess() {
            return ops.value(readable.accessOp);
        }
        function sortPermissions(permissions) {
            const softPrivileges = [];
            const hardPrivileges = [];
            for (const privilege of permissions.privileges) {
                if (privilege.hard)
                    hardPrivileges.push(privilege);
                else
                    softPrivileges.push(privilege);
            }
            softPrivileges.sort((a, b) => b.time - a.time);
            return {
                ...permissions,
                privileges: [...softPrivileges, ...hardPrivileges],
            };
        }
        async function reload() {
            await ops.operation({
                promise: Promise.resolve()
                    .then(async () => getUserCanCustomizePermissions()
                    ? permissionsService.fetchPermissions().then(sortPermissions)
                    : undefined),
                setOp: op => writable.permissionsDisplay = op,
            });
        }
        function getUserCanCustomizePermissions() {
            const access = getAccess();
            return access?.user
                ? access.permit.privileges.includes(appPermissions.privileges["customize permissions"])
                : false;
        }
        async function initialize() {
            writable.active = true;
            if (getUserCanCustomizePermissions())
                await reload();
        }
        function reloadAfter(func) {
            return (async (...args) => {
                const result = await func(...args);
                await reload();
                await reauthorize();
                return result;
            });
        }
        return {
            readable,
            subscribe,
            track,
            getAccess,
            initialize,
            getUserCanCustomizePermissions,
            createRole: reloadAfter(permissionsService.createRole),
            deleteRole: reloadAfter(permissionsService.deleteRole),
            assignPrivilege: reloadAfter(permissionsService.assignPrivilege),
            unassignPrivilege: reloadAfter(permissionsService.unassignPrivilege),
            deletePrivilege: reloadAfter(permissionsService.deletePrivilege),
            createPrivilege: reloadAfter(permissionsService.createPrivilege),
            async updateAccessOp(op) {
                writable.accessOp = op;
                const access = getAccess();
                if (access?.user && writable.active)
                    await reload();
            },
        };
    }

    function makeDacastModel({ dacastService }) {
        const linkChange = subbies();
        const state = snapstate({
            accessOp: ops.none(),
            linkedAccountOp: ops.none(),
        });
        let alreadyInitialized = false;
        const isModerator = () => {
            const access = ops.value(state.readable.accessOp);
            return access && access.permit.privileges.includes(videoPrivileges["moderate videos"]);
        };
        async function loadLinkedAccount() {
            return ops.operation({
                promise: dacastService.getLink(),
                setOp: op => state.writable.linkedAccountOp = op,
            });
        }
        async function refresh() {
            if (alreadyInitialized && isModerator()) {
                await loadLinkedAccount();
            }
        }
        return {
            state: state.readable,
            subscribe: state.subscribe,
            onLinkChange: linkChange.subscribe,
            async updateAccessOp(op) {
                state.writable.accessOp = op;
                state.writable.linkedAccountOp = ops.none();
                await refresh();
            },
            async initialize() {
                if (!alreadyInitialized) {
                    alreadyInitialized = true;
                    await refresh();
                }
            },
            get linkedAccount() {
                return ops.value(state.readable.linkedAccountOp);
            },
            async linkAccount({ apiKey }) {
                return ops.operation({
                    setOp: op => state.writable.linkedAccountOp = op,
                    promise: dacastService.setLink({ apiKey })
                        .then(link => {
                        linkChange.publish(link);
                        return link;
                    }),
                });
            },
            async unlinkAccount() {
                return ops.operation({
                    setOp: op => state.writable.linkedAccountOp = op,
                    promise: dacastService.clearLink()
                        .then(() => linkChange.publish(undefined)),
                });
            },
        };
    }

    function makeContentModel({ contentService, }) {
        const state = snapstate({
            accessOp: ops.none(),
            catalogOp: ops.none(),
            viewsOp: ops.none(),
            privilegesOp: ops.none(),
            showsOp: ops.none(),
        });
        const activeShowLabels = new Set();
        async function loadModerationData() {
            const access = ops.value(state.readable.accessOp);
            const isModerator = access && access.permit
                .privileges.includes(videoPrivileges["moderate videos"]);
            if (isModerator) {
                await ops.operation({
                    promise: contentService.fetchModerationData(),
                    setOp: op => {
                        const data = ops.value(op);
                        state.writable.catalogOp = ops.replaceValue(op, data?.catalog ?? []);
                        state.writable.viewsOp = ops.replaceValue(op, data?.views ?? []);
                        state.writable.privilegesOp = ops.replaceValue(op, data?.privileges ?? []);
                    },
                });
            }
        }
        async function loadShow(label) {
            activeShowLabels.add(label);
            const oldShows = ops.value(state.readable.showsOp) ?? [];
            let updatedShow;
            await ops.operation({
                setOp: op => state.writable.showsOp = op,
                promise: contentService.getShows({ labels: [label] })
                    .then(shows => shows[0])
                    .then(show => updatedShow = show)
                    .then(show => [
                    ...oldShows.filter(s => s.label !== label),
                    ...(show ? [show] : [{ label, details: undefined }]),
                ]),
            });
            return updatedShow;
        }
        async function refreshShows() {
            const labels = Array.from(activeShowLabels);
            if (labels.length)
                await ops.operation({
                    setOp: op => state.writable.showsOp = op,
                    promise: contentService.getShows({ labels })
                        .then(shows => shows.filter(s => !!s))
                });
        }
        let alreadyInitialized = false;
        async function initialize(label) {
            if (!alreadyInitialized) {
                alreadyInitialized = true;
                await Promise.all([
                    loadShow(label),
                    loadModerationData(),
                ]);
            }
        }
        return {
            state: state.readable,
            subscribe: state.subscribe,
            async updateAccessOp(op) {
                state.writable.accessOp = op;
                state.writable.catalogOp = ops.none();
                state.writable.viewsOp = ops.none();
                await loadModerationData();
                await refreshShows();
            },
            async onVideoHostingUpdate() {
                await loadModerationData();
                await refreshShows();
            },
            initialize,
            loadModerationData,
            get allowance() {
                const access = ops.value(state.readable.accessOp);
                const can = (p) => access
                    ? access.permit.privileges.includes(videoPrivileges[p])
                    : false;
                return {
                    canModerateVideos: can("moderate videos"),
                    canViewAllVideos: can("view all videos"),
                };
            },
            get catalog() {
                return ops.value(state.readable.catalogOp) ?? [];
            },
            get views() {
                return ops.value(state.readable.viewsOp) ?? [];
            },
            get privileges() {
                return ops.value(state.readable.privilegesOp) ?? [];
            },
            get shows() {
                return ops.value(state.readable.showsOp) ?? [];
            },
            getView(label) {
                return (ops.value(state.readable.viewsOp) ?? [])
                    .find(view => view.label === label);
            },
            getPrivilege(id) {
                return (ops.value(state.readable.privilegesOp) ?? [])
                    .find(p => p.privilegeId === id);
            },
            getShow(label) {
                return (ops.value(state.readable.showsOp) ?? [])
                    .find(show => show.label === label);
            },
            async setView(options) {
                const oldViews = ops.value(state.readable.viewsOp) ?? [];
                await ops.operation({
                    setOp: op => state.writable.viewsOp = op,
                    promise: contentService.writeView(options).then(() => [
                        ...oldViews.filter(v => v.label !== options.label),
                        {
                            ...options.reference,
                            label: options.label,
                            privileges: options.privileges,
                        },
                    ]),
                });
                await loadShow(options.label);
            },
            async deleteView(label) {
                const oldViews = ops.value(state.readable.viewsOp) ?? [];
                await ops.operation({
                    setOp: op => state.writable.viewsOp = op,
                    promise: contentService.deleteView({ label })
                        .then(() => oldViews.filter(v => v.label !== label)),
                });
                const oldShows = ops.value(state.readable.showsOp) ?? [];
                state.writable.showsOp = ops.replaceValue(state.readable.showsOp, oldShows.filter(s => s.label !== label));
            },
        };
    }

    function makeVideoModels(options) {
        const dacastModel = makeDacastModel(options);
        const contentModel = makeContentModel(options);
        dacastModel.onLinkChange(contentModel.onVideoHostingUpdate);
        return {
            dacastModel,
            contentModel,
            updateAccessOp(accessOp) {
                dacastModel.updateAccessOp(accessOp);
                contentModel.updateAccessOp(accessOp);
            },
        };
    }

    async function assembleModels({ appId, remote, popups, storage, authMediator, }) {
        const accessModel = makeAccessModel({
            authMediator,
            loginService: remote.auth.users.loginService,
        });
        const { getAccessOp, getAccess, getValidAccess, reauthorize } = accessModel;
        const exampleModel = makeExampleModel({ getAccessOp });
        const personalModel = makePersonalModel({
            personalService: remote.auth.users.personalService,
            getAccessOp,
            reauthorize,
        });
        const appsModel = makeAppsModel({
            appService: remote.auth.apps.appService,
            appEditService: remote.auth.apps.appEditService,
            getValidAccess,
        });
        const permissionsModel = makePermissionsModel({
            permissionsService: remote.auth.permissions.permissionsService,
            reauthorize,
        });
        const livestreamModel = makeLivestreamModel({
            ...remote.livestream,
            getAccessOp,
        });
        const videoModels = makeVideoModels({
            dacastService: remote.videos.dacastService,
            contentService: remote.videos.contentService,
        });
        // // TODO reactivate store
        // const storeModel = makeStoreModel({
        // 	appId,
        // 	storage,
        // 	shopkeepingService: remote.store.shopkeepingService,
        // 	stripeAccountsService: remote.store.stripeConnectService,
        // 	statusCheckerService: remote.store.ecommerce.statusCheckerService,
        // 	statusTogglerService: remote.store.ecommerce.statusTogglerService,
        // 	triggerBankPopup: popups.triggerBankPopup,
        // })
        const administrativeModel = makeAdministrativeModel({
            roleAssignmentService: remote.administrative.roleAssignmentService,
            reauthorize: () => accessModel.reauthorize(),
        });
        const questionsModel = makeQuestionsModel({
            ...remote.questions,
            getAccess: getAccessOp,
        });
        accessModel.subscribe(async ({ accessOp }) => {
            const access = getAccess();
            await Promise.all([
                exampleModel.updateAccessOp(accessOp),
                personalModel.updateAccessOp(accessOp),
                livestreamModel.updateAccessOp(accessOp),
                appsModel.updateAccessOp(accessOp),
                permissionsModel.updateAccessOp(accessOp),
                questionsModel.accessChange(access),
                administrativeModel.updateAccessOp(accessOp),
                videoModels.dacastModel.updateAccessOp(accessOp),
                videoModels.contentModel.updateAccessOp(accessOp),
                // storeModel.accessChange(access),
            ]);
        });
        return {
            exampleModel,
            appsModel,
            accessModel,
            videoModels,
            // storeModel,
            personalModel,
            questionsModel,
            livestreamModel,
            permissionsModel,
            administrativeModel,
        };
    }

    function mockModalSystem() {
        const nextModalResults = {
            confirm: true,
            prompt: undefined,
        };
        const modals = {
            popup() {
                return {
                    controls: {
                        close() { },
                        rerender() { },
                    },
                    modal: {},
                };
            },
            async alert() { },
            async confirm() {
                return nextModalResults.confirm;
            },
            async prompt() {
                return nextModalResults.prompt;
            },
        };
        return {
            modals,
            nextModalResults,
        };
    }

    async function mockBrowser({ api, mockStripeOperations }) {
        async function mockAppWindow({ appId, apiLink, windowLink, }) {
            const storage = memoryFlexStorage();
            const { remote, authMediator } = mockRemote({
                api,
                appId,
                apiLink,
                storage,
                origin: new URL(windowLink).origin,
            });
            const { nextModalResults } = mockModalSystem();
            const models = await assembleModels({
                appId,
                remote,
                storage,
                authMediator,
                popups: mockPopups({ mockStripeOperations }),
            });
            return { models, remote, nextModalResults };
        }
        return { mockAppWindow };
    }

    function asApi(api) {
        return api;
    }

    function isFunction(value) {
        return typeof value === "function";
    }

    const _context = Symbol();

    const _descriptor = Symbol();

    function apiContext() {
        return function recurse({ expose, policy }) {
            return objectMap$1(expose, (value, key) => {
                if (isFunction(value))
                    return {
                        [_descriptor]: true,
                        policy,
                        func: value,
                    };
                else if (isObject(value))
                    return {
                        [_context]: true,
                        ...recurse({ policy, expose: value })
                    };
                else
                    throw new Error(`unknown api context subtype for "${key}"`);
            });
        };
    }

    function isPlatform(appId, config) {
        return config.platform.appDetails.appId === appId;
    }

    async function concurrent(obj) {
        const keys = Object.keys(obj);
        const awaitables = Object.values(obj);
        const values = await Promise.all(awaitables);
        const result = {};
        keys.forEach((key, i) => result[key] = values[i]);
        return result;
    }

    function validator$1(...conditions) {
        return value => {
            let problems = [];
            for (const condition of conditions) {
                problems = condition(value);
                if (problems.length > 0)
                    break;
            }
            return problems;
        };
    }
    function schema(schematic) {
        const schematicKeys = Object.keys(schematic);
        return object => (Object.entries(object).flatMap(([key, value]) => schematicKeys.includes(key)
            ? schematic[key](value)
                .map(problem => `"${key}": ${problem}`)
            : [`"${key} not in schema"`]));
    }
    function branch(...conditions) {
        return value => {
            const results = conditions.map(condition => condition(value));
            let anySuccess = false;
            for (const problems of results)
                if (problems.length === 0)
                    anySuccess = true;
            return anySuccess
                ? []
                : results.flat()
                    .map((problem, index) => index > 0 ? `or, ${problem}` : problem);
        };
    }
    function each(...conditions) {
        return arr => {
            if (!Array.isArray(arr))
                return ["must be array"];
            const validate = validator$1(...conditions);
            const problems = [];
            arr.forEach((item, index) => {
                for (const problem of validate(item))
                    problems.push(`(${index + 1}) ${problem}`);
            });
            return problems;
        };
    }
    const notDefined = () => value => (value !== undefined && value !== null)
        ? ["must be undefined or null"]
        : [];
    const string = () => value => typeof value !== "string"
        ? ["must be a string"]
        : [];
    const number = () => value => typeof value !== "number"
        ? ["must be a number"]
        : [];
    const boolean = () => value => typeof value !== "boolean"
        ? ["must be a boolean"]
        : [];
    const array = () => value => Array.isArray(value)
        ? []
        : ["must be an array"];
    const length = (len) => value => value.length !== len
        ? [`length must be ${len}`]
        : [];
    const minLength = (min) => value => value.length < min
        ? ["too small"]
        : [];
    const maxLength = (max) => value => value.length > max
        ? ["too big"]
        : [];
    const notWhitespace = () => value => value.length > 0 && value.trim().length === 0
        ? ["can't be all whitespace"]
        : [];
    const zeroWhitespace = () => value => /\s/.test(value)
        ? ["must not have any spaces"]
        : [];
    const url = () => value => {
        try {
            void new URL(value);
            return [];
        }
        catch (error) {
            return ["invalid url"];
        }
    };
    const localhost = () => value => {
        return /^https?:\/\/(127\.0\.0\.1|localhost)(|:\d{1,5})(|\/\S*)$/i.test(value)
            ? []
            : ["must be a localhost address"];
    };
    const https = () => value => {
        return /^https:\/\//i.test(value)
            ? []
            : ["must be secure, starting with 'https'"];
    };
    const origin = () => value => /^https?:\/\/[a-zA-Z\.\d-]+(?:|:\d+)$/i.test(value)
        ? []
        : ["invalid origin"];
    const regex = (r, problem = "invalid string") => value => r.test(value)
        ? []
        : [problem];
    const emailRegex = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    const email = () => value => emailRegex.test(value)
        ? []
        : ["invalid email"];
    // const labelValidator = validator<string>(string(), minLength(1), maxLength(50), notWhitespace())
    // const problems = labelValidator("hello")
    // const problems2 = validator<string[]>(array(string()))

    const validateAppOrigin = validator$1(string(), maxLength(1000), origin(), branch(https(), localhost()));
    const appDraftValidators = Object.freeze({
        home: validator$1(string(), minLength(1), maxLength(2000), branch(validator$1(url(), https()), localhost())),
        label: validator$1(string(), minLength(1), maxLength(50), notWhitespace()),
        origins: validator$1(minLength(1), maxLength(100), each(validateAppOrigin)),
        additionalOrigins: validator$1(array(), minLength(0), maxLength(99), each(validateAppOrigin)),
    });

    const validator = schema({
        home: appDraftValidators.home,
        label: appDraftValidators.label,
        origins: appDraftValidators.origins,
    });
    function validateAppDraft(appDraft) {
        return validator(appDraft);
    }

    function originsToDatabase(origins) {
        return origins.join(";");
    }

    const _dbbyTableSymbol = Symbol("dbbyTable");

    function and(...conditions) {
        return ["and", ...conditions];
    }
    function or(...conditions) {
        return ["or", ...conditions];
    }
    function find(...rows) {
        return rows.length
            ? { conditions: or(...rows.map(row => ({ equal: row }))) }
            : { conditions: false };
    }
    function findAll(values, valueForRow) {
        return { conditions: or(...values.map(v => ({ equal: valueForRow(v) }))) };
    }
    function isDbbyTable(x) {
        return x && typeof x === "object" && x[_dbbyTableSymbol];
    }

    function originsFromDatabase(origins) {
        return origins.split(";");
    }

    function throwProblems(problems) {
        if (problems.length > 0)
            throw new ApiError(400, problems.join("; "));
    }

    const makeAppService = ({ rando, config, authPolicies, }) => apiContext()({
        policy: authPolicies.platformUserPolicy,
        expose: {
            async listApps({ appTables, statsHub }, { ownerUserId: ownerUserIdString }) {
                const ownerUserId = DamnId.fromString(ownerUserIdString);
                const ownerships = await appTables.owners.read(find({ userId: ownerUserId }));
                const appRows = ownerships.length
                    ? await appTables.registrations.read({
                        conditions: and(or(...ownerships.map(own => ({ equal: { appId: own.appId } }))), { equal: { archived: false } })
                    })
                    : [];
                return Promise.all(appRows.map(async (row) => ({
                    appId: row.appId.toString(),
                    label: row.label,
                    home: row.home,
                    origins: originsFromDatabase(row.origins),
                    platform: isPlatform(row.appId.toString(), config),
                    stats: await concurrent({
                        users: statsHub.countUsers(row.appId),
                        usersActiveDaily: statsHub.countUsersActiveDaily(row.appId),
                        usersActiveMonthly: statsHub.countUsersActiveMonthly(row.appId),
                    }),
                })));
            },
            async registerApp({ appTables }, { appDraft, ownerUserId: ownerUserIdString }) {
                throwProblems(validateAppDraft(appDraft));
                const ownerUserId = DamnId.fromString(ownerUserIdString);
                const appId = rando.randomId();
                await Promise.all([
                    appTables.registrations.create({
                        appId,
                        label: appDraft.label,
                        home: appDraft.home,
                        origins: originsToDatabase(appDraft.origins),
                        archived: false,
                    }),
                    appTables.owners.create({
                        appId,
                        userId: ownerUserId,
                    }),
                ]);
                return {
                    ...appDraft,
                    appId: appId.toString(),
                    stats: {
                        users: 1,
                        usersActiveDaily: 0,
                        usersActiveMonthly: 0,
                    },
                };
            },
        },
    });

    const profileFromRow = ({ avatar, tagline, nickname, }) => ({
        tagline,
        nickname,
        avatar: JSON.parse(avatar),
    });

    async function fetchUsers({ userIds, authTables, permissionsEngine }) {
        if (!userIds.length)
            throw new Error("invalid: userIds cannot be empty");
        const conditions = or(...userIds.map(userId => ({ equal: { userId } })));
        const accounts = await authTables.users.accounts.read({ conditions });
        const profiles = await authTables.users.profiles.read({ conditions });
        const publicRolesForUsers = await permissionsEngine
            .getPublicRolesForUsers(userIds.map(id => id.toString()));
        function assembleDetailsForEachUser(userId) {
            const account = accounts.find(a => a.userId.toString() === userId.toString());
            const profile = profiles.find(p => p.userId.toString() === userId.toString());
            if (!account)
                throw new ApiError(404, `account not found for user id ${userId}`);
            if (!profile)
                throw new ApiError(404, `profile not found for user id ${userId}`);
            const roles = publicRolesForUsers
                .find(r => r.userId.toString() === userId.toString())
                .publicUserRoles;
            const stats = {
                joined: account.created,
            };
            return {
                userId: userId.toString(),
                profile: profileFromRow(profile),
                roles,
                stats,
            };
        }
        return userIds.map(assembleDetailsForEachUser);
    }

    async function fetchUser({ userId, ...options }) {
        const results = await fetchUsers({ ...options, userIds: [userId] });
        return results.find(r => r.userId);
    }

    function permissionsMergingFacility({ isPlatform }) {
        const hardPermissions = isPlatform
            ? platformPermissions
            : appPermissions;
        function getHardPrivilegeDetails(...roleIds) {
            const results = [];
            for (const roleId of roleIds) {
                const found = Object.entries(hardPermissions.roles)
                    .find(([, role2]) => role2.roleId === roleId);
                if (found) {
                    const [, role] = found;
                    for (const [label, has] of Object.entries(role.hasPrivileges)) {
                        const privilegeId = hardPermissions.privileges[label];
                        const already = results.find(detail => detail.roleId === roleId &&
                            detail.privilegeId === privilegeId);
                        if (!already) {
                            results.push({
                                privilegeId,
                                roleId,
                                active: has.active,
                                immutable: has.immutable,
                            });
                        }
                    }
                }
            }
            return results;
        }
        function mergeRoleHasPrivileges({ hard, soft }) {
            const results = [];
            function rowMatch(hardy, softy) {
                return hardy.roleId === softy.roleId.toString()
                    && hardy.privilegeId === softy.privilegeId.toString();
            }
            function toSofty(hardy) {
                return {
                    roleId: DamnId.fromString(hardy.roleId),
                    privilegeId: DamnId.fromString(hardy.privilegeId),
                    active: hardy.active,
                    immutable: hardy.immutable,
                    time: 0,
                };
            }
            for (const softy of soft) {
                const hardy = hard.find(h => rowMatch(h, softy));
                results.push(hardy && hardy.immutable
                    ? toSofty(hardy)
                    : softy);
            }
            for (const hardy of hard) {
                const alreadyIncluded = !!results.find(softy => rowMatch(hardy, softy));
                if (!alreadyIncluded)
                    results.push(toSofty(hardy));
            }
            return results;
        }
        function getActivePrivilegeIds(roleHasPrivileges) {
            return roleHasPrivileges
                .filter(row => row.active)
                .map(row => row.privilegeId);
        }
        return {
            hardPermissions,
            mergeRoleHasPrivileges,
            getActivePrivilegeIds,
            getHardPrivilegeDetails,
        };
    }

    function isDefined(x) {
        return x !== undefined && x !== null;
    }

    function isCurrentlyWithinTimeframe({ timeframeStart, timeframeEnd }) {
        const time = Date.now();
        const tooEarly = isDefined(timeframeStart)
            ? time > timeframeStart
            : false;
        const tooLate = isDefined(timeframeEnd)
            ? time < timeframeEnd
            : false;
        return !tooEarly && !tooLate;
    }

    function makePermissionsEngine({ isPlatform, permissionsTables }) {
        const { hardPermissions, mergeRoleHasPrivileges, getActivePrivilegeIds, getHardPrivilegeDetails, } = permissionsMergingFacility({ isPlatform });
        async function getAnonymousPrivileges() {
            const roleId = hardPermissions.roles.everybody.roleId;
            const hard = getHardPrivilegeDetails(roleId);
            const soft = await permissionsTables.roleHasPrivilege
                .read(find({ roleId: DamnId.fromString(roleId) }));
            return getActivePrivilegeIds(mergeRoleHasPrivileges({ hard, soft }))
                .map(id => id.toString());
        }
        async function getUsersHaveRoles({ userIds, onlyGetPublicRoles }) {
            const isPublic = (row) => row.public;
            const all = (x) => x;
            const usersHaveRoles = await (async () => {
                if (userIds.length === 0)
                    throw new Error("invalid: user ids cannot be empty");
                const usersHaveRolesRaw = await permissionsTables.userHasRole.read({
                    conditions: or(...userIds.map(userId => ({
                        equal: { userId: DamnId.fromString(userId) }
                    }))),
                });
                const roleIds = usersHaveRolesRaw.map(u => u.roleId);
                const roles = roleIds.length
                    ? await permissionsTables.role.read({
                        conditions: or(...roleIds.map(roleId => ({ equal: { roleId } })))
                    })
                    : [];
                const roleIdsThatActuallyExist = [
                    ...roles.map(r => r.roleId.toString()),
                    ...Object.entries(hardPermissions.roles)
                        .map(([, role]) => role.roleId),
                ];
                return usersHaveRolesRaw
                    .filter(u => roleIdsThatActuallyExist.includes(u.roleId.toString()));
            })();
            return userIds.map(userId => {
                const raw = usersHaveRoles.filter(r => r.userId.toString() === userId);
                const userHasRoles = raw
                    .filter(isCurrentlyWithinTimeframe)
                    .filter(onlyGetPublicRoles ? isPublic : all);
                return { userId, userHasRoles };
            });
        }
        async function getPrivilegesForUsers(userIds) {
            const rolesForUsers = await getUsersHaveRoles({
                userIds,
                onlyGetPublicRoles: false,
            });
            const allRoleIds = rolesForUsers
                .flatMap(r => r.userHasRoles.map(r2 => r2.roleId));
            const allRolesHavePrivileges = allRoleIds.length
                ? await permissionsTables.roleHasPrivilege.read({
                    conditions: or(...allRoleIds.map(roleId => ({ equal: ({ roleId }) })))
                })
                : [];
            function resolvePrivilegesForEachUser(userId) {
                const roleIds = rolesForUsers
                    .find(r => r.userId === userId)
                    .userHasRoles
                    .map(r => r.roleId.toString());
                // roleIds.push(universalPermissions.roles.authenticated.roleId)
                const hard = getHardPrivilegeDetails(...roleIds);
                const soft = roleIds
                    .flatMap(roleId => allRolesHavePrivileges.filter(p => p.roleId.toString() === roleId));
                const privileges = getActivePrivilegeIds(mergeRoleHasPrivileges({ hard, soft }))
                    .map(id => id.toString());
                return { userId, privileges };
            }
            return userIds.map(resolvePrivilegesForEachUser);
        }
        async function getPublicRolesForUsers(userIds) {
            const rolesForUsers = await getUsersHaveRoles({
                userIds,
                onlyGetPublicRoles: true,
            });
            const allRoleIds = rolesForUsers
                .flatMap(r => r.userHasRoles
                .filter(r2 => r2.public)
                .map(r2 => r2.roleId.toString()));
            const allHardRoles = allRoleIds.map(roleId => {
                const found = Object.entries(hardPermissions.roles)
                    .find(([, role]) => role.roleId === roleId);
                if (found) {
                    const [label, role] = found;
                    return {
                        roleId: DamnId.fromString(roleId),
                        label,
                        hard: true,
                        public: role.public,
                        assignable: role.assignable,
                        time: 0,
                    };
                }
                else {
                    return undefined;
                }
            }).filter(r => !!r);
            const allSoftRoles = allRoleIds.length
                ? await permissionsTables.role.read({
                    conditions: or(...allRoleIds.map(roleId => ({ equal: { roleId: DamnId.fromString(roleId) } })))
                })
                : [];
            const mergedRoles = [...allHardRoles];
            for (const role of allSoftRoles) {
                const found = mergedRoles.find(r => r.roleId === role.roleId);
                if (!found)
                    mergedRoles.push(role);
            }
            function assemblePublicRolesForEachUser(userId) {
                const userHasRoles = rolesForUsers
                    .find(r => r.userId === userId)
                    .userHasRoles;
                const publicUserRoles = userHasRoles
                    .map(userHasRole => {
                    const role = mergedRoles.find(r => r.roleId.toString() === userHasRole.roleId.toString());
                    if (!role)
                        console.warn(`missing role "${userHasRole.roleId.toString()}"`);
                    return role
                        ? { ...userHasRole, ...role }
                        : undefined;
                })
                    .filter(r => !!r)
                    .filter(isCurrentlyWithinTimeframe)
                    .map(r => ({
                    label: r.label,
                    roleId: r.roleId.toString(),
                    timeframeEnd: r.timeframeEnd,
                    timeframeStart: r.timeframeStart,
                }));
                return { userId, publicUserRoles };
            }
            return userIds.map(assemblePublicRolesForEachUser);
        }
        const { getPrivileges, getAllPrivileges } = (() => {
            function getAllHardPrivileges() {
                return Object.entries(hardPermissions.privileges)
                    .map(([label, privilegeId]) => ({
                    privilegeId: DamnId.fromString(privilegeId),
                    label,
                    hard: true,
                    time: 0,
                }));
            }
            function mergePrivileges(soft, hard) {
                return merge(soft, hard, (a, b) => a.privilegeId === b.privilegeId)
                    .map(({ hard, label, time, privilegeId }) => ({
                    hard,
                    label,
                    time: time ?? 0,
                    privilegeId: privilegeId.toString(),
                }));
            }
            return {
                async getPrivileges(privilegeIds) {
                    const soft = privilegeIds.length
                        ? await permissionsTables.privilege
                            .read(findAll(privilegeIds, id => ({
                            privilegeId: DamnId.fromString(id)
                        })))
                        : [];
                    const hard = getAllHardPrivileges()
                        .filter(p => privilegeIds.includes(p.privilegeId.toString()));
                    return mergePrivileges(soft, hard);
                },
                async getAllPrivileges() {
                    const soft = await permissionsTables.privilege.read({ conditions: false });
                    const hard = getAllHardPrivileges();
                    return mergePrivileges(soft, hard);
                },
            };
        })();
        async function getPermissionsDisplay() {
            const all = { conditions: false };
            return concurrent({
                roles: (async () => {
                    const soft = await permissionsTables.role.read(all);
                    const hard = Object.entries(hardPermissions.roles)
                        .map(([label, r]) => ({
                        label,
                        hard: true,
                        roleId: DamnId.fromString(r.roleId),
                        public: r.public,
                        assignable: r.assignable,
                        time: 0,
                    }));
                    return merge(soft, hard, (a, b) => a.roleId === b.roleId)
                        .map(result => ({
                        roleId: result.roleId.toString(),
                        assignable: result.assignable,
                        label: result.label,
                        hard: result.hard,
                        public: result.public,
                        time: result.time ?? 0,
                    }));
                })(),
                privileges: getAllPrivileges(),
                rolesHavePrivileges: (async () => {
                    const roleIds = Object.values(hardPermissions.roles)
                        .map(role => role.roleId);
                    const hard = getHardPrivilegeDetails(...roleIds);
                    const soft = await permissionsTables.roleHasPrivilege.read(all);
                    return mergeRoleHasPrivileges({ hard, soft })
                        .map(({ active, privilegeId, immutable, roleId, time }) => ({
                        active,
                        immutable,
                        time: time ?? 0,
                        roleId: roleId.toString(),
                        privilegeId: privilegeId.toString(),
                    }));
                })(),
            });
        }
        async function getUserPrivileges(userId) {
            const result = await getPrivilegesForUsers([userId]);
            return result
                .find(r => r.userId === userId)
                .privileges;
        }
        // async function getUserPublicRoles(userId: string) {
        // 	const result = await getPublicRolesForUsers([userId])
        // 	return result
        // 		.find(r => r.userId === userId)
        // 		.publicUserRoles
        // }
        return {
            getUsersHaveRoles,
            getAnonymousPrivileges,
            getPrivilegesForUsers,
            getPublicRolesForUsers,
            getPermissionsDisplay,
            getPrivileges,
            getAllPrivileges,
            getUserPrivileges,
        };
    }

    const makeUserService = (options) => apiContext()({
        policy: options.authPolicies.anonPolicy,
        expose: {
            async getUser({ authTables, access }, { userId }) {
                const permissionsEngine = makePermissionsEngine({
                    permissionsTables: authTables.permissions,
                    isPlatform: access.appId === options.config.platform.appDetails.appId,
                });
                return await fetchUser({
                    authTables,
                    permissionsEngine,
                    userId: DamnId.fromString(userId),
                });
            },
        },
    });

    const makeGreenService = (options) => apiContext()({
        policy: options.authPolicies.greenPolicy,
        expose: {
            async authorize({ appTables, authTables: unconstrainedAuthTables }, { scope, refreshToken, appId: appIdString, }) {
                const appId = DamnId.fromString(appIdString);
                const authTables = unconstrainedAuthTables.namespaceForApp(appId);
                const permissionsEngine = makePermissionsEngine({
                    permissionsTables: authTables.permissions,
                    isPlatform: appId.toString() === options.config.platform.appDetails.appId,
                });
                const appRow = await appTables.registrations.one(find({ appId }));
                if (!appRow)
                    throw new ApiError(400, "incorrect app id");
                if (appRow.archived)
                    throw new ApiError(403, "app has been archived");
                if (refreshToken) {
                    const { userId: userIdString } = await options.verifyToken(refreshToken);
                    const userId = DamnId.fromString(userIdString);
                    const user = await fetchUser({
                        userId,
                        authTables,
                        permissionsEngine,
                    });
                    await authTables.users.latestLogins.update({
                        ...find({ userId }),
                        upsert: { userId, time: Date.now() },
                    });
                    const privileges = await permissionsEngine
                        .getUserPrivileges(userId.toString());
                    return options.signToken({
                        lifespan: options.config.crypto.tokenLifespans.access,
                        payload: {
                            user,
                            scope,
                            permit: { privileges },
                            appId: appId.toString(),
                            origins: originsFromDatabase(appRow.origins),
                        },
                    });
                }
                else {
                    const privileges = await permissionsEngine.getAnonymousPrivileges();
                    return options.signToken({
                        lifespan: options.config.crypto.tokenLifespans.access,
                        payload: {
                            user: undefined,
                            appId: appId.toString(),
                            scope,
                            origins: originsFromDatabase(appRow.origins),
                            permit: { privileges },
                        },
                    });
                }
            }
        },
    });

    async function signAuthTokens({ scope, appId, userId, origins, lifespans, authTables, permissionsEngine, signToken, }) {
        const { user, permit } = await concurrent({
            user: fetchUser({ userId, authTables, permissionsEngine }),
            permit: (async () => ({
                privileges: await permissionsEngine.getUserPrivileges(userId.toString()),
            }))(),
        });
        return concurrent({
            accessToken: signToken({
                payload: { appId, origins, user, permit, scope },
                lifespan: lifespans.access,
            }),
            refreshToken: signToken({
                payload: { userId: user.userId },
                lifespan: lifespans.refresh,
            }),
        });
    }

    function generateAccountRow({ rando }) {
        return {
            userId: rando.randomId(),
            created: Date.now(),
        };
    }

    const generateProfileRow = ({ userId, avatar, generateNickname }) => ({
        userId,
        avatar,
        tagline: "",
        nickname: generateNickname(),
    });

    function randomXioAvatarSimpleSpec() {
        return {
            type: "simple",
            value: Math.random(),
        };
    }

    async function initializeUserProfile({ userId, authTables, email, generateNickname }) {
        const avatar = JSON.stringify(randomXioAvatarSimpleSpec());
        await authTables.users.profiles.create(generateProfileRow({
            userId,
            avatar,
            generateNickname,
        }));
    }

    const standardRoleIds = {
        everybody: universalPermissions.roles.everybody.roleId,
        authenticated: universalPermissions.roles.authenticated.roleId,
        technician: universalPermissions.roles.technician.roleId,
    };
    async function assertEmailAccount({ rando, email, authTables, config, generateNickname, }) {
        email = email.toLowerCase();
        const accountViaEmail = await authTables.users.emails.assert({
            ...find({ email }),
            make: async function makeNewAccountViaEmail() {
                const isTechnician = email === config.platform.technician.email;
                const account = generateAccountRow({ rando });
                const { userId } = account;
                const createAccount = authTables.users.accounts.create(account);
                const createProfile = initializeUserProfile({
                    userId,
                    email,
                    authTables,
                    generateNickname,
                });
                const assignEverybody = authTables.permissions.userHasRole.create({
                    userId,
                    hard: true,
                    public: false,
                    roleId: DamnId.fromString(standardRoleIds.everybody),
                    timeframeEnd: undefined,
                    timeframeStart: undefined,
                    time: Date.now(),
                });
                const assignAuthenticated = authTables.permissions.userHasRole.create({
                    userId,
                    hard: true,
                    public: false,
                    roleId: DamnId.fromString(standardRoleIds.authenticated),
                    timeframeEnd: undefined,
                    timeframeStart: undefined,
                    time: Date.now(),
                });
                const assignTechnician = isTechnician
                    ? authTables.permissions.userHasRole.create({
                        userId,
                        hard: true,
                        public: true,
                        timeframeEnd: undefined,
                        roleId: DamnId.fromString(standardRoleIds.technician),
                        timeframeStart: undefined,
                        time: Date.now(),
                    })
                    : Promise.resolve();
                await Promise.all([
                    createAccount,
                    createProfile,
                    assignEverybody,
                    assignAuthenticated,
                    assignTechnician,
                ]);
                return { email, userId };
            },
        });
        return { userId: accountViaEmail.userId };
    }

    function runValidation(value, validator) {
        const problems = validator(value);
        if (problems.length > 0)
            throw new ApiError(400, problems.join("; "));
        else
            return value;
    }

    const makeLoginService = ({ rando, config, authPolicies, signToken, verifyToken, sendLoginEmail, generateNickname, }) => apiContext()({
        policy: authPolicies.anonPolicy,
        expose: {
            async sendLoginLink({ access, authTables, appTables }, inputs) {
                const { email: rawEmail } = runValidation(inputs, schema({ email: email() }));
                const email$1 = rawEmail.toLowerCase();
                const appId = DamnId.fromString(access.appId);
                const appRow = await appTables.registrations.one(find({ appId }));
                const { userId } = await assertEmailAccount({
                    rando, email: email$1, config, authTables, generateNickname,
                });
                const loginTokenPayload = { userId: userId.toString() };
                await sendLoginEmail({
                    appHome: appRow.home,
                    appLabel: appRow.label,
                    to: email$1,
                    legalLink: config.platform.legalLink,
                    platformLink: config.platform.appDetails.home,
                    lifespan: config.crypto.tokenLifespans.login,
                    loginToken: await signToken({
                        payload: loginTokenPayload,
                        lifespan: config.crypto.tokenLifespans.login,
                    }),
                });
            },
            async authenticateViaLoginToken({ access, authTables }, { loginToken }) {
                const verified = await verifyToken(loginToken);
                const userId = DamnId.fromString(verified.userId);
                const authTokens = await signAuthTokens({
                    userId,
                    authTables,
                    scope: { core: true },
                    appId: access.appId,
                    origins: access.origins,
                    lifespans: config.crypto.tokenLifespans,
                    permissionsEngine: makePermissionsEngine({
                        isPlatform: access.appId === config.platform.appDetails.appId,
                        permissionsTables: authTables.permissions,
                    }),
                    signToken,
                });
                await authTables.users.latestLogins.update({
                    ...find({ userId }),
                    upsert: { userId, time: Date.now() },
                });
                return authTokens;
            },
        },
    });

    const adminRoleId$1 = DamnId.fromString(appPermissions.roles.admin.roleId);
    async function appointAdmin({ email, authTables, rando, config, generateNickname, }) {
        const { userId: adminUserId } = await assertEmailAccount({
            rando,
            email,
            config,
            authTables,
            generateNickname,
        });
        await authTables.permissions.userHasRole.assert({
            ...find({ userId: adminUserId, roleId: adminRoleId$1 }),
            make: async () => ({
                userId: adminUserId,
                roleId: adminRoleId$1,
                hard: false,
                public: true,
                timeframeEnd: undefined,
                timeframeStart: undefined,
                time: Date.now()
            })
        });
    }

    const emailValidator = validator$1(string(), maxLength(256), email());

    const adminRoleId = DamnId.fromString(appPermissions.roles.admin.roleId);
    const makeAppEditService = ({ rando, config, authPolicies, generateNickname, }) => apiContext()({
        policy: authPolicies.appOwnerPolicy,
        expose: {
            async updateApp({ appTables, authorizeAppOwner }, { appId: appIdString, appDraft }) {
                throwProblems(validateAppDraft(appDraft));
                const appId = DamnId.fromString(appIdString);
                await authorizeAppOwner(appId);
                await appTables.registrations.update({
                    ...find({ appId }),
                    whole: {
                        appId,
                        home: appDraft.home,
                        label: appDraft.label,
                        origins: originsToDatabase(appDraft.origins),
                        archived: false,
                    },
                });
            },
            async deleteApp({ appTables, authorizeAppOwner }, { appId: appIdString }) {
                const appId = DamnId.fromString(appIdString);
                await authorizeAppOwner(appId);
                await appTables.registrations.update({
                    ...find({ appId }),
                    write: { archived: true },
                });
            },
            async listAdmins({ authorizeAppOwner }, { appId: appIdString }) {
                const appId = DamnId.fromString(appIdString);
                const { authTables } = await authorizeAppOwner(appId);
                const usersWithAdminRole = await authTables.permissions.userHasRole
                    .read(find({ roleId: adminRoleId }));
                const adminsViaEmail = await authTables.users.emails
                    .read(find(...usersWithAdminRole.map(({ userId }) => ({ userId }))));
                return adminsViaEmail.map(({ userId, email }) => ({
                    userId: userId.toString(),
                    email,
                }));
            },
            async assignPlatformUserAsAdmin({ authTablesForPlatform, authorizeAppOwner }, { appId: appIdString, platformUserId: platformUserIdString }) {
                const appId = DamnId.fromString(appIdString);
                const { authTables } = await authorizeAppOwner(appId);
                const platformUserId = DamnId.fromString(platformUserIdString);
                const platformAccount = await authTablesForPlatform.users.emails
                    .one(find({ userId: platformUserId }));
                if (!platformAccount)
                    throw new ApiError(404, "platform email account not found");
                const { email } = platformAccount;
                await appointAdmin({
                    rando,
                    config,
                    email,
                    authTables,
                    generateNickname,
                });
            },
            async assignAdmin({ authorizeAppOwner }, { appId: appIdString, email }) {
                const appId = DamnId.fromString(appIdString);
                const { authTables } = await authorizeAppOwner(appId);
                const problems = emailValidator(email);
                if (problems.length)
                    throw new ApiError(400, "email failed validation: " + problems.join(";"));
                await appointAdmin({
                    rando,
                    config,
                    email: email.toLowerCase(),
                    authTables,
                    generateNickname,
                });
            },
            async revokeAdmin({ authorizeAppOwner }, { appId: appIdString, userId: userIdString }) {
                const appId = DamnId.fromString(appIdString);
                const userId = DamnId.fromString(userIdString);
                const { authTables } = await authorizeAppOwner(appId);
                await authTables.permissions.userHasRole.delete(find({
                    userId,
                    roleId: adminRoleId,
                }));
            },
        },
    });

    const profileValidators = {
        nickname: validator$1(string(), minLength(1), maxLength(24), notWhitespace()),
        tagline: validator$1(string(), minLength(0), maxLength(32), notWhitespace()),
    };
    const validateProfileDraft = schema({
        ...profileValidators,
    });

    const makePersonalService = (options) => apiContext()({
        policy: options.authPolicies.userPolicy,
        expose: {
            async setProfile({ access, authTables, checker }, { userId: userIdString, profileDraft }) {
                const userId = DamnId.fromString(userIdString);
                const isProfileOwner = access.user.userId === userIdString;
                const canEditAnyProfile = checker.hasPrivilege("edit any profile");
                const allowed = isProfileOwner || canEditAnyProfile;
                if (!allowed)
                    throw new ApiError(403, "forbidden: you are not allowed to edit this profile");
                throwProblems(validateProfileDraft(profileDraft));
                await authTables.users.profiles.update({
                    ...find({ userId }),
                    write: {
                        nickname: profileDraft.nickname,
                        tagline: profileDraft.tagline,
                    },
                });
            },
        },
    });

    const roleLabelValidator = validator$1(string(), minLength(1), maxLength(16), notWhitespace());

    const makePermissionsService = ({ rando, config, authPolicies, }) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authPolicies.userPolicy(meta, request);
            auth.checker.requirePrivilege("customize permissions");
            const engine = makePermissionsEngine({
                permissionsTables: auth.authTables.permissions,
                isPlatform: auth.access.appId === config.platform.appDetails.appId,
            });
            return { ...auth, engine };
        },
        expose: {
            async fetchPermissions({ engine }) {
                return engine.getPermissionsDisplay();
            },
            async createRole({ authTables }, { label }) {
                const problems = roleLabelValidator(label);
                if (problems.length)
                    throw new ApiError(400, `validation error on label: ${problems.join("; ")}`);
                await authTables.permissions.role.create({
                    label,
                    hard: false,
                    public: true,
                    assignable: true,
                    roleId: rando.randomId(),
                    time: Date.now(),
                });
            },
            async deleteRole({ authTables }, { roleId: roleIdString }) {
                const roleId = DamnId.fromString(roleIdString);
                const role = await authTables.permissions.role.one(find({ roleId }));
                if (!role)
                    throw new ApiError(404, "role not found");
                if (role.hard)
                    throw new ApiError(400, "cannot delete hard role");
                await Promise.all([
                    authTables.permissions.userHasRole.delete(find({ roleId })),
                    authTables.permissions.role.delete(find({ roleId })),
                ]);
            },
            async assignPrivilege({ authTables }, { roleId: roleIdString, privilegeId: privilegeIdString }) {
                const roleId = DamnId.fromString(roleIdString);
                const privilegeId = DamnId.fromString(privilegeIdString);
                await authTables.permissions.roleHasPrivilege.update({
                    ...find({ roleId, privilegeId }),
                    upsert: {
                        roleId,
                        privilegeId,
                        active: true,
                        immutable: false,
                        time: Date.now(),
                    },
                });
            },
            async unassignPrivilege({ authTables }, { roleId: roleIdString, privilegeId: privilegeIdString }) {
                const roleId = DamnId.fromString(roleIdString);
                const privilegeId = DamnId.fromString(privilegeIdString);
                await authTables.permissions.roleHasPrivilege.update({
                    ...find({ roleId, privilegeId }),
                    upsert: {
                        roleId,
                        privilegeId,
                        active: false,
                        immutable: false,
                        time: Date.now(),
                    },
                });
            },
            async createPrivilege({ authTables }, { label }) {
                const privilege = {
                    label,
                    hard: false,
                    privilegeId: rando.randomId(),
                    time: Date.now(),
                };
                await authTables.permissions.privilege.create(privilege);
                return {
                    hard: privilege.hard,
                    label: privilege.label,
                    privilegeId: privilege.privilegeId.toString(),
                    time: Date.now(),
                };
            },
            async deletePrivilege({ engine, authTables }, { privilegeId: privilegeIdString }) {
                const privilegeId = DamnId.fromString(privilegeIdString);
                const [privilege] = await engine.getPrivileges([privilegeId.toString()]);
                if (!privilege)
                    throw new ApiError(400, `cannot delete missing privilege "${privilege.privilegeId.toString()}"`);
                if (privilege.hard)
                    throw new ApiError(400, `cannot delete hard privilege "${privilege.privilegeId.toString()}"`);
                await Promise.all([
                    authTables.permissions
                        .roleHasPrivilege.delete(find({ privilegeId })),
                    authTables.permissions
                        .privilege.delete(find({ privilegeId })),
                ]);
            },
        },
    });

    function authApi(options) {
        return asApi({
            apps: {
                appService: makeAppService(options),
                appEditService: makeAppEditService(options),
            },
            users: {
                greenService: makeGreenService(options),
                loginService: makeLoginService(options),
                userService: makeUserService(options),
                personalService: makePersonalService(options),
            },
            permissions: {
                permissionsService: makePermissionsService(options),
            },
        });
    }

    const goodApiKey = "good";

    const validateDacastApiKey = validator$1(validator$1(string(), minLength(16), maxLength(64), zeroWhitespace()));
    const validateDacastApiKeyAllowingMock = value => value === goodApiKey
        ? []
        : validateDacastApiKey(value);

    function makePrivilegeChecker(permit, privileges) {
        function hasPrivilege(key) {
            return permit.privileges.includes(privileges[key]);
        }
        function requirePrivilege(key) {
            if (!hasPrivilege(key))
                throw new ApiError(403, `forbidden; privilege required "${key}"`);
        }
        function requireNotHavePrivilege(key) {
            if (hasPrivilege(key))
                throw new ApiError(403, `forbidden: must not have privilege "${key}"`);
        }
        return {
            hasPrivilege,
            requirePrivilege,
            requireNotHavePrivilege,
        };
    }

    function toLinkDisplay(secret) {
        return secret
            ? { time: secret.time }
            : undefined;
    }
    const makeDacastService = ({ config, dacastSdk, videoTables: rawVideoTables, basePolicy, }) => apiContext()({
        policy: async (meta, request) => {
            const auth = await basePolicy(meta, request);
            const appId = DamnId.fromString(auth.access.appId);
            const checker = makePrivilegeChecker(auth.access.permit, videoPrivileges);
            checker.requirePrivilege("moderate videos");
            const engine = makePermissionsEngine({
                isPlatform: auth.access.appId === config.platform.appDetails.appId,
                permissionsTables: auth.authTables.permissions,
            });
            return {
                ...auth,
                checker,
                engine,
                videoTables: rawVideoTables.namespaceForApp(appId),
            };
        },
        expose: {
            async getLink({ videoTables }) {
                const secret = await videoTables.dacastAccountLinks.one({
                    conditions: false,
                });
                return toLinkDisplay(secret);
            },
            async setLink({ videoTables }, inputs) {
                const { apiKey } = runValidation(inputs, schema({
                    apiKey: validateDacastApiKeyAllowingMock,
                }));
                const good = await dacastSdk.verifyApiKey(apiKey);
                let secret;
                if (good) {
                    secret = { apiKey, time: Date.now() };
                    await videoTables.dacastAccountLinks.update({
                        conditions: false,
                        upsert: secret,
                    });
                }
                return toLinkDisplay(secret);
            },
            async clearLink({ videoTables }) {
                await videoTables.dacastAccountLinks.delete({
                    conditions: false,
                });
                return undefined;
            },
        },
    });

    function ingestDacastContent({ type, data, }) {
        const pictures = (data.pictures ?? []);
        const picture = pictures[0];
        const thumb = picture
            ? (picture.thumbnail ?? [])[0]
            : undefined;
        return {
            type,
            thumb,
            id: data.id,
            title: data.title,
            provider: "dacast",
        };
    }

    async function getCatalog({ dacast }) {
        const results = await concurrent({
            vods: dacast.vods.get(),
            channels: dacast.channels.get(),
            playlists: dacast.playlists.get(),
        });
        function convert2(type, paginated) {
            return (paginated?.data ?? []).map(data => ingestDacastContent({
                type,
                data,
            }));
        }
        return [
            ...convert2("vod", results.vods),
            ...convert2("channel", results.channels),
            ...convert2("playlist", results.playlists),
        ];
    }

    async function getAllViews({ videoTables }) {
        const viewDacastRows = await videoTables.viewDacast.read({
            conditions: false,
        });
        const viewPrivilegeRows = await videoTables.viewPrivileges.read({
            conditions: false,
        });
        return viewDacastRows.map(dacastRow => {
            const privileges = viewPrivilegeRows
                .filter(privilegeRow => privilegeRow.label === dacastRow.label)
                .map(r => r.privilegeId.toString());
            return {
                id: dacastRow.dacastId,
                label: dacastRow.label,
                privileges,
                provider: "dacast",
                type: dacastRow.type,
            };
        });
    }

    function isPermittedToView({ viewPrivileges, userPrivileges }) {
        for (const requiredPrivilege of viewPrivileges) {
            const userHasPrivilege = userPrivileges
                .find(p => p === requiredPrivilege);
            if (userHasPrivilege)
                return true;
        }
        return false;
    }

    async function getVideoViews({ labels, videoTables, checker, userPrivileges, }) {
        if (labels.length === 0)
            return [];
        const dacastRows = await videoTables.viewDacast.read(findAll(labels, label => ({ label })));
        const privilegeRows = await videoTables.viewPrivileges.read(findAll(labels, label => ({ label })));
        return dacastRows.map(({ label, type, dacastId }) => {
            const privileges = privilegeRows
                .filter(p => p.label === label)
                .map(p => p.privilegeId.toString());
            const hasExplicitPrivilege = isPermittedToView({
                viewPrivileges: privileges,
                userPrivileges,
            });
            const isPermitted = hasExplicitPrivilege
                || checker.hasPrivilege("view all videos");
            return isPermitted
                ? {
                    type,
                    label,
                    privileges,
                    id: dacastId,
                    provider: "dacast",
                }
                : undefined;
        });
    }

    const embedType = "iframe";
    async function getDacastEmbed({ dacast, reference: { id, type }, }) {
        switch (type) {
            case "vod": return dacast.vods.id(id).embed(embedType).get();
            case "channel": return dacast.channels.id(id).embed(embedType).get();
            case "playlist": return dacast.playlists.id(id).embed(embedType).get();
            default: throw new Error(`unknown dacast type "${type}"`);
        }
    }

    async function getDacastApiKey(videoTables) {
        const link = await videoTables.dacastAccountLinks
            .one({ conditions: false });
        return link
            ? link.apiKey
            : undefined;
    }

    async function getAllPrivileges({ access, platformAppId, permissionsTables, }) {
        const permissionsEngine = makePermissionsEngine({
            permissionsTables,
            isPlatform: access.appId === platformAppId
        });
        const permissionsDisplay = await permissionsEngine
            .getPermissionsDisplay();
        return permissionsDisplay.privileges;
    }

    async function getDacastContent({ dacast, reference: { provider, type, id }, }) {
        if (provider !== "dacast")
            throw new ApiError(500, `video content provider mismatch (expected "dacast", got "${provider}")`);
        switch (type) {
            case "vod": return dacast.vods.id(id).get();
            case "channel": return dacast.channels.id(id).get();
            case "playlist": return dacast.playlists.id(id).get();
            default: throw new ApiError(500, `unknown dacast type "${type}"`);
        }
    }

    async function setViewPermissions({ label, engine, videoTables, privileges: privilegesInput, }) {
        // delete all privileges associated with this view
        await videoTables.viewPrivileges.delete(find({ label }));
        // select only privileges that actually exist
        const privileges = await engine.getPrivileges(privilegesInput)
            .then(displays => displays
            .filter(d => !!d)
            .map(d => d.privilegeId.toString()));
        // add new privileges for this view
        await videoTables.viewPrivileges.create(...privileges.map(privilege => ({
            label,
            privilegeId: DamnId.fromString(privilege),
        })));
    }

    const makeContentService = ({ config, dacastSdk, videoTables: rawVideoTables, basePolicy, }) => apiContext()({
        policy: async (meta, request) => {
            const auth = await basePolicy(meta, request);
            const appId = DamnId.fromString(auth.access.appId);
            const checker = makePrivilegeChecker(auth.access.permit, videoPrivileges);
            const engine = makePermissionsEngine({
                isPlatform: auth.access.appId === config.platform.appDetails.appId,
                permissionsTables: auth.authTables.permissions,
            });
            return {
                ...auth,
                engine,
                checker,
                videoTables: rawVideoTables.namespaceForApp(appId),
            };
        },
        expose: {
            async fetchModerationData({ access, authTables, videoTables, checker, }) {
                checker.requirePrivilege("moderate videos");
                return concurrent({
                    views: getAllViews({ videoTables }),
                    privileges: getAllPrivileges({
                        access,
                        permissionsTables: authTables.permissions,
                        platformAppId: config.platform.appDetails.appId,
                    }),
                    catalog: getDacastApiKey(videoTables)
                        .then(async (apiKey) => apiKey
                        ? getCatalog({ dacast: dacastSdk.getClient(apiKey) })
                        : []),
                });
            },
            async writeView({ videoTables, checker, engine }, { label, privileges, reference, }) {
                checker.requirePrivilege("moderate videos");
                await videoTables.viewDacast.update({
                    ...find({ label }),
                    whole: {
                        label,
                        type: reference.type,
                        dacastId: reference.id,
                    },
                });
                await setViewPermissions({
                    label,
                    engine,
                    privileges,
                    videoTables,
                });
            },
            async deleteView({ videoTables, checker }, { label }) {
                checker.requirePrivilege("moderate videos");
                await videoTables.viewDacast.delete(find({ label }));
                await videoTables.viewPrivileges.delete(find({ label }));
            },
            async getShows({ videoTables, access, checker }, { labels }) {
                const apiKey = await getDacastApiKey(videoTables);
                if (!apiKey)
                    return [];
                const views = await getVideoViews({
                    labels,
                    checker,
                    videoTables,
                    userPrivileges: access.permit.privileges,
                });
                const dacast = dacastSdk.getClient(apiKey);
                return Promise.all(views.map(async (view, index) => {
                    const label = labels[index];
                    if (!view)
                        return { label, details: undefined };
                    const [data, embed] = await Promise.all([
                        getDacastContent({ dacast, reference: view }),
                        getDacastEmbed({ dacast, reference: view }),
                    ]);
                    const show = {
                        label,
                        details: {
                            ...ingestDacastContent({
                                type: view.type,
                                data,
                            }),
                            embed: embed.code,
                        },
                    };
                    return show;
                }));
            },
        },
    });

    function videosApi({ config, videoTables, authPolicies, dacastSdk, }) {
        const options = {
            config,
            videoTables,
            basePolicy: authPolicies.anonPolicy,
        };
        return asApi({
            dacastService: makeDacastService({
                ...options,
                dacastSdk,
            }),
            contentService: makeContentService({
                ...options,
                dacastSdk,
            }),
        });
    }

    const makeExampleService = ({ rando, config, authPolicies, exampleTables, }) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authPolicies.userPolicy(meta, request);
            const appId = DamnId.fromString(auth.access.appId);
            return {
                ...auth,
                exampleTables: exampleTables.namespaceForApp(appId),
            };
        },
        expose: {
            async exampleFunction({ access, exampleTables }, { something }) {
                await exampleTables.examplePosts.create({
                    exampleId: rando.randomId(),
                    something,
                });
            }
        },
    });

    function exampleApi(options) {
        return asApi({
            exampleService: makeExampleService(options),
        });
    }

    async function makeVotingBooth({ userId, itemIds, likesTable, reportsTable }) {
        const likes = await makeVoteCounter({ itemIds, voteTable: likesTable });
        const reports = await makeVoteCounter({ itemIds, voteTable: reportsTable });
        return {
            getVotingDetails(itemId) {
                return {
                    likes: likes.countVotes(itemId),
                    reports: reports.countVotes(itemId),
                    liked: userId
                        ? likes.voteStatus({ userId, itemId })
                        : false,
                    reported: userId
                        ? reports.voteStatus({ userId, itemId })
                        : false,
                };
            }
        };
    }
    async function makeVoteCounter({ itemIds, voteTable }) {
        const votes = itemIds.length
            ? await voteTable.read(findAll(itemIds, itemId => ({ itemId })))
            : [];
        return {
            countVotes(itemId) {
                return votes
                    .filter(vote => vote.itemId.toString() === itemId.toString())
                    .length;
            },
            voteStatus({ userId, itemId }) {
                const votesByUser = votes
                    .filter(vote => vote.userId.toString() === userId.toString());
                const userVotedForItem = !!votesByUser
                    .find(vote => vote.itemId.toString() === itemId.toString());
                return userVotedForItem;
            },
        };
    }

    async function resolveQuestions({ userId, questionPosts, questionsTables }) {
        const questionIds = questionPosts.map(post => post.questionId);
        const answerPosts = questionIds.length
            ? await questionsTables.answerPosts
                .read(findAll(questionIds, questionId => ({ questionId, archive: false })))
            : [];
        const answerIds = answerPosts.map(post => post.answerId);
        const boothDetails = {
            userId,
            likesTable: questionsTables.likes,
            reportsTable: questionsTables.reports,
        };
        const questionsVotingBooth = await makeVotingBooth({
            ...boothDetails,
            itemIds: questionIds,
        });
        const answersVotingBooth = await makeVotingBooth({
            ...boothDetails,
            itemIds: answerIds,
        });
        return questionIds.map(questionId => {
            const questionPost = questionPosts.find(post => post.questionId === questionId);
            const question = {
                archive: questionPost.archive,
                authorUserId: questionPost.authorUserId.toString(),
                questionId: questionPost.questionId.toString(),
                board: questionPost.board,
                content: questionPost.content,
                timePosted: questionPost.timePosted,
                ...questionsVotingBooth.getVotingDetails(questionId),
                answers: answerPosts
                    .filter(answer => answer.questionId.toString() === questionId.toString())
                    .map(answerPost => ({
                    answerId: answerPost.answerId.toString(),
                    questionId: answerPost.questionId.toString(),
                    authorUserId: answerPost.authorUserId.toString(),
                    content: answerPost.content,
                    archive: answerPost.archive,
                    timePosted: answerPost.timePosted,
                    ...answersVotingBooth.getVotingDetails(answerPost.answerId),
                })),
            };
            return question;
        });
    }

    async function spikeQuestionsAuth(meta, request, questionsTables, basePolicy) {
        const auth = await basePolicy(meta, request);
        const appId = DamnId.fromString(auth.access.appId);
        return {
            ...auth,
            questionsTables: questionsTables.namespaceForApp(appId),
        };
    }

    function anonQuestionsPolicy({ authPolicies, questionsTables, }) {
        return async function (meta, request) {
            return spikeQuestionsAuth(meta, request, questionsTables, authPolicies.anonPolicy);
        };
    }

    const makeQuestionsReadingService = (options) => apiContext()({
        policy: async (meta, request) => {
            const auth = await anonQuestionsPolicy(options)(meta, request);
            auth.checker.requirePrivilege("read questions");
            return auth;
        },
        expose: {
            async fetchQuestions({ access, authTables, questionsTables }, { board }) {
                const posts = await questionsTables.questionPosts.read({
                    ...find({ board, archive: false }),
                    limit: 100,
                    offset: 0,
                    order: { timePosted: "descend" },
                });
                if (!posts.length)
                    return { users: [], questions: [] };
                const resolvedQuestions = await resolveQuestions({
                    questionsTables,
                    questionPosts: posts,
                    userId: access?.user?.userId
                        ? DamnId.fromString(access.user.userId)
                        : undefined,
                });
                const userIds = (() => {
                    const ids = [];
                    const rememberUserId = (userId) => {
                        if (!ids.includes(userId))
                            ids.push(userId);
                    };
                    for (const question of resolvedQuestions) {
                        rememberUserId(question.authorUserId);
                        for (const answer of question.answers)
                            rememberUserId(answer.authorUserId);
                    }
                    return ids;
                })();
                const permissionsEngine = makePermissionsEngine({
                    permissionsTables: authTables.permissions,
                    isPlatform: access.appId === options.config.platform.appDetails.appId,
                });
                const bannedUserIds = (await permissionsEngine.getPrivilegesForUsers(userIds))
                    .filter(p => p.privileges.includes(appPermissions.privileges["banned"]))
                    .map(p => p.userId);
                const users = (await fetchUsers({
                    authTables,
                    permissionsEngine,
                    userIds: userIds.map(id => DamnId.fromString(id)),
                }))
                    .filter(u => !bannedUserIds.includes(u.userId));
                const questions = resolvedQuestions
                    .filter(q => !bannedUserIds.includes(q.authorUserId.toString()))
                    .map(q => ({
                    ...q,
                    answers: q.answers.filter(a => !bannedUserIds.includes(a.authorUserId))
                }));
                return { users, questions };
            },
        },
    });

    async function vote({ status, userId, itemId, voteTable }) {
        const previousVote = await voteTable.count(find({ userId, itemId }))
            > 0;
        const addVote = () => voteTable.create({ userId, itemId });
        const removeVote = () => voteTable.delete(find({ userId, itemId }));
        if (!previousVote && status)
            await addVote();
        else if (previousVote && !status)
            await removeVote();
    }

    const timeframe = 1 * day;
    const allowedNumberOfPostsInTimeframe = 10;
    async function rateLimitQuestions({ userId, questionsTables }) {
        const count = await questionsTables.questionPosts.count({
            conditions: and({ equal: { authorUserId: userId, archive: false } }, { greater: { timePosted: Date.now() - timeframe } })
        });
        if (count > allowedNumberOfPostsInTimeframe)
            throw new ApiError(429, "too many posts");
    }
    async function rateLimitAnswers({ userId, questionId, questionsTables }) {
        const count = await questionsTables.answerPosts.count({
            conditions: and({ equal: { authorUserId: userId, questionId, archive: false } }, { greater: { timePosted: Date.now() - timeframe } })
        });
        if (count > allowedNumberOfPostsInTimeframe)
            throw new ApiError(429, "too many posts");
    }

    const validatePostContent = validator$1(string(), minLength(10), maxLength(280), notWhitespace());
    const validateQuestionDraft = schema({
        board: validator$1(string(), minLength(1), maxLength(32), notWhitespace()),
        content: validatePostContent,
    });
    const validateAnswerDraft = schema({
        content: validatePostContent,
    });

    function requireUserCanEditQuestion({ userId, questionPost, checker }) {
        const userIsModerator = checker.hasPrivilege("moderate questions");
        const userIsOwner = questionPost.authorUserId.toString() === userId;
        const isAllowed = userIsModerator || userIsOwner;
        if (!isAllowed)
            throw new Error(`user is not authorized to edit question`);
    }

    const validateId = validator$1(string(), length(64), regex(/^[0-9a-z]{64}$/));

    function authenticatedQuestionsPolicy({ authPolicies, questionsTables, }) {
        return async function (meta, request) {
            return spikeQuestionsAuth(meta, request, questionsTables, authPolicies.userPolicy);
        };
    }

    const makeQuestionsPostingService = (options) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authenticatedQuestionsPolicy(options)(meta, request);
            auth.checker.requireNotHavePrivilege("banned");
            const canPostQuestions = auth.checker.hasPrivilege("post questions");
            const isModerator = auth.checker.hasPrivilege("moderate questions");
            const allowed = canPostQuestions || isModerator;
            if (!allowed)
                throw new ApiError(403, "not allowed to post questions");
            return auth;
        },
        expose: {
            async postQuestion({ access, questionsTables }, inputs) {
                const { questionDraft } = runValidation(inputs, schema({
                    questionDraft: validateQuestionDraft,
                }));
                await rateLimitQuestions({
                    questionsTables,
                    userId: DamnId.fromString(access.user.userId),
                });
                const row = {
                    questionId: options.rando.randomId(),
                    authorUserId: DamnId.fromString(access.user.userId),
                    archive: false,
                    timePosted: Date.now(),
                    ...questionDraft,
                };
                await questionsTables.questionPosts.create(row);
                return {
                    archive: row.archive,
                    authorUserId: row.authorUserId.toString(),
                    questionId: row.questionId.toString(),
                    content: row.content,
                    timePosted: row.timePosted,
                    board: row.board,
                    likes: 0,
                    reports: 0,
                    liked: false,
                    reported: false,
                    answers: [],
                };
            },
            async archiveQuestion({ access: { user: { userId } }, questionsTables, checker }, inputs) {
                const { archive, questionId: questionIdString } = runValidation(inputs, schema({
                    archive: boolean(),
                    questionId: validateId,
                }));
                const questionId = DamnId.fromString(questionIdString);
                const questionPost = await questionsTables.questionPosts
                    .one(find({ questionId }));
                requireUserCanEditQuestion({ userId, checker, questionPost });
                await questionsTables.questionPosts.update({
                    ...find({ questionId }),
                    write: { archive: !!archive },
                });
            },
            async likeQuestion({ questionsTables, checker, access: { user: { userId: userIdString } } }, inputs) {
                const { like, questionId: questionIdString } = runValidation(inputs, schema({
                    like: boolean(),
                    questionId: validateId,
                }));
                checker.requirePrivilege("like questions");
                await vote({
                    status: like,
                    voteTable: questionsTables.likes,
                    userId: DamnId.fromString(userIdString),
                    itemId: DamnId.fromString(questionIdString),
                });
            },
            async reportQuestion({ questionsTables, checker, access: { user: { userId: userIdString } } }, inputs) {
                const { report, questionId: questionIdString } = runValidation(inputs, schema({
                    report: boolean(),
                    questionId: validateId,
                }));
                checker.requirePrivilege("report questions");
                await vote({
                    status: report,
                    voteTable: questionsTables.reports,
                    userId: DamnId.fromString(userIdString),
                    itemId: DamnId.fromString(questionIdString),
                });
            },
        },
    });

    function requireUserCanEditAnswer({ userId, answerPost, checker }) {
        const userIsModerator = checker.hasPrivilege("moderate questions");
        const userIsOwner = answerPost.authorUserId.toString() === userId;
        const isAllowed = userIsModerator || userIsOwner;
        if (!isAllowed)
            throw new Error(`user is not authorized to edit answer`);
    }

    const makeQuestionsAnsweringService = (options) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authenticatedQuestionsPolicy(options)(meta, request);
            auth.checker.requireNotHavePrivilege("banned");
            const canPostQuestions = auth.checker.hasPrivilege("answer questions");
            const isModerator = auth.checker.hasPrivilege("moderate questions");
            const allowed = canPostQuestions || isModerator;
            if (!allowed)
                throw new ApiError(403, "not allowed to post questions");
            return auth;
        },
        expose: {
            async postAnswer({ access, questionsTables }, inputs) {
                const { questionId: questionIdString, answerDraft } = runValidation(inputs, schema({
                    questionId: validateId,
                    answerDraft: validateAnswerDraft,
                }));
                const questionId = DamnId.fromString(questionIdString);
                const questionPost = await questionsTables.questionPosts
                    .one(find({ questionId }));
                if (!questionPost)
                    throw new ApiError(400, "unknown questionId");
                const userId = DamnId.fromString(access.user.userId);
                await rateLimitAnswers({
                    userId,
                    questionId,
                    questionsTables,
                });
                const row = {
                    questionId,
                    answerId: options.rando.randomId(),
                    authorUserId: userId,
                    archive: false,
                    timePosted: Date.now(),
                    board: questionPost.board,
                    ...answerDraft,
                };
                await questionsTables.answerPosts.create(row);
                const answer = {
                    answerId: row.answerId.toString(),
                    questionId: row.questionId.toString(),
                    authorUserId: row.authorUserId.toString(),
                    content: row.content,
                    timePosted: row.timePosted,
                    liked: false,
                    likes: 0,
                    archive: false,
                    reported: false,
                    reports: 0,
                };
                return answer;
            },
            async archiveAnswer({ access: { user: { userId } }, checker, questionsTables }, inputs) {
                const { archive, answerId: answerIdString } = runValidation(inputs, schema({
                    archive: boolean(),
                    answerId: validateId,
                }));
                const answerId = DamnId.fromString(answerIdString);
                const answerPost = await questionsTables.answerPosts.one(find({ answerId }));
                requireUserCanEditAnswer({ userId, checker, answerPost });
                await questionsTables.answerPosts.update({
                    ...find({ answerId }),
                    write: { archive },
                });
            },
            async likeAnswer({ questionsTables, checker, access: { user: { userId: userIdString } } }, inputs) {
                const { like, answerId: answerIdString } = runValidation(inputs, schema({
                    like: boolean(),
                    answerId: validateId,
                }));
                checker.requirePrivilege("like questions");
                await vote({
                    status: like,
                    voteTable: questionsTables.likes,
                    userId: DamnId.fromString(userIdString),
                    itemId: DamnId.fromString(answerIdString),
                });
            },
            async reportAnswer({ questionsTables, checker, access: { user: { userId: userIdString } } }, inputs) {
                const { report, answerId: answerIdString } = runValidation(inputs, schema({
                    report: boolean(),
                    answerId: validateId,
                }));
                checker.requirePrivilege("report questions");
                await vote({
                    status: report,
                    voteTable: questionsTables.reports,
                    userId: DamnId.fromString(userIdString),
                    itemId: DamnId.fromString(answerIdString),
                });
            },
        },
    });

    const makeQuestionsModerationService = (options) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authenticatedQuestionsPolicy(options)(meta, request);
            auth.checker.requirePrivilege("moderate questions");
            auth.checker.requireNotHavePrivilege("banned");
            return auth;
        },
        expose: {
            async archiveBoard({ questionsTables }, { board }) {
                await questionsTables.questionPosts.update({
                    ...find({ board }),
                    write: { archive: true },
                });
            },
            // TODO implement
            async fetchReportedQuestions() { },
        },
    });

    function questionsApi(options) {
        return asApi({
            questionsReadingService: makeQuestionsReadingService(options),
            questionsPostingService: makeQuestionsPostingService(options),
            questionsAnsweringService: makeQuestionsAnsweringService(options),
            questionsModerationService: makeQuestionsModerationService(options),
        });
    }

    const validateShowLabel = validator$1(string(), minLength(1), maxLength(64));
    const validateVimeoId = validator$1(string(), minLength(1), maxLength(64), regex(/^\d+$/, "invalid vimeo id"));
    const validateVimeoUrl = validator$1(string(), maxLength(2048), url(), value => {
        const url = new URL(value);
        return url.origin.toLowerCase() === "https://vimeo.com"
            ? []
            : ["invalid vimeo url (wrong origin)"];
    });
    const isUrl = (value) => url()(value).length === 0;
    const validateVimeoField = validator$1(string(), maxLength(2048), value => value === ""
        ? []
        : isUrl(value)
            ? validateVimeoUrl(value)
            : validateVimeoId(value));
    const validateShowInput = schema({
        label: validateShowLabel,
        vimeoId: (value) => value === ""
            ? []
            : validateVimeoId(value),
    });

    const makeLivestreamViewingService = ({ authPolicies, livestreamTables, }) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authPolicies.userPolicy(meta, request);
            auth.checker.requirePrivilege("view livestream");
            const appId = DamnId.fromString(auth.access.appId);
            return {
                ...auth,
                livestreamTables: livestreamTables.namespaceForApp(appId),
            };
        },
        expose: {
            async getShows({ livestreamTables }, inputs) {
                if (inputs.labels.length === 0)
                    return [];
                const { labels } = runValidation(inputs, schema({
                    labels: validator$1(array(), minLength(1), maxLength(100), each(validateShowLabel)),
                }));
                const shows = await livestreamTables.shows.read(findAll(labels, label => ({ label })));
                return labels.map(label => {
                    return shows.find(show => show.label === label)
                        ?? { label, vimeoId: "" };
                });
            }
        },
    });

    const makeLivestreamModerationService = ({ authPolicies, livestreamTables, }) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authPolicies.userPolicy(meta, request);
            auth.checker.requirePrivilege("moderate livestream");
            const appId = DamnId.fromString(auth.access.appId);
            return {
                ...auth,
                livestreamTables: livestreamTables.namespaceForApp(appId),
            };
        },
        expose: {
            async setShow({ livestreamTables }, inputs) {
                const { label, vimeoId } = runValidation(inputs, validateShowInput);
                await livestreamTables.shows.update({
                    ...find({ label }),
                    write: { label, vimeoId },
                });
            }
        },
    });

    function livestreamApi(options) {
        return asApi({
            livestreamViewingService: makeLivestreamViewingService(options),
            livestreamModerationService: makeLivestreamModerationService(options),
        });
    }

    function isOriginValid(request, origins) {
        return !!request.headers.origin
            && origins.some(origin => origin === request.headers.origin);
    }

    function prepareStatsHub({ appTables, authTables }) {
        return async function getStatsHub(userId) {
            async function throwForbiddenUser(appId) {
                const row = await appTables.owners.one(find({ appId }));
                if (row.userId.toString() !== userId.toString())
                    throw new ApiError(403, "forbidden");
            }
            return {
                countUsers: async (appId) => {
                    await throwForbiddenUser(appId);
                    return await authTables.namespaceForApp(appId).users.accounts.count({ conditions: false });
                },
                countUsersActiveDaily: async (appId) => {
                    await throwForbiddenUser(appId);
                    const timeToStartCounting = Date.now() - day;
                    return authTables.namespaceForApp(appId).users.latestLogins.count({
                        conditions: and({ greater: { time: timeToStartCounting } }),
                    });
                },
                countUsersActiveMonthly: async (appId) => {
                    await throwForbiddenUser(appId);
                    const timeToStartCounting = Date.now() - month;
                    return authTables.namespaceForApp(appId).users.latestLogins.count({
                        conditions: and({ greater: { time: timeToStartCounting } }),
                    });
                },
            };
        };
    }

    async function isUserOwnerOfApp({ appId, access, appTables }) {
        const { userId: userIdString } = access.user;
        const userId = DamnId.fromString(userIdString);
        const ownershipRow = await appTables.owners.one(find({ userId, appId }));
        return !!ownershipRow;
    }

    function prepareAuthPolicies({ config, appTables, authTables, verifyToken, }) {
        const getStatsHub = prepareStatsHub({ appTables, authTables });
        const greenPolicy = async (meta, request) => ({
            appTables,
            authTables,
        });
        const anonPolicy = async ({ accessToken }, request) => {
            const access = await verifyToken(accessToken);
            if (isOriginValid(request, access.origins))
                return {
                    access,
                    appTables,
                    authTables: authTables.namespaceForApp(DamnId.fromString(access.appId)),
                    checker: makePrivilegeChecker(access.permit, appPermissions.privileges),
                };
            else
                throw new ApiError(403, "invalid origin");
        };
        const userPolicy = async (meta, request) => {
            const auth = await anonPolicy(meta, request);
            if (auth.access.user)
                return auth;
            else
                throw new ApiError(403, "not logged in");
        };
        const platformUserPolicy = async (meta, request) => {
            const auth = await userPolicy(meta, request);
            if (auth.access.appId === config.platform.appDetails.appId) {
                return {
                    ...auth,
                    appTables,
                    authTables,
                    checker: makePrivilegeChecker(auth.access.permit, platformPermissions.privileges),
                    statsHub: await getStatsHub(DamnId.fromString(auth.access.user.userId)),
                };
            }
            else
                throw new ApiError(403, "not platform app");
        };
        const appOwnerPolicy = async (meta, request) => {
            const auth = await platformUserPolicy(meta, request);
            async function authorizeAppOwner(appId) {
                const allowedToEditAnyApp = auth.checker.hasPrivilege("edit any app");
                const isOwnerOfApp = isUserOwnerOfApp({ appId, appTables, access: auth.access });
                const allowed = isOwnerOfApp || allowedToEditAnyApp;
                if (allowed)
                    return { authTables: auth.authTables.namespaceForApp(appId) };
                else
                    throw new ApiError(403, "forbidden: lacking privileges to edit app");
            }
            return {
                access: auth.access,
                checker: auth.checker,
                statsHub: auth.statsHub,
                appTables: auth.appTables,
                authTablesForPlatform: auth.authTables.namespaceForApp(DamnId.fromString(config.platform.appDetails.appId)),
                authorizeAppOwner,
            };
        };
        return {
            greenPolicy,
            anonPolicy,
            userPolicy,
            platformUserPolicy,
            appOwnerPolicy,
        };
    }

    function escapeRegex(subject) {
        return subject.replace(/[-\/\\^$*+?.()|[\]{}]/g, "\\$&");
    }

    const validateTimeframe = branch(notDefined(), number());

    const validateUserSearchTerm = validator$1(string(), minLength(1), maxLength(64));

    const makeRoleAssignmentService = ({ config, authPolicies, }) => apiContext()({
        policy: async (meta, request) => {
            const auth = await authPolicies.userPolicy(meta, request);
            auth.checker.requirePrivilege("administrate user roles");
            const engine = makePermissionsEngine({
                permissionsTables: auth.authTables.permissions,
                isPlatform: auth.access.appId === config.platform.appDetails.appId,
            });
            return { ...auth, engine };
        },
        expose: {
            async fetchPermissions({ engine }) {
                return engine.getPermissionsDisplay();
            },
            async searchUsers({ access, engine, authTables }, options) {
                const { term } = runValidation(options, schema({
                    term: validateUserSearchTerm,
                }));
                const regex = new RegExp(escapeRegex(term), "i");
                const profiles = await authTables.users.profiles.read({
                    limit: 100,
                    conditions: DamnId.isId(term)
                        ? or({ equal: { userId: DamnId.fromString(term) } })
                        : or({ search: { nickname: regex } }, { search: { tagline: regex } })
                });
                const userIds = profiles.map(profile => profile.userId);
                if (!userIds.length)
                    return [];
                const users = await fetchUsers({
                    userIds,
                    authTables,
                    permissionsEngine: engine,
                });
                const usersAndRoles = await engine.getUsersHaveRoles({
                    userIds: users.map(user => user.userId),
                    onlyGetPublicRoles: false,
                });
                return users.map(user => ({
                    user,
                    roleIds: usersAndRoles
                        .find(u => u.userId === user.userId)
                        .userHasRoles
                        .map(role => role.roleId.toString())
                }));
            },
            async assignRoleToUser({ authTables }, options) {
                const { roleId: roleIdString, userId: userIdString, isPublic, timeframeEnd, timeframeStart } = (runValidation(options, schema({
                    roleId: validateId,
                    userId: validateId,
                    isPublic: boolean(),
                    timeframeEnd: validateTimeframe,
                    timeframeStart: validateTimeframe,
                })));
                const roleId = DamnId.fromString(roleIdString);
                const userId = DamnId.fromString(userIdString);
                const existing = await authTables.permissions.userHasRole.one(find({
                    userId,
                    roleId,
                }));
                if (existing?.hard)
                    throw new ApiError(400, "hard role assignment cannot be overwritten");
                else
                    await authTables.permissions.userHasRole.assert({
                        conditions: or({ equal: { roleId, userId } }),
                        make: async () => ({
                            hard: false,
                            public: isPublic,
                            roleId,
                            userId,
                            timeframeEnd,
                            timeframeStart,
                            time: Date.now(),
                        }),
                    });
            },
            async revokeRoleFromUser({ authTables }, options) {
                const { roleId: roleIdString, userId: userIdString } = runValidation(options, schema({
                    roleId: validateId,
                    userId: validateId,
                }));
                const roleId = DamnId.fromString(roleIdString);
                const userId = DamnId.fromString(userIdString);
                const existing = await authTables.permissions.userHasRole.one(find({
                    userId,
                    roleId,
                }));
                if (existing?.hard)
                    throw new ApiError(400, "hard role assignment cannot be overwritten");
                else
                    await authTables.permissions.userHasRole.delete({
                        conditions: or({ equal: { roleId, userId } }),
                    });
            },
        },
    });

    function makeAdministrativeApi(options) {
        return asApi({
            roleAssignmentService: makeRoleAssignmentService(options),
        });
    }

    const colors = [
        "Red",
        "Yellow",
        "Green",
        "Cyan",
        "Blue",
        "Magenta",
        "Dark",
        "Grey",
        "Silver",
        "Bright",
        "Light",
        "Gold",
        "Golden",
        "Aqua",
        "Minty",
    ];

    const animals = [
        "Aardvark",
        "Mongoose",
        "Alligator",
        "Robin",
        "Monkey",
        "Seahorse",
        "Shark",
        "Leopard",
        "Eagle",
        "Raven",
        "Owl",
        "Hawk",
        "Duck",
        "Wolf",
        "Coyote",
        "Dingo",
        "Ostrich",
        "Llama",
        "Alpaca",
        "Zebra",
        "Bear",
        "Kiwi",
        "Penguin",
        "Otter",
        "Dolphin",
        "Starfish",
        "Sunfish",
        "Salmon",
        "Swordfish",
        "Sparrow",
        "Turtle",
        "Puppy",
        "Kitten",
    ];

    const adjectives = [
        "Funny",
        "Charming",
        "Wonderful",
        "Fancy",
        "Whimsical",
        "Gregarious",
        "Curious",
        "Fabulous",
        "Saavy",
        "Groovy",
        "Happy",
        "Snappy",
        "Quick",
        "Super",
        "Fast",
        "Fuzzy",
        "Clever",
        "Genius",
        "Marvelous",
        "Eccentric",
        "Fantastic",
        "Quaint",
        "Young",
        "Wise",
        "Creative",
        "Helpful",
        "Nice",
        "Sincere",
        "Loyal",
        "Honest",
        "Truthful",
        "Thoughtful",
        "Clean",
        "Friendly",
        "Gentle",
        "Pleasant",
        "Brilliant",
        "Polite",
        "Original",
        "Ambitious",
        "Lively",
        "Wholesome",
        "Cordial",
        "Eager",
        "Relaxed",
        "Skilled",
        "Tidy",
        "Calm",
        "Hopeful",
        "Positive",
        "Social",
        "Quick",
        "Fearless",
        "Daring",
        "Bold",
        "Quiet",
    ];

    const standardDictionary = {
        colors,
        animals,
        adjectives,
    };

    function makeNicknameGenerator({ rando, delimiter, dictionary, nicknameStructure, }) {
        const nicknameData = nicknameStructure.map(dictionarySet => dictionarySet.reduce((previous, wordGroupName) => [
            ...previous,
            ...dictionary[wordGroupName],
        ], []));
        return () => nicknameData
            .map(names => rando.randomSample(names))
            .join(delimiter);
    }

    function standardNicknameGenerator({ rando }) {
        return makeNicknameGenerator({
            rando,
            delimiter: " ",
            dictionary: standardDictionary,
            nicknameStructure: [["adjectives", "colors"], ["animals"]],
        });
    }

    async function assimilateApi({ config, rando, database, dacastSdk, sendLoginEmail, signToken, verifyToken, }) {
        const generateNickname = standardNicknameGenerator({ rando });
        const authPolicies = prepareAuthPolicies({
            config,
            verifyToken,
            appTables: database.apps,
            authTables: database.auth,
        });
        return asApi({
            auth: authApi({
                rando,
                config,
                authPolicies,
                signToken,
                verifyToken,
                sendLoginEmail,
                generateNickname,
            }),
            administrative: makeAdministrativeApi({
                config,
                authPolicies,
            }),
            questions: questionsApi({
                rando,
                config,
                authPolicies,
                questionsTables: database.questions,
            }),
            example: exampleApi({
                rando,
                config,
                authPolicies,
                exampleTables: database.example,
            }),
            livestream: livestreamApi({
                authPolicies,
                livestreamTables: database.livestream,
            }),
            videos: videosApi({
                config,
                dacastSdk,
                authPolicies,
                videoTables: database.videos,
            }),
        });
    }

    function loginEmailRecaller(sendLoginEmail) {
        let latestLoginEmail;
        return {
            sendLoginEmail: async (details) => {
                latestLoginEmail = details;
                await sendLoginEmail(details);
            },
            recallLatestLoginEmail: () => latestLoginEmail,
        };
    }

    const mockSendEmail = async (email) => console.log(`

====== EMAIL ======
from: ${email.fromLabel}
to: ${email.to}
subject: ${email.subject}
time: ${new Date().toLocaleString()}

${email.text}

===================

`);

    function makeEmailEnabler(actuallySendEmail) {
        let enabled = true;
        const sendEmail = async (details) => {
            if (enabled)
                return actuallySendEmail(details);
        };
        return {
            sendEmail,
            enableEmails() { enabled = true; },
            disableEmails() { enabled = false; },
        };
    }

    function makeLoginLink({ home, loginToken }) {
        const url = new URL(home);
        url.hash = `#login=${loginToken}`;
        return url;
    }

    function escapeHtml(text) {
        return text
            .replace(/&/g, "&amp;")
            .replace(/</g, "&lt;")
            .replace(/>/g, "&gt;")
            .replace(/"/g, "&quot;")
            .replace(/'/g, "&#039;");
    }

    function noop(strings, ...keys) {
        const lastIndex = strings.length - 1;
        return strings
            .slice(0, lastIndex)
            .reduce((a, b, c) => a + b + keys[c], "")
            + strings[lastIndex];
    }

    function prepareSendLoginEmail({ sendEmail }) {
        return async function sendLoginEmail({ to, appHome, appLabel, lifespan, legalLink, loginToken, platformLink, }) {
            const loginLink = makeLoginLink({ loginToken, home: appHome });
            const minutesLeft = Math.round((lifespan / minute)).toFixed(0);
            const safeLabel = escapeHtml(appLabel);
            const platformHost = new URL(platformLink).host;
            return sendEmail({
                to,
                fromLabel: appLabel,
                subject: `Login link`,
                html: (noop `
<h1>Login link for ${safeLabel}</h1>
<p style="font-size: 1.2em; padding-left: 0.5rem;">
	<a href="${loginLink.toString()}">» Login to ${safeLabel} «</a>
</p>
<div style="opacity: 0.7; font-size: 0.8em; margin-top: 2rem;">
	<p>Expires in ${minutesLeft} minutes.</p>
	<p>If you didn't request this login link, just ignore it.</p>
	<p>You can contact support by replying to this email.</p>
	<ul style="padding-left: 0.5rem;">
		<li>Logins powered by <a href="${platformLink}">${platformHost}</a></li>
		<li><a href="${legalLink}">Policies and terms of service</a></li>
	</ul>
	<p style="opacity: 0.3; font-size: 0.4em;">
		${Date.now()}
	</p>
</div>
`),
                text: (`
Login link for ${appLabel}

  » ${loginLink.toString()}


Expires in ${minutesLeft} minutes

If you didn't request this login link, just ignore it.

You can contact support by replying to this email.

  * Logins powered by ${platformLink}

  * Policies and terms of service ${legalLink}

${Date.now()}
`),
            });
        };
    }

    function assimilateEmails({ config, configureMailgun, }) {
        let sendEmail;
        if (config.email === "mock-console") {
            sendEmail = mockSendEmail;
        }
        else {
            sendEmail = configureMailgun(config.email).sendEmail;
        }
        const enabler = makeEmailEnabler(sendEmail);
        sendEmail = enabler.sendEmail;
        const { disableEmails, enableEmails } = enabler;
        const { sendLoginEmail, recallLatestLoginEmail } = loginEmailRecaller(prepareSendLoginEmail({ sendEmail }));
        return {
            sendEmail,
            enableEmails,
            disableEmails,
            sendLoginEmail,
            recallLatestLoginEmail,
        };
    }

    const tokenize = (object) => (url$1.encode(JSON.stringify(object)));
    function mockSignToken() {
        return async ({ payload, lifespan, }) => {
            const data = {
                payload,
                iat: Date.now() / 1000,
                exp: (Date.now() + lifespan) / 1000,
            };
            const header = {
                alg: null,
                typ: "JWT",
            };
            const signature = "fake-mock-token-signature";
            return [
                tokenize(header),
                tokenize(data),
                tokenize(signature),
            ].join(".");
        };
    }

    const mockVerifyToken = () => async (token) => {
        const decoded = tokenDecode(token);
        const expiry = decoded.data.exp * 1000;
        if (Date.now() > expiry)
            throw new Error("token expired");
        return decoded.data.payload;
    };

    function assimilateCrypto({ config, configureTokenFunctions, }) {
        if (config.crypto.keys === "mock-mode") {
            return {
                signToken: mockSignToken(),
                verifyToken: mockVerifyToken(),
            };
        }
        else {
            return configureTokenFunctions(config.crypto.keys);
        }
    }

    function stripeWebhooks({ logger }) {
        return {
            async "checkout.session.completed"(event) {
                logger.info("stripe-webhook checkout.session.completed:", event.data.object);
            },
            async "invoice.paid"(event) {
                logger.info("stripe-webhook invoice.paid:", event.data.object);
            },
            async "invoice.payment_failed"(event) {
                logger.info("stripe-webhook invoice.payment_failed:", event.data.object);
            },
            async "customer.subscription.updated"(event) {
                logger.info("stripe-webhook customer.subscription.updated:", event.data.object);
            },
        };
    }

    function dbbyConstrain({ table, namespace, }) {
        const spike = (conditionTree) => (conditionTree
            ? and({ equal: namespace }, conditionTree)
            : and({ equal: namespace }));
        return {
            [_dbbyTableSymbol]: true,
            async create(...rows) {
                return table.create(...rows.map(row => ({ ...row, ...namespace })));
            },
            async read(options) {
                return table.read({
                    ...options,
                    conditions: spike(options.conditions),
                });
            },
            async one(options) {
                return table.one({
                    ...options,
                    conditions: spike(options.conditions),
                });
            },
            async assert(options) {
                return table.assert({
                    ...options,
                    conditions: spike(options.conditions),
                    make: async () => {
                        const row = await options.make();
                        return { ...row, ...namespace };
                    },
                });
            },
            async update(options) {
                const opts = options;
                return table.update({
                    ...options,
                    conditions: spike(options.conditions),
                    upsert: opts.upsert
                        ? { ...opts.upsert, ...namespace }
                        : undefined,
                    whole: opts.whole
                        ? { ...opts.whole, ...namespace }
                        : undefined,
                    write: opts.write
                        ? { ...opts.write, ...namespace }
                        : undefined,
                });
            },
            async delete(options) {
                return table.delete({
                    ...options,
                    conditions: spike(options.conditions),
                });
            },
            async count(options) {
                return table.count({
                    ...options,
                    conditions: spike(options.conditions),
                });
            },
        };
    }
    function dbbyConstrainTables({ namespace, tables, }) {
        return objectMap(tables, value => {
            return isDbbyTable(value)
                ? dbbyConstrain({ namespace, table: value })
                : dbbyConstrainTables({ namespace, tables: value });
        });
    }

    function mockStripeComplex({ rando, tables, webhooks }) {
        const generateId = () => rando.randomId();
        function respond(resource) {
            return {
                headers: {},
                lastResponse: undefined,
                ...resource,
            };
        }
        const stripeLiaisonForPlatform = {
            accounts: {
                async create(params) {
                    const account = {
                        id: generateId().toString(),
                        type: params.type,
                        email: params.email,
                    };
                    await tables.accounts.create(account);
                    return respond(account);
                },
                async retrieve(id) {
                    const account = await tables.accounts.one(find({ id }));
                    return respond(account);
                },
            },
            accountLinks: {
                async create(params) {
                    const accountLink = {
                        url: `https://fake.xiome.io/stripe-account-setup`,
                    };
                    return respond(accountLink);
                },
            },
        };
        async function webhookEvent(type, object) {
            return webhooks[type]({
                type,
                data: { object },
            });
        }
        const rawTables = tables;
        function connectStripeLiaisonForApp(stripeConnectAccountId) {
            const tables = dbbyConstrainTables({
                tables: rawTables,
                namespace: { "_connectedAccount": stripeConnectAccountId },
            });
            function ignoreUndefined(input) {
                const output = {};
                for (const [key, value] of Object.entries(input)) {
                    if (value !== undefined)
                        output[key] = value;
                }
                return output;
            }
            function prepMockResource(table) {
                return {
                    create({ makeData, hook = async () => { } }) {
                        return async function (params) {
                            const resource = {
                                id: generateId(),
                                ...makeData(params),
                            };
                            await table.create(resource);
                            await hook(resource);
                            return respond(resource);
                        };
                    },
                    retrieve() {
                        return async function (id) {
                            const resource = await table.one(find({ id }));
                            return respond(resource);
                        };
                    },
                    update({ makeData, hook = async () => { } }) {
                        return async function (id, params) {
                            await table.update({
                                ...find({ id }),
                                write: ignoreUndefined(makeData(params)),
                            });
                            const resource = await table.one(find({ id }));
                            await hook(resource);
                            return respond(resource);
                        };
                    },
                    delete() {
                        return async function (id) {
                            await table.delete(find({ id }));
                        };
                    },
                };
            }
            function mockResource({ table, createData, updateData, createHook, updateHook, }) {
                const context = prepMockResource(table);
                return {
                    create: context.create({
                        makeData: createData,
                        hook: createHook,
                    }),
                    retrieve: context.retrieve(),
                    update: context.update({
                        makeData: updateData,
                        hook: updateHook,
                    }),
                    delete: context.delete(),
                };
            }
            return {
                customers: mockResource({
                    table: tables.customers,
                    createData: params => ({
                        email: params.email,
                        invoice_settings: params.invoice_settings
                            ?? { default_payment_method: undefined },
                    }),
                    updateData: params => ({
                        email: params.email,
                        invoice_settings: params.invoice_settings,
                    }),
                }),
                products: mockResource({
                    table: tables.products,
                    createData: params => ({
                        name: params.name,
                        description: params.description,
                    }),
                    updateData: params => ({
                        name: params.name,
                        description: params.description,
                    }),
                }),
                prices: mockResource({
                    table: tables.prices,
                    createData: params => ({
                        product: params.product,
                        currency: params.currency,
                        unit_amount: params.unit_amount,
                        recurring: params.recurring,
                    }),
                    updateData: params => ({
                        active: params.active,
                    }),
                }),
                checkout: {
                    sessions: mockResource({
                        table: tables.checkoutSessions,
                        createData: params => ({
                            customer: params.customer,
                        }),
                        updateData: params => ({}),
                        createHook: async (session) => {
                            await webhookEvent("checkout.session.completed", session);
                        },
                        updateHook: async (session) => {
                            await webhookEvent("checkout.session.completed", session);
                        },
                    }),
                },
                paymentMethods: mockResource({
                    table: tables.paymentMethods,
                    createData: params => ({
                        type: params.type,
                        customer: params.customer,
                    }),
                    updateData: params => ({}),
                }),
                setupIntents: mockResource({
                    table: tables.setupIntents,
                    createData: params => ({
                        customer: params.customer,
                        payment_method: params.payment_method,
                        usage: params.usage,
                    }),
                    updateData: params => ({
                        payment_method: params.payment_method,
                    }),
                }),
                subscriptions: mockResource({
                    table: tables.subscriptions,
                    createData: params => ({
                        customer: params.customer,
                        default_payment_method: params.default_payment_method,
                        cancel_at_period_end: params.cancel_at_period_end,
                        items: {
                            url: "",
                            object: "list",
                            has_more: false,
                            data: params.items.map(itemParams => ({
                                id: generateId(),
                                billing_thresholds: itemParams.billing_thresholds,
                                price: itemParams.price,
                                price_data: itemParams.price_data,
                                quantity: itemParams.quantity,
                                tax_rates: itemParams.tax_rates,
                            })),
                        },
                    }),
                    updateData: params => ({
                        cancel_at_period_end: params.cancel_at_period_end,
                        default_payment_method: params.default_payment_method,
                    }),
                    updateHook: async (subscription) => {
                        await webhookEvent("customer.subscription.updated", subscription);
                    },
                }),
            };
        }
        return { stripeLiaisonForPlatform, connectStripeLiaisonForApp };
    }

    async function dbbyX(flexStorage, tableName) {
        const table = (() => {
            const serializationKey = "__serialized_type__";
            function copy(x) {
                if (x === undefined)
                    return undefined;
                return JSON.parse(JSON.stringify(x));
            }
            function serialize(table) {
                return table.map(row => {
                    const serializedRow = {};
                    for (const [key, value] of Object.entries(row)) {
                        if (value instanceof DamnId) {
                            serializedRow[key] = {
                                [serializationKey]: "DamnId",
                                serializedValue: value.toString(),
                            };
                        }
                        else
                            serializedRow[key] = copy(value);
                    }
                    return serializedRow;
                });
            }
            function deserialize(data) {
                return data.map(datum => {
                    const row = {};
                    for (const [key, value] of Object.entries(datum)) {
                        if (value && typeof value === "object" && value[serializationKey]) {
                            if (value[serializationKey] === "DamnId") {
                                row[key] = DamnId.fromString(value.serializedValue);
                            }
                            else
                                throw new Error(`dbby-x unknown serialization type "${value[serializationKey]}"`);
                        }
                        else {
                            row[key] = value;
                        }
                    }
                    return row;
                });
            }
            const storageKey = `dbby-${tableName}`;
            async function load() {
                return deserialize(await flexStorage.read(storageKey) ?? []);
            }
            async function save(rows) {
                await flexStorage.write(storageKey, serialize(rows));
            }
            return { load, save };
        })();
        const readers = (() => {
            async function select(conditional) {
                return (await table.load())
                    .filter(row => rowVersusConditional(row, conditional));
            }
            async function discriminate(conditional) {
                const yep = [];
                const nope = [];
                for (const row of await table.load()) {
                    if (rowVersusConditional(row, conditional))
                        yep.push(row);
                    else
                        nope.push(row);
                }
                return { yep, nope };
            }
            async function selectOne(conditional) {
                return (await table.load())
                    .find(row => rowVersusConditional(row, conditional));
            }
            return { select, selectOne, discriminate };
        })();
        const writers = (() => {
            async function applyPatchToRows(conditional, patch) {
                const { yep, nope } = await readers.discriminate(conditional);
                const updated = yep.map(row => ({ ...row, ...patch }));
                await table.save([...updated, ...nope]);
            }
            async function eliminateRow(conditional) {
                const flippedFilterRow = (row) => !rowVersusConditional(row, conditional);
                const rows = await table.load();
                await table.save(rows.filter(flippedFilterRow));
            }
            async function insert(...rows) {
                const old = await table.load();
                await table.save([...old, ...rows]);
            }
            return { applyPatchToRows, eliminateRow, insert };
        })();
        const sequential = (() => {
            let promiseChain = Promise.resolve();
            function functionQueuesInSequence(func) {
                return (async (...args) => {
                    const next = promiseChain.then(() => func(...args));
                    promiseChain = next.catch(() => { });
                    return next;
                });
            }
            function groupOfFunctionsQueueInSequence(obj) {
                const newObject = {};
                for (const [key, value] of Object.entries(obj)) {
                    newObject[key] = functionQueuesInSequence(value);
                }
                return newObject;
            }
            return { groupOfFunctionsQueueInSequence };
        })();
        return {
            [_dbbyTableSymbol]: true,
            ...sequential.groupOfFunctionsQueueInSequence({
                async create(...rows) {
                    await writers.insert(...rows);
                },
                async read({ order, offset = 0, limit = 1000, ...conditional }) {
                    const rows = await readers.select(conditional);
                    if (order) {
                        for (const [key, value] of Object.entries(order)) {
                            rows.sort((a, b) => value === "ascend"
                                ? a[key] > b[key] ? 1 : -1
                                : a[key] > b[key] ? -1 : 1);
                        }
                    }
                    return rows.slice(offset, offset + limit);
                },
                async one(conditional) {
                    return readers.selectOne(conditional);
                },
                async assert({ make, ...conditional }) {
                    let row = await readers.selectOne(conditional);
                    if (!row) {
                        const made = await make();
                        await writers.insert(made);
                        row = made;
                    }
                    return row;
                },
                async update({ write, whole, upsert, ...conditional }) {
                    const rows = await readers.select(conditional);
                    if (write && rows.length) {
                        await writers.applyPatchToRows(conditional, write);
                    }
                    else if (upsert) {
                        if (rows.length)
                            await writers.applyPatchToRows(conditional, upsert);
                        else
                            await writers.insert(upsert);
                    }
                    else if (whole) {
                        await writers.eliminateRow(conditional);
                        await writers.insert(whole);
                    }
                    else
                        throw new Error("invalid update");
                },
                async delete(conditional) {
                    await writers.eliminateRow(conditional);
                },
                async count(conditional) {
                    return (await readers.select(conditional)).length;
                },
            })
        };
    }
    function compare(row, conditional, evaluator) {
        let failures = 0;
        for (const [key, value] of Object.entries(conditional)) {
            if (!evaluator(row[key], value))
                failures += 1;
        }
        return !failures;
    }
    function rowVersusConditional(row, conditional) {
        function crawl(conditions) {
            if (conditions === false)
                return true;
            const [operation, ...rawconds] = conditions;
            const conds = rawconds.filter(c => !!c);
            if (conds.length === 0)
                throw new Error("empty and/or conditions are not allowed");
            const and = operation === "and";
            let valid = and;
            const applyResult = (result) => valid = and
                ? valid && result
                : valid || result;
            for (const condition of conds) {
                if (condition === false || Array.isArray(condition)) {
                    applyResult(crawl(condition));
                }
                else {
                    const result = rowVersusCondition(row, condition);
                    applyResult(result);
                }
            }
            return valid;
        }
        return crawl(conditional.conditions);
    }
    function rowVersusCondition(row, condition) {
        if (!Object.keys(condition).length)
            return true;
        let failures = 0;
        const check = (conditions, evaluator) => {
            if (conditions && !compare(row, conditions, evaluator))
                failures += 1;
        };
        const checks = {
            set: a => a !== undefined && a !== null,
            equal: (a, b) => a instanceof DamnId
                ? a.string === b.string
                : a === b,
            greater: (a, b) => a > b,
            greatery: (a, b) => a >= b,
            less: (a, b) => a < b,
            lessy: (a, b) => a <= b,
            listed: (a, b) => a.includes(b),
            search: (a, b) => typeof b === "string" ? a.includes(b) : b.test(a),
        };
        function not(evaluator) {
            return (a, b) => !evaluator(a, b);
        }
        check(condition.set, checks.set);
        check(condition.equal, checks.equal);
        check(condition.greater, checks.greater);
        check(condition.greatery, checks.greatery);
        check(condition.less, checks.less);
        check(condition.lessy, checks.lessy);
        check(condition.listed, checks.listed);
        check(condition.search, checks.search);
        check(condition.notSet, not(checks.set));
        check(condition.notEqual, not(checks.equal));
        check(condition.notGreater, not(checks.greater));
        check(condition.notGreatery, not(checks.greatery));
        check(condition.notLess, not(checks.less));
        check(condition.notLessy, not(checks.lessy));
        check(condition.notListed, not(checks.listed));
        check(condition.notSearch, not(checks.search));
        return !failures;
    }

    async function mockStripeTables({ tableStorage }) {
        const spec = {
            prices: undefined,
            accounts: undefined,
            products: undefined,
            customers: undefined,
            setupIntents: undefined,
            subscriptions: undefined,
            paymentMethods: undefined,
            checkoutSessions: undefined,
        };
        const tables = await concurrent(objectMap2(spec, (value, key) => dbbyX(tableStorage, `mock-stripe-${key}`)));
        return tables;
    }

    async function mockStripeCircuit({ rando, tableStorage, authTables, storeTables, }) {
        const { publishers: webhookPublishers, subscribers: webhookSubscribers, } = pubsubs({
            "checkout.session.completed": pubsub(),
            "invoice.paid": pubsub(),
            "invoice.payment_failed": pubsub(),
            "customer.subscription.updated": pubsub(),
        });
        const stripeTables = await mockStripeTables({ tableStorage });
        const stripeComplex = mockStripeComplex({
            rando,
            tables: stripeTables,
            webhooks: webhookPublishers,
        });
        const webhooks = stripeWebhooks({
            authTables,
            storeTables,
            stripeComplex,
            logger: console,
        });
        for (const [key, subscribe] of Object.entries(webhookSubscribers))
            subscribe(webhooks[key].bind(webhooks));
        return {
            stripeComplex,
            mockStripeOperations: {
                linkBankWithExistingStripeAccount: async (stripeAccountId) => {
                    await stripeTables.accounts.update({
                        ...find({ id: stripeAccountId }),
                        write: {
                            payouts_enabled: true,
                            details_submitted: true,
                        },
                    });
                },
            },
        };
    }

    async function assimilateStripe({ database, mockStorage, config, rando, }) {
        if (config.stripe === "mock-mode") {
            return mockStripeCircuit({
                rando,
                tableStorage: mockStorage,
                authTables: database.auth,
                storeTables: database.store,
            });
        }
        else {
            throw new Error("real stripe not implemented");
        }
    }

    function mockDacastClient({ goodApiKey }) {
        let count = 1;
        function fakeContent(type) {
            const id = count++;
            return {
                online: true,
                id: id.toString(),
                title: `content ${id}`,
                creation_date: "1999-12-25",
            };
        }
        function fakeEmbedCode(type) {
            return type === "iframe"
                ? "<iframe src=\"https://iframe.dacast.com/vod/0ffd60be-91b7-22b0-1353-7dba7af43261/db07d056-2170-3940-cc4a-4e1add8b504b\" width=\"100%\" height=\"100%\" frameborder=\"0\" scrolling=\"no\" allowfullscreen webkitallowfullscreen mozallowfullscreen oallowfullscreen msallowfullscreen></iframe>"
                // ? "<iframe src=\"https://iframe.dacast.com/vod/0ffd60be-91b7-22b0-1353-7dba7af43261/db07d056-2170-3940-cc4a-4e1add8b504b\"  width=\"512\" height=\"288 frameborder=\"0\" scrolling=\"no\" allowfullscreen webkitallowfullscreen mozallowfullscreen oallowfullscreen msallowfullscreen></iframe>"
                : "<script id=\"0ffd60be-91b7-22b0-1353-7dba7af43261-vod-db07d056-2170-3940-cc4a-4e1add8b504b\" width=\"100%\" height=\"100%\" src=\"https://player.dacast.com/js/player.js?contentId=0ffd60be-91b7-22b0-1353-7dba7af43261-vod-db07d056-2170-3940-cc4a-4e1add8b504b\"  class=\"dacast-video\"></script>";
        }
        const data = {
            vods: [0, 0].map(() => fakeContent()),
            channels: [0, 0].map(() => fakeContent()),
            playlists: [0, 0].map(() => fakeContent()),
        };
        return ({ apiKey }) => {
            function resource(content) {
                function fun(f) {
                    return ((...args) => {
                        if (apiKey !== goodApiKey)
                            throw new Error("mock dacast invalid api key");
                        return f(...args);
                    });
                }
                function paginate(content) {
                    return {
                        totalCount: content.length.toString(),
                        data: content,
                        paging: {
                            last: "",
                            next: "",
                            previous: "",
                            self: "",
                        },
                    };
                }
                return {
                    get: fun(async () => paginate(content)),
                    id: (contentId) => ({
                        get: fun(async () => content.find(c => c.id === contentId)),
                        embed: (embedType) => ({
                            get: fun(async () => ({ code: fakeEmbedCode(embedType) })),
                        }),
                    }),
                };
            }
            return {
                vods: resource(data.vods),
                channels: resource(data.channels),
                playlists: resource(data.playlists),
            };
        };
    }

    function mockVerifyDacastApiKey({ goodApiKey }) {
        return async (apiKey) => apiKey === goodApiKey;
    }

    function mockDacastSdk({ goodApiKey }) {
        const getClient = apiKey => mockDacastClient({ goodApiKey })({ apiKey });
        return {
            getClient,
            verifyApiKey: mockVerifyDacastApiKey({ goodApiKey }),
        };
    }

    function assimilateDacast({ config, configureDacast, }) {
        if (config.dacast === "mock-mode") {
            return mockDacastSdk({ goodApiKey });
        }
        else if (config.dacast === true) {
            return configureDacast();
        }
        else {
            throw new Error("unknown dacast config");
        }
    }

    function isPlainObject(x) {
        return (typeof x === "object"
            && x !== null
            && !Array.isArray(x)
            && !(x instanceof Promise));
    }
    function unzipRecursive(object, path = []) {
        let spec = [];
        for (const [key, value] of Object.entries(object)) {
            if (isPlainObject(value))
                spec = [
                    ...spec,
                    ...unzipRecursive(value, [...path, key])
                ];
            else
                spec.push([[...path, key], value]);
        }
        return spec;
    }
    function unzip(object) {
        return unzipRecursive(object);
    }
    async function waitSpec(spec) {
        const paths = [];
        const awaitables = [];
        for (const [path, value] of spec) {
            paths.push(path);
            awaitables.push(value);
        }
        const values = await Promise.all(awaitables);
        return paths.map((path, index) => [path, values[index]]);
    }
    function applyProperty(object, path, value) {
        let level = object;
        for (let i = 0; i < path.length; i++) {
            const final = i === (path.length - 1);
            const key = path[i];
            const existing = level[key];
            if (final)
                level[key] = value;
            else {
                const existingIsObject = isPlainObject(existing);
                if (existingIsObject)
                    level = existing;
                else {
                    const next = {};
                    level[key] = next;
                    level = next;
                }
            }
        }
    }
    function zip(spec) {
        const result = {};
        for (const [path, value] of spec)
            applyProperty(result, path, value);
        return result;
    }
    async function waitForProperties(input) {
        const specWithPromises = unzip(input);
        const specWithValues = await waitSpec(specWithPromises);
        return zip(specWithValues);
    }

    function processBlueprintRecursive({ blueprint, path = [], process, }) {
        return objectMap2(blueprint, (value, key) => (value === true
            ? process([...path, key])
            : processBlueprintRecursive({
                blueprint: value,
                path: [...path, key],
                process,
            })));
    }
    function processBlueprint({ blueprint, process }) {
        return processBlueprintRecursive({ blueprint, process });
    }

    async function dbbyMemory(rows) {
        const table = await dbbyX(memoryFlexStorage(), "");
        if (rows)
            await table.create(...rows);
        return table;
    }

    function dbbyHardback({ frontTable, backTable }) {
        async function one(options) {
            return await backTable.one(options) ?? await frontTable.one(options);
        }
        return {
            [_dbbyTableSymbol]: true,
            one,
            create: frontTable.create,
            update: frontTable.update,
            delete: frontTable.delete,
            async read(options) {
                const [hardRows, actualRows] = await Promise.all([
                    backTable.read(options),
                    await frontTable.read(options),
                ]);
                return [...hardRows, ...actualRows];
            },
            async assert(options) {
                const row = await one(options);
                return row || await (async () => {
                    const newRow = await options.make();
                    await frontTable.create(newRow);
                    return newRow;
                })();
            },
            async count(options) {
                const [hardCount, actualCount] = await Promise.all([
                    backTable.count(options),
                    frontTable.count(options),
                ]);
                return hardCount + actualCount;
            },
        };
    }

    const namespaceKeyAppId = "namespace-appId";

    class UnconstrainedTables {
        constructor(tables) {
            this.tables = tables;
        }
        get unconstrained() {
            return this.tables;
        }
        namespaceForApp(appId) {
            return dbbyConstrainTables({
                tables: this.tables,
                namespace: { [namespaceKeyAppId]: appId },
            });
        }
    }
    // export type UnconstrainedTables<xTables extends DbbyTables> = (
    // 	DbbyUnconstrainTables<AppNamespaceRow, xTables>
    // )
    // export type ConstrainedTables<xTables extends DbbyTables> = (
    // 	DbbyConstrainTables<AppNamespaceRow, xTables>
    // )
    // export type SafelyConstrainedTable<xTable extends DbbyTable<DbbyRow>> = (
    // 	xTable extends DbbyTable<infer xRow>
    // 		? xRow extends AppNamespaceRow
    // 			? never
    // 			: DbbyTable<xRow>
    // 		: never
    // )
    // export type SafelyConstrainedTables<xTables extends DbbyTables> = {
    // 	[P in keyof xTables]: xTables[P] extends SafelyConstrainedTable<DbbyTable<DbbyRow>>
    // 		? xTables[P] extends DbbyTables
    // 			? SafelyConstrainedTables<xTables[P]>
    // 			: never
    // 		: never
    // }
    // type LolTable = DbbyTable<{a: number}>
    // type LolUnconstrained = DbbyUnconstrainTable<AppNamespaceRow, LolTable>
    // type LolConstrained = DbbyConstrainTable<AppNamespaceRow, LolTable>
    // let unconstrainedTable: LolUnconstrained
    // let constrainedTable: LolConstrained
    // let T: SafelyConstrainedTable<LolUnconstrained>
    // unconstrainedTable.create({a: 1, "appId-namespace": DamnId.fromString("")})
    // constrainedTable.create({a: 1})
    // function test(table: SafelyConstrainedTable<LolUnconstrained>) {}
    // test(unconstrainedTable)
    // type DisallowB<Stuff> = 
    // function userQuery(stuff: ) {}

    async function assimilateDatabase({ config, configureMongo, configureMockStorage, }) {
        const blueprintForRawDatabase = {
            apps: {
                registrations: true,
                owners: true,
            },
        };
        const blueprintForNamespacedDatabase = {
            auth: {
                users: {
                    accounts: true,
                    emails: true,
                    latestLogins: true,
                    profiles: true,
                },
                permissions: {
                    privilege: true,
                    role: true,
                    roleHasPrivilege: true,
                    userHasRole: true,
                },
            },
            example: {
                examplePosts: true,
            },
            questions: {
                likes: true,
                reports: true,
                answerPosts: true,
                questionPosts: true,
            },
            livestream: {
                shows: true,
            },
            store: {
                billing: {
                    customers: true,
                    storeInfo: true,
                    subscriptionPlans: true,
                    subscriptions: true,
                },
                merchant: {
                    stripeAccounts: true,
                },
            },
            videos: {
                dacastAccountLinks: true,
                viewDacast: true,
                viewPrivileges: true,
            },
        };
        async function mockWithStorage(mockStorage) {
            const databaseRaw = await waitForProperties(processBlueprint({
                blueprint: blueprintForRawDatabase,
                process: path => dbbyX(mockStorage, path.join("-")),
            }));
            const databaseUnconstrained = await (async () => {
                const databaseNamespaced = await waitForProperties(processBlueprint({
                    blueprint: blueprintForNamespacedDatabase,
                    process: path => dbbyX(mockStorage, path.join("-")),
                }));
                return objectMap(databaseNamespaced, value => new UnconstrainedTables(value));
            })();
            return {
                mockStorage,
                database: {
                    ...databaseRaw,
                    ...databaseUnconstrained,
                }
            };
        }
        const results = config.database === "mock-storage"
            ? await mockWithStorage(configureMockStorage())
            : await configureMongo({
                blueprintForRawDatabase,
                blueprintForNamespacedDatabase,
                config: { ...config, database: config.database },
            });
        const bakedAppTables = await (async () => {
            const platformApp = config.platform.appDetails;
            const { apps: appTables } = results.database;
            return {
                ...appTables,
                registrations: dbbyHardback({
                    frontTable: appTables.registrations,
                    backTable: await dbbyMemory([
                        {
                            appId: DamnId.fromString(platformApp.appId),
                            home: platformApp.home,
                            label: platformApp.label,
                            origins: originsToDatabase(platformApp.origins),
                            archived: false,
                        }
                    ])
                }),
            };
        })();
        return {
            database: {
                ...results.database,
                apps: bakedAppTables,
            },
            mockStorage: results.mockStorage,
        };
    }

    function prepareBackend(configurators) {
        return async function configureApi(config) {
            const rando = await getRando();
            const options = { ...configurators, config, rando };
            const emails = assimilateEmails(options);
            const { database, mockStorage } = await assimilateDatabase(options);
            const { signToken, verifyToken } = assimilateCrypto(options);
            const { stripeComplex, mockStripeOperations } = await assimilateStripe({
                ...options,
                database,
                mockStorage,
            });
            const dacastSdk = assimilateDacast(options);
            const api = await assimilateApi({
                ...options,
                database,
                dacastSdk,
                signToken,
                verifyToken,
                sendLoginEmail: emails.sendLoginEmail,
            });
            return {
                api,
                config,
                emails,
                database,
                stripeComplex,
                mockStripeOperations,
                platformAppId: config.platform.appDetails.appId,
                mockBrowser: async () => mockBrowser({
                    api,
                    mockStripeOperations,
                }),
            };
        };
    }

    class ConfigNotSupportedInBrowserError extends Error {
        constructor(message) {
            super(message);
        }
    }
    function nope(message) {
        return function () {
            throw new ConfigNotSupportedInBrowserError(message);
        };
    }
    const backendForBrowser = (storage) => prepareBackend({
        configureMongo: nope("config.database.mongo not supported in browser"),
        configureMailgun: nope("config.email.mailgun not supported in browser"),
        configureTokenFunctions: nope("config.crypto.keys must be 'mock-mode' in browser"),
        configureMockStorage: () => storage,
        configureDacast: nope("config.dacast must be in 'mock-mode' in browser"),
    });

    async function mockConnectApp({ origins, storage, appWindowLink, }) {
        const backend = await backendForBrowser(storage)(mockConfig({
            platformHome: appWindowLink,
            platformOrigins: origins,
        }));
        backend.emails.disableEmails();
        const apiLink = apiOrigin + "/";
        const ownerEmail = "creative@xiome.io";
        const adminEmail = "admin@xiome.io";
        let appId = await storage.read("mock-app");
        if (!appId) {
            appId = await mockRegisterApp({
                apiLink,
                backend,
                ownerEmail,
                adminEmail,
                appOrigins: origins,
            });
            await storage.write("mock-app", appId);
        }
        console.log(`mock: app owner email "${ownerEmail}"`);
        backend.emails.enableEmails();
        const { remote, authMediator } = await mockWiredRemote({
            appId,
            apiLink,
            backend,
            storage,
            appWindowLink,
        });
        const popups = mockPopups({
            mockStripeOperations: backend.mockStripeOperations,
        });
        return { appId, remote, storage, authMediator, backend, popups };
    }

    function addMockLatency({ latency, remote }) {
        function getRandomizedDelay() {
            if (latency === false)
                return 0;
            const range = latency.max - latency.min;
            const randomRange = Math.random() * range;
            return latency.min + randomRange;
        }
        function recurse(remote) {
            return objectMap(remote, (value, key) => {
                if (typeof value === "function") {
                    return async (...args) => {
                        const delay = getRandomizedDelay();
                        await nap(delay);
                        return value(...args);
                    };
                }
                else if (typeof (value ?? undefined) === "object")
                    return recurse(value);
                else
                    throw new Error("unknown remote type");
            });
        }
        return recurse(remote);
    }

    /**
     * @license
     * Copyright 2019 Google LLC
     * SPDX-License-Identifier: BSD-3-Clause
     */
    const t$1=window.ShadowRoot&&(void 0===window.ShadyCSS||window.ShadyCSS.nativeShadow)&&"adoptedStyleSheets"in Document.prototype&&"replace"in CSSStyleSheet.prototype,e$3=Symbol(),n$3=new Map;class s$2{constructor(t,n){if(this._$cssResult$=!0,n!==e$3)throw Error("CSSResult is not constructable. Use `unsafeCSS` or `css` instead.");this.cssText=t;}get styleSheet(){let e=n$3.get(this.cssText);return t$1&&void 0===e&&(n$3.set(this.cssText,e=new CSSStyleSheet),e.replaceSync(this.cssText)),e}toString(){return this.cssText}}const o$4=t=>new s$2("string"==typeof t?t:t+"",e$3),r$3=(t,...n)=>{const o=1===t.length?t[0]:n.reduce(((e,n,s)=>e+(t=>{if(!0===t._$cssResult$)return t.cssText;if("number"==typeof t)return t;throw Error("Value passed to 'css' function must be a 'css' function result: "+t+". Use 'unsafeCSS' to pass non-literal values, but take care to ensure page security.")})(n)+t[s+1]),t[0]);return new s$2(o,e$3)},i$3=(e,n)=>{t$1?e.adoptedStyleSheets=n.map((t=>t instanceof CSSStyleSheet?t:t.styleSheet)):n.forEach((t=>{const n=document.createElement("style"),s=window.litNonce;void 0!==s&&n.setAttribute("nonce",s),n.textContent=t.cssText,e.appendChild(n);}));},S$1=t$1?t=>t:t=>t instanceof CSSStyleSheet?(t=>{let e="";for(const n of t.cssRules)e+=n.cssText;return o$4(e)})(t):t;

    /**
     * @license
     * Copyright 2017 Google LLC
     * SPDX-License-Identifier: BSD-3-Clause
     */var s$1,e$2;const r$2={toAttribute(t,i){switch(i){case Boolean:t=t?"":null;break;case Object:case Array:t=null==t?t:JSON.stringify(t);}return t},fromAttribute(t,i){let s=t;switch(i){case Boolean:s=null!==t;break;case Number:s=null===t?null:Number(t);break;case Object:case Array:try{s=JSON.parse(t);}catch(t){s=null;}}return s}},h$1=(t,i)=>i!==t&&(i==i||t==t),o$3={attribute:!0,type:String,converter:r$2,reflect:!1,hasChanged:h$1};class n$2 extends HTMLElement{constructor(){super(),this._$Et=new Map,this.isUpdatePending=!1,this.hasUpdated=!1,this._$Ei=null,this.o();}static addInitializer(t){var i;null!==(i=this.l)&&void 0!==i||(this.l=[]),this.l.push(t);}static get observedAttributes(){this.finalize();const t=[];return this.elementProperties.forEach(((i,s)=>{const e=this._$Eh(s,i);void 0!==e&&(this._$Eu.set(e,s),t.push(e));})),t}static createProperty(t,i=o$3){if(i.state&&(i.attribute=!1),this.finalize(),this.elementProperties.set(t,i),!i.noAccessor&&!this.prototype.hasOwnProperty(t)){const s="symbol"==typeof t?Symbol():"__"+t,e=this.getPropertyDescriptor(t,s,i);void 0!==e&&Object.defineProperty(this.prototype,t,e);}}static getPropertyDescriptor(t,i,s){return {get(){return this[i]},set(e){const r=this[t];this[i]=e,this.requestUpdate(t,r,s);},configurable:!0,enumerable:!0}}static getPropertyOptions(t){return this.elementProperties.get(t)||o$3}static finalize(){if(this.hasOwnProperty("finalized"))return !1;this.finalized=!0;const t=Object.getPrototypeOf(this);if(t.finalize(),this.elementProperties=new Map(t.elementProperties),this._$Eu=new Map,this.hasOwnProperty("properties")){const t=this.properties,i=[...Object.getOwnPropertyNames(t),...Object.getOwnPropertySymbols(t)];for(const s of i)this.createProperty(s,t[s]);}return this.elementStyles=this.finalizeStyles(this.styles),!0}static finalizeStyles(i){const s=[];if(Array.isArray(i)){const e=new Set(i.flat(1/0).reverse());for(const i of e)s.unshift(S$1(i));}else void 0!==i&&s.push(S$1(i));return s}static _$Eh(t,i){const s=i.attribute;return !1===s?void 0:"string"==typeof s?s:"string"==typeof t?t.toLowerCase():void 0}o(){var t;this._$Ev=new Promise((t=>this.enableUpdating=t)),this._$AL=new Map,this._$Ep(),this.requestUpdate(),null===(t=this.constructor.l)||void 0===t||t.forEach((t=>t(this)));}addController(t){var i,s;(null!==(i=this._$Em)&&void 0!==i?i:this._$Em=[]).push(t),void 0!==this.renderRoot&&this.isConnected&&(null===(s=t.hostConnected)||void 0===s||s.call(t));}removeController(t){var i;null===(i=this._$Em)||void 0===i||i.splice(this._$Em.indexOf(t)>>>0,1);}_$Ep(){this.constructor.elementProperties.forEach(((t,i)=>{this.hasOwnProperty(i)&&(this._$Et.set(i,this[i]),delete this[i]);}));}createRenderRoot(){var t;const s=null!==(t=this.shadowRoot)&&void 0!==t?t:this.attachShadow(this.constructor.shadowRootOptions);return i$3(s,this.constructor.elementStyles),s}connectedCallback(){var t;void 0===this.renderRoot&&(this.renderRoot=this.createRenderRoot()),this.enableUpdating(!0),null===(t=this._$Em)||void 0===t||t.forEach((t=>{var i;return null===(i=t.hostConnected)||void 0===i?void 0:i.call(t)}));}enableUpdating(t){}disconnectedCallback(){var t;null===(t=this._$Em)||void 0===t||t.forEach((t=>{var i;return null===(i=t.hostDisconnected)||void 0===i?void 0:i.call(t)}));}attributeChangedCallback(t,i,s){this._$AK(t,s);}_$Eg(t,i,s=o$3){var e,h;const n=this.constructor._$Eh(t,s);if(void 0!==n&&!0===s.reflect){const o=(null!==(h=null===(e=s.converter)||void 0===e?void 0:e.toAttribute)&&void 0!==h?h:r$2.toAttribute)(i,s.type);this._$Ei=t,null==o?this.removeAttribute(n):this.setAttribute(n,o),this._$Ei=null;}}_$AK(t,i){var s,e,h;const o=this.constructor,n=o._$Eu.get(t);if(void 0!==n&&this._$Ei!==n){const t=o.getPropertyOptions(n),l=t.converter,a=null!==(h=null!==(e=null===(s=l)||void 0===s?void 0:s.fromAttribute)&&void 0!==e?e:"function"==typeof l?l:null)&&void 0!==h?h:r$2.fromAttribute;this._$Ei=n,this[n]=a(i,t.type),this._$Ei=null;}}requestUpdate(t,i,s){let e=!0;void 0!==t&&(((s=s||this.constructor.getPropertyOptions(t)).hasChanged||h$1)(this[t],i)?(this._$AL.has(t)||this._$AL.set(t,i),!0===s.reflect&&this._$Ei!==t&&(void 0===this._$ES&&(this._$ES=new Map),this._$ES.set(t,s))):e=!1),!this.isUpdatePending&&e&&(this._$Ev=this._$EC());}async _$EC(){this.isUpdatePending=!0;try{await this._$Ev;}catch(t){Promise.reject(t);}const t=this.scheduleUpdate();return null!=t&&await t,!this.isUpdatePending}scheduleUpdate(){return this.performUpdate()}performUpdate(){var t;if(!this.isUpdatePending)return;this.hasUpdated,this._$Et&&(this._$Et.forEach(((t,i)=>this[i]=t)),this._$Et=void 0);let i=!1;const s=this._$AL;try{i=this.shouldUpdate(s),i?(this.willUpdate(s),null===(t=this._$Em)||void 0===t||t.forEach((t=>{var i;return null===(i=t.hostUpdate)||void 0===i?void 0:i.call(t)})),this.update(s)):this._$ET();}catch(t){throw i=!1,this._$ET(),t}i&&this._$AE(s);}willUpdate(t){}_$AE(t){var i;null===(i=this._$Em)||void 0===i||i.forEach((t=>{var i;return null===(i=t.hostUpdated)||void 0===i?void 0:i.call(t)})),this.hasUpdated||(this.hasUpdated=!0,this.firstUpdated(t)),this.updated(t);}_$ET(){this._$AL=new Map,this.isUpdatePending=!1;}get updateComplete(){return this.getUpdateComplete()}getUpdateComplete(){return this._$Ev}shouldUpdate(t){return !0}update(t){void 0!==this._$ES&&(this._$ES.forEach(((t,i)=>this._$Eg(i,this[i],t))),this._$ES=void 0),this._$ET();}updated(t){}firstUpdated(t){}}n$2.finalized=!0,n$2.elementProperties=new Map,n$2.elementStyles=[],n$2.shadowRootOptions={mode:"open"},null===(s$1=globalThis.reactiveElementPolyfillSupport)||void 0===s$1||s$1.call(globalThis,{ReactiveElement:n$2}),(null!==(e$2=globalThis.reactiveElementVersions)&&void 0!==e$2?e$2:globalThis.reactiveElementVersions=[]).push("1.0.0");

    /**
     * @license
     * Copyright 2017 Google LLC
     * SPDX-License-Identifier: BSD-3-Clause
     */
    var t,i$2;const s=globalThis.trustedTypes,e$1=s?s.createPolicy("lit-html",{createHTML:t=>t}):void 0,o$2=`lit$${(Math.random()+"").slice(9)}$`,n$1="?"+o$2,l$1=`<${n$1}>`,h=document,r$1=(t="")=>h.createComment(t),d=t=>null===t||"object"!=typeof t&&"function"!=typeof t,u=Array.isArray,v=t=>{var i;return u(t)||"function"==typeof(null===(i=t)||void 0===i?void 0:i[Symbol.iterator])},c=/<(?:(!--|\/[^a-zA-Z])|(\/?[a-zA-Z][^>\s]*)|(\/?$))/g,a=/-->/g,f=/>/g,_=/>|[ 	\n\r](?:([^\s"'>=/]+)([ 	\n\r]*=[ 	\n\r]*(?:[^ 	\n\r"'`<>=]|("|')|))|$)/g,g=/'/g,m=/"/g,$=/^(?:script|style|textarea)$/i,p=t=>(i,...s)=>({_$litType$:t,strings:i,values:s}),y=p(1),b=p(2),T=Symbol.for("lit-noChange"),x=Symbol.for("lit-nothing"),w=new WeakMap,A=(t,i,s)=>{var e,o;const n=null!==(e=null==s?void 0:s.renderBefore)&&void 0!==e?e:i;let l=n._$litPart$;if(void 0===l){const t=null!==(o=null==s?void 0:s.renderBefore)&&void 0!==o?o:null;n._$litPart$=l=new S(i.insertBefore(r$1(),t),t,void 0,null!=s?s:{});}return l._$AI(t),l},C=h.createTreeWalker(h,129,null,!1),P=(t,i)=>{const s=t.length-1,n=[];let h,r=2===i?"<svg>":"",d=c;for(let i=0;i<s;i++){const s=t[i];let e,u,v=-1,p=0;for(;p<s.length&&(d.lastIndex=p,u=d.exec(s),null!==u);)p=d.lastIndex,d===c?"!--"===u[1]?d=a:void 0!==u[1]?d=f:void 0!==u[2]?($.test(u[2])&&(h=RegExp("</"+u[2],"g")),d=_):void 0!==u[3]&&(d=_):d===_?">"===u[0]?(d=null!=h?h:c,v=-1):void 0===u[1]?v=-2:(v=d.lastIndex-u[2].length,e=u[1],d=void 0===u[3]?_:'"'===u[3]?m:g):d===m||d===g?d=_:d===a||d===f?d=c:(d=_,h=void 0);const y=d===_&&t[i+1].startsWith("/>")?" ":"";r+=d===c?s+l$1:v>=0?(n.push(e),s.slice(0,v)+"$lit$"+s.slice(v)+o$2+y):s+o$2+(-2===v?(n.push(void 0),i):y);}const u=r+(t[s]||"<?>")+(2===i?"</svg>":"");return [void 0!==e$1?e$1.createHTML(u):u,n]};class V{constructor({strings:t,_$litType$:i},e){let l;this.parts=[];let h=0,d=0;const u=t.length-1,v=this.parts,[c,a]=P(t,i);if(this.el=V.createElement(c,e),C.currentNode=this.el.content,2===i){const t=this.el.content,i=t.firstChild;i.remove(),t.append(...i.childNodes);}for(;null!==(l=C.nextNode())&&v.length<u;){if(1===l.nodeType){if(l.hasAttributes()){const t=[];for(const i of l.getAttributeNames())if(i.endsWith("$lit$")||i.startsWith(o$2)){const s=a[d++];if(t.push(i),void 0!==s){const t=l.getAttribute(s.toLowerCase()+"$lit$").split(o$2),i=/([.?@])?(.*)/.exec(s);v.push({type:1,index:h,name:i[2],strings:t,ctor:"."===i[1]?k:"?"===i[1]?H:"@"===i[1]?I:M});}else v.push({type:6,index:h});}for(const i of t)l.removeAttribute(i);}if($.test(l.tagName)){const t=l.textContent.split(o$2),i=t.length-1;if(i>0){l.textContent=s?s.emptyScript:"";for(let s=0;s<i;s++)l.append(t[s],r$1()),C.nextNode(),v.push({type:2,index:++h});l.append(t[i],r$1());}}}else if(8===l.nodeType)if(l.data===n$1)v.push({type:2,index:h});else {let t=-1;for(;-1!==(t=l.data.indexOf(o$2,t+1));)v.push({type:7,index:h}),t+=o$2.length-1;}h++;}}static createElement(t,i){const s=h.createElement("template");return s.innerHTML=t,s}}function E(t,i,s=t,e){var o,n,l,h;if(i===T)return i;let r=void 0!==e?null===(o=s._$Cl)||void 0===o?void 0:o[e]:s._$Cu;const u=d(i)?void 0:i._$litDirective$;return (null==r?void 0:r.constructor)!==u&&(null===(n=null==r?void 0:r._$AO)||void 0===n||n.call(r,!1),void 0===u?r=void 0:(r=new u(t),r._$AT(t,s,e)),void 0!==e?(null!==(l=(h=s)._$Cl)&&void 0!==l?l:h._$Cl=[])[e]=r:s._$Cu=r),void 0!==r&&(i=E(t,r._$AS(t,i.values),r,e)),i}class N{constructor(t,i){this.v=[],this._$AN=void 0,this._$AD=t,this._$AM=i;}get parentNode(){return this._$AM.parentNode}get _$AU(){return this._$AM._$AU}p(t){var i;const{el:{content:s},parts:e}=this._$AD,o=(null!==(i=null==t?void 0:t.creationScope)&&void 0!==i?i:h).importNode(s,!0);C.currentNode=o;let n=C.nextNode(),l=0,r=0,d=e[0];for(;void 0!==d;){if(l===d.index){let i;2===d.type?i=new S(n,n.nextSibling,this,t):1===d.type?i=new d.ctor(n,d.name,d.strings,this,t):6===d.type&&(i=new L(n,this,t)),this.v.push(i),d=e[++r];}l!==(null==d?void 0:d.index)&&(n=C.nextNode(),l++);}return o}m(t){let i=0;for(const s of this.v)void 0!==s&&(void 0!==s.strings?(s._$AI(t,s,i),i+=s.strings.length-2):s._$AI(t[i])),i++;}}class S{constructor(t,i,s,e){var o;this.type=2,this._$AH=x,this._$AN=void 0,this._$AA=t,this._$AB=i,this._$AM=s,this.options=e,this._$Cg=null===(o=null==e?void 0:e.isConnected)||void 0===o||o;}get _$AU(){var t,i;return null!==(i=null===(t=this._$AM)||void 0===t?void 0:t._$AU)&&void 0!==i?i:this._$Cg}get parentNode(){let t=this._$AA.parentNode;const i=this._$AM;return void 0!==i&&11===t.nodeType&&(t=i.parentNode),t}get startNode(){return this._$AA}get endNode(){return this._$AB}_$AI(t,i=this){t=E(this,t,i),d(t)?t===x||null==t||""===t?(this._$AH!==x&&this._$AR(),this._$AH=x):t!==this._$AH&&t!==T&&this.$(t):void 0!==t._$litType$?this.T(t):void 0!==t.nodeType?this.S(t):v(t)?this.M(t):this.$(t);}A(t,i=this._$AB){return this._$AA.parentNode.insertBefore(t,i)}S(t){this._$AH!==t&&(this._$AR(),this._$AH=this.A(t));}$(t){this._$AH!==x&&d(this._$AH)?this._$AA.nextSibling.data=t:this.S(h.createTextNode(t)),this._$AH=t;}T(t){var i;const{values:s,_$litType$:e}=t,o="number"==typeof e?this._$AC(t):(void 0===e.el&&(e.el=V.createElement(e.h,this.options)),e);if((null===(i=this._$AH)||void 0===i?void 0:i._$AD)===o)this._$AH.m(s);else {const t=new N(o,this),i=t.p(this.options);t.m(s),this.S(i),this._$AH=t;}}_$AC(t){let i=w.get(t.strings);return void 0===i&&w.set(t.strings,i=new V(t)),i}M(t){u(this._$AH)||(this._$AH=[],this._$AR());const i=this._$AH;let s,e=0;for(const o of t)e===i.length?i.push(s=new S(this.A(r$1()),this.A(r$1()),this,this.options)):s=i[e],s._$AI(o),e++;e<i.length&&(this._$AR(s&&s._$AB.nextSibling,e),i.length=e);}_$AR(t=this._$AA.nextSibling,i){var s;for(null===(s=this._$AP)||void 0===s||s.call(this,!1,!0,i);t&&t!==this._$AB;){const i=t.nextSibling;t.remove(),t=i;}}setConnected(t){var i;void 0===this._$AM&&(this._$Cg=t,null===(i=this._$AP)||void 0===i||i.call(this,t));}}class M{constructor(t,i,s,e,o){this.type=1,this._$AH=x,this._$AN=void 0,this.element=t,this.name=i,this._$AM=e,this.options=o,s.length>2||""!==s[0]||""!==s[1]?(this._$AH=Array(s.length-1).fill(new String),this.strings=s):this._$AH=x;}get tagName(){return this.element.tagName}get _$AU(){return this._$AM._$AU}_$AI(t,i=this,s,e){const o=this.strings;let n=!1;if(void 0===o)t=E(this,t,i,0),n=!d(t)||t!==this._$AH&&t!==T,n&&(this._$AH=t);else {const e=t;let l,h;for(t=o[0],l=0;l<o.length-1;l++)h=E(this,e[s+l],i,l),h===T&&(h=this._$AH[l]),n||(n=!d(h)||h!==this._$AH[l]),h===x?t=x:t!==x&&(t+=(null!=h?h:"")+o[l+1]),this._$AH[l]=h;}n&&!e&&this.k(t);}k(t){t===x?this.element.removeAttribute(this.name):this.element.setAttribute(this.name,null!=t?t:"");}}class k extends M{constructor(){super(...arguments),this.type=3;}k(t){this.element[this.name]=t===x?void 0:t;}}class H extends M{constructor(){super(...arguments),this.type=4;}k(t){t&&t!==x?this.element.setAttribute(this.name,""):this.element.removeAttribute(this.name);}}class I extends M{constructor(t,i,s,e,o){super(t,i,s,e,o),this.type=5;}_$AI(t,i=this){var s;if((t=null!==(s=E(this,t,i,0))&&void 0!==s?s:x)===T)return;const e=this._$AH,o=t===x&&e!==x||t.capture!==e.capture||t.once!==e.once||t.passive!==e.passive,n=t!==x&&(e===x||o);o&&this.element.removeEventListener(this.name,this,e),n&&this.element.addEventListener(this.name,this,t),this._$AH=t;}handleEvent(t){var i,s;"function"==typeof this._$AH?this._$AH.call(null!==(s=null===(i=this.options)||void 0===i?void 0:i.host)&&void 0!==s?s:this.element,t):this._$AH.handleEvent(t);}}class L{constructor(t,i,s){this.element=t,this.type=6,this._$AN=void 0,this._$AM=i,this.options=s;}get _$AU(){return this._$AM._$AU}_$AI(t){E(this,t);}}null===(t=globalThis.litHtmlPolyfillSupport)||void 0===t||t.call(globalThis,V,S),(null!==(i$2=globalThis.litHtmlVersions)&&void 0!==i$2?i$2:globalThis.litHtmlVersions=[]).push("2.0.0");

    /**
     * @license
     * Copyright 2017 Google LLC
     * SPDX-License-Identifier: BSD-3-Clause
     */var l,o$1,r;class n extends n$2{constructor(){super(...arguments),this.renderOptions={host:this},this._$Dt=void 0;}createRenderRoot(){var t,e;const i=super.createRenderRoot();return null!==(t=(e=this.renderOptions).renderBefore)&&void 0!==t||(e.renderBefore=i.firstChild),i}update(t){const i=this.render();this.hasUpdated||(this.renderOptions.isConnected=this.isConnected),super.update(t),this._$Dt=A(i,this.renderRoot,this.renderOptions);}connectedCallback(){var t;super.connectedCallback(),null===(t=this._$Dt)||void 0===t||t.setConnected(!0);}disconnectedCallback(){var t;super.disconnectedCallback(),null===(t=this._$Dt)||void 0===t||t.setConnected(!1);}render(){return T}}n.finalized=!0,n._$litElement$=!0,null===(l=globalThis.litElementHydrateSupport)||void 0===l||l.call(globalThis,{LitElement:n}),null===(o$1=globalThis.litElementPolyfillSupport)||void 0===o$1||o$1.call(globalThis,{LitElement:n});(null!==(r=globalThis.litElementVersions)&&void 0!==r?r:globalThis.litElementVersions=[]).push("3.0.0");

    class AutowatcherCircularError extends Error {
        constructor(scheduled) {
            super(`an effect changes an observable, named "${scheduled.key}", that triggers that same effect, causing an infinite circle`);
        }
    }
    class AutowatcherLeakError extends Error {
        constructor(key) {
            super(`cannot set observable, named "${key}", outside a formalized action`);
        }
    }

    function autowatcherCore() {
        const context = {
            observableRecords: new Map(),
            newObservablesSchedule: [],
            activeAction: undefined,
            activeStakeout: undefined,
        };
        function obtainRecord(object) {
            if (context.observableRecords.has(object)) {
                return context.observableRecords.get(object);
            }
            else {
                const record = {};
                context.observableRecords.set(object, record);
                return record;
            }
        }
        function forbidCircularProblems(object, key) {
            const scheduled = context.newObservablesSchedule.find(s => s.object === object &&
                s.key === key);
            if (scheduled)
                throw new AutowatcherCircularError(scheduled);
        }
        function subscribe(subscription) {
            const { object, key, stakeout } = subscription;
            const record = obtainRecord(object);
            const stakeouts = record[key] ?? [];
            record[key] = [...stakeouts, stakeout];
            return { subscription, record };
        }
        function triggerObservation(object, key) {
            forbidCircularProblems(object, key);
            const record = obtainRecord(object);
            const stakeouts = record[key] ?? [];
            for (const { watcher, effect } of stakeouts) {
                if (effect)
                    effect(watcher());
                else
                    watcher();
            }
        }
        return {
            context,
            subscribe,
            triggerObservation,
        };
    }

    //
    // deprecated
    // now we prefer to use happystate
    //
    function autowatcher() {
        const { context, subscribe, triggerObservation } = autowatcherCore();
        function observable(object) {
            return new Proxy(object, {
                get(t, key) {
                    if (context.activeStakeout)
                        context.newObservablesSchedule.push({
                            object,
                            key,
                            stakeout: context.activeStakeout,
                        });
                    return object[key];
                },
                set(t, key, value) {
                    const oldValue = object[key];
                    object[key] = value;
                    const isChanged = value !== oldValue;
                    if (!context.activeAction)
                        throw new AutowatcherLeakError(key);
                    if (isChanged)
                        triggerObservation(object, key);
                    return true;
                },
            });
        }
        function track(stakeout) {
            let unsubscribe = () => { };
            let done = () => { };
            context.activeStakeout = stakeout;
            context.newObservablesSchedule = [];
            try {
                const initialObservation = stakeout.watcher();
                done = stakeout.effect
                    ? () => stakeout.effect(initialObservation)
                    : done;
                const recent = context.newObservablesSchedule
                    .map(scheduled => subscribe(scheduled));
                unsubscribe = () => {
                    for (const { subscription: { key }, record } of recent) {
                        const existingStakeouts = record[key] ?? [];
                        record[key] = existingStakeouts.filter(s => s !== stakeout);
                    }
                };
            }
            finally {
                context.activeStakeout = undefined;
                context.newObservablesSchedule = [];
                done();
            }
            return unsubscribe;
        }
        function watch(watcher, effect) {
            return track({ watcher, effect });
        }
        function runInAction(act) {
            context.activeAction = act;
            act();
            context.activeAction = undefined;
        }
        function action(act) {
            return ((...args) => {
                context.activeAction = act;
                const result = act(...args);
                context.activeAction = undefined;
                return result;
            });
        }
        function state(object) {
            for (const [key, value] of Object.entries(object)) {
                if (value !== null && typeof value === "object" && !Array.isArray(value))
                    object[key] = state(value);
            }
            return observable(object);
        }
        function actions(object) {
            return objectMap(object, value => {
                if (value !== null && typeof value === "object")
                    return actions(value);
                else if (typeof value === "function")
                    return action(value);
                else
                    return value;
            });
        }
        function dispose() {
            context.observableRecords = new Map();
        }
        // function setup<xState extends {}, xActions extends Actions>({
        // 		state: stateObject,
        // 		actions: actionsObject,
        // 	}: {
        // 		state: xState
        // 		actions: xActions
        // 	}) {
        // 	return {
        // 		state: state(stateObject),
        // 		actions: actions(actionsObject),
        // 	}
        // }
        return {
            observable,
            state,
            track,
            watch,
            action,
            actions,
            dispose,
            runInAction,
            // setup,
        };
    }

    var __classPrivateFieldGet$a = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var __classPrivateFieldSet$8 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    function mixinAutowatcher(Base) {
        var _unsubscribers, _a;
        return _a = class extends Base {
                constructor() {
                    super(...arguments);
                    this.auto = autowatcher();
                    _unsubscribers.set(this, []);
                    this.subscribeAutotrack = (track) => {
                        __classPrivateFieldGet$a(this, _unsubscribers, "f").push(track({
                            watcher: () => this.render(),
                            effect: () => this.requestUpdate(),
                        }));
                    };
                }
                render() { }
                firstUpdated(changes) {
                    this.subscribeAutotrack(this.auto.track);
                }
                dispose() {
                    for (const unsubscribe of __classPrivateFieldGet$a(this, _unsubscribers, "f"))
                        unsubscribe();
                    __classPrivateFieldSet$8(this, _unsubscribers, [], "f");
                    this.auto.dispose();
                }
            },
            _unsubscribers = new WeakMap(),
            _a;
    }

    function mixinInitiallyHidden(Base) {
        return class extends Base {
            connectedCallback() {
                super.connectedCallback();
                if (this.hasAttribute("initially-hidden"))
                    this.removeAttribute("initially-hidden");
            }
        };
    }

    /**
     * @license
     * Copyright 2017 Google LLC
     * SPDX-License-Identifier: BSD-3-Clause
     */
    const i$1=(i,e)=>"method"===e.kind&&e.descriptor&&!("value"in e.descriptor)?{...e,finisher(n){n.createProperty(e.key,i);}}:{kind:"field",key:Symbol(),placement:"own",descriptor:{},originalKey:e.key,initializer(){"function"==typeof e.initializer&&(this[e.key]=e.initializer.call(this));},finisher(n){n.createProperty(e.key,i);}};function e(e){return (n,t)=>void 0!==t?((i,e,n)=>{e.constructor.createProperty(n,i);})(e,n,t):i$1(e,n)}

    /**
     * @license
     * Copyright 2017 Google LLC
     * SPDX-License-Identifier: BSD-3-Clause
     */
    const o=({finisher:e,descriptor:t})=>(o,n)=>{var r;if(void 0===n){const n=null!==(r=o.originalKey)&&void 0!==r?r:o.key,i=null!=t?{kind:"method",placement:"prototype",key:n,descriptor:t(o.key)}:{...o,key:n};return null!=e&&(i.finisher=function(t){e(t,n);}),i}{const r=o.constructor;void 0!==t&&Object.defineProperty(o,n,t(n)),null==e||e(r,n);}};

    /**
     * @license
     * Copyright 2017 Google LLC
     * SPDX-License-Identifier: BSD-3-Clause
     */function i(i,n){return o({descriptor:o=>{const t={get(){var o,n;return null!==(n=null===(o=this.renderRoot)||void 0===o?void 0:o.querySelector(i))&&void 0!==n?n:null},enumerable:!0,configurable:!0};if(n){const n="symbol"==typeof o?Symbol():"__"+o;t.get=function(){var o,t;return void 0===this[n]&&(this[n]=null!==(t=null===(o=this.renderRoot)||void 0===o?void 0:o.querySelector(i))&&void 0!==t?t:null),this[n]};}return t}})}

    function arrayize(item) {
        return [item].flat();
    }
    const notUndefined = (x) => x !== undefined;
    function combineStyles(parentStyles, newStyles) {
        const styles = [
            ...(arrayize(parentStyles) ?? []),
            ...arrayize(newStyles),
        ];
        return styles
            .flat()
            .filter(notUndefined);
    }
    function mixinStyles(...newStyles) {
        return function (Base) {
            var _a;
            return _a = class extends Base {
                },
                _a.styles = combineStyles(Base.styles, newStyles),
                _a;
        };
    }

    const themeComponents = (theme, components) => {
        return objectMap(components, Component => mixinStyles(theme)(Component));
    };

    var __classPrivateFieldSet$7 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var __classPrivateFieldGet$9 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    function mixinHappy(...onStateChanges) {
        return function (Base) {
            var _unsubscribes, _a;
            return _a = class extends Base {
                    constructor() {
                        super(...arguments);
                        _unsubscribes.set(this, []);
                    }
                    connectedCallback() {
                        super.connectedCallback();
                        const update = () => {
                            this.requestUpdate();
                        };
                        __classPrivateFieldSet$7(this, _unsubscribes, onStateChanges
                            .map(onStateChange => onStateChange(update)), "f");
                    }
                    disconnectedCallback() {
                        super.disconnectedCallback();
                        for (const unsubscribe of __classPrivateFieldGet$9(this, _unsubscribes, "f"))
                            unsubscribe();
                        __classPrivateFieldSet$7(this, _unsubscribes, [], "f");
                    }
                },
                _unsubscribes = new WeakMap(),
                _a;
        };
    }

    function mixinFocusable(Base) {
        return class extends Base {
            connectedCallback() {
                super.connectedCallback();
                this.setAttribute("focusable", "");
            }
        };
    }

    var __classPrivateFieldGet$8 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var __classPrivateFieldSet$6 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    function mixinSnapstateSubscriptions(...subscriptions) {
        return function (Base) {
            var _unsubscribes_1, _a;
            return _a = class extends Base {
                    constructor() {
                        super(...arguments);
                        _unsubscribes_1.set(this, []);
                    }
                    connectedCallback() {
                        super.connectedCallback();
                        const update = () => { this.requestUpdate(); };
                        __classPrivateFieldSet$6(this, _unsubscribes_1, subscriptions.map(subscription => subscription(update)), "f");
                    }
                    disconnectedCallback() {
                        super.disconnectedCallback();
                        for (const unsubscribe of __classPrivateFieldGet$8(this, _unsubscribes_1, "f"))
                            unsubscribe();
                        __classPrivateFieldSet$6(this, _unsubscribes_1, [], "f");
                    }
                },
                _unsubscribes_1 = new WeakMap(),
                _a;
        };
    }

    function mixinShare(s) {
        return function (Base) {
            return class extends Base {
                get share() {
                    return s;
                }
            };
        };
    }

    var __classPrivateFieldSet$5 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var __classPrivateFieldGet$7 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    function mixinTicker(period) {
        return function (Base) {
            var _interval, _a;
            return _a = class extends Base {
                    constructor() {
                        super(...arguments);
                        _interval.set(this, void 0);
                    }
                    tick() { }
                    connectedCallback() {
                        super.connectedCallback();
                        __classPrivateFieldSet$5(this, _interval, setInterval(() => {
                            this.tick();
                            this.requestUpdate();
                        }, period), "f");
                    }
                    disconnectedCallback() {
                        super.disconnectedCallback();
                        clearInterval(__classPrivateFieldGet$7(this, _interval, "f"));
                    }
                },
                _interval = new WeakMap(),
                _a;
        };
    }

    var __classPrivateFieldGet$6 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var __classPrivateFieldSet$4 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var _Component_subscriptions, _Component_unsubscribe;
    class Component extends mixinInitiallyHidden(n) {
        constructor() {
            super(...arguments);
            _Component_subscriptions.set(this, []);
            _Component_unsubscribe.set(this, () => { });
        }
        init() { }
        firstUpdated(changes) {
            super.firstUpdated(changes);
            this.init();
        }
        addSubscription(subscribe) {
            __classPrivateFieldGet$6(this, _Component_subscriptions, "f").push(subscribe);
        }
        subscribe() {
            const unsubscribes = __classPrivateFieldGet$6(this, _Component_subscriptions, "f").map(s => s());
            return () => unsubscribes.forEach(u => u());
        }
        connectedCallback() {
            super.connectedCallback();
            __classPrivateFieldSet$4(this, _Component_unsubscribe, this.subscribe(), "f");
        }
        disconnectedCallback() {
            super.disconnectedCallback();
            __classPrivateFieldGet$6(this, _Component_unsubscribe, "f").call(this);
            __classPrivateFieldSet$4(this, _Component_unsubscribe, () => { }, "f");
        }
        render() {
            throw new Error("component render method not implemented");
        }
    }
    _Component_subscriptions = new WeakMap(), _Component_unsubscribe = new WeakMap();
    class ComponentWithShare extends Component {
        constructor() {
            super();
            if (this.share === undefined)
                throw new Error("component requires share");
        }
    }
    class AutowatcherComponent extends mixinAutowatcher(mixinInitiallyHidden(n)) {
        init() { }
        firstUpdated(changes) {
            this.init();
            super.firstUpdated(changes);
        }
    }
    class AutowatcherComponentWithShare extends AutowatcherComponent {
        constructor() {
            super();
            if (this.share === undefined)
                throw new Error("component requires share");
        }
    }

    var theme = r$3 `

* {
	margin: 0;
	padding: 0;
	box-sizing: border-box;
}

*:focus {
	outline: var(--focus-outline, 3px solid cyan);
}

`;

    var adminManagerCss = r$3 `

.adminmanager .adminassigner {
	display: flex;
	flex-direction: row;
	margin: 0.5em 0;
}

.adminmanager .adminassigner > * {
	display: block;
}

.adminmanager .adminassigner > xio-text-input {
	flex: 1 1 auto;
}

.adminmanager .adminassigner > xio-text-input {
	width: 100%;
	max-width: 100%;
}

.adminmanager .adminassigner > xio-text-input::part(problems) {
	min-width: unset;
}

.adminmanager .adminassigner > xio-button {
	margin-top: 1.1em;
	margin-left: 0.3em;
}

.adminmanager .adminlist ul {
	list-style: none;
	padding: 0 1em;
	border: 1px solid #fff1;
}

.adminmanager .adminlist li {
	display: flex;
	flex-direction: row;
	align-content: center;
	padding: 0.3em 0;
	font-size: 0.8em;
}

.adminmanager .adminlist li + li {
	border-top: 1px solid #fff1;
}

.adminmanager .adminlist li > * {
	display: block;
}

.adminmanager .adminlist li > span {
	flex: 1 1 auto;
	display: flex;
	align-items: center;
	word-break: break-all;
}

.adminmanager .adminlist li > xio-button {
	color: #fff3;
}

`;

    var styles$j = r$3 `

:host {
	display: block;
}

.app-list {
	margin-top: 2em;
	margin-bottom: 2em;
}

.app-list > xio-op {
	display: block;
	border: 1px solid #f00;
	margin-top: 0.5em;
	border: 1px solid #fff1;
	border-radius: 0.2em;
}

.app-list > xio-op[mode="loading"] {
	padding: 4em 2em;
}

.app > * + * {
	margin-top: 0.3em;
}

.app + .app {
	margin-top: 1.5em;
}

.app > * {
	padding: 0.4em 1rem;
}

.app-header {
	display: flex;
	flex-direction: row;
	flex-wrap: wrap;
	justify-content: flex-end;
	padding: 0;
	background: #0001;
}

.app-header .title {
	max-width: 100%;
	flex: 1 0 auto;
	padding: 0.3em 1em;
}

.app-header .title h3 {
	font-size: 1.5em;
	font-weight: bold;
}

.app-header .title h3::before {
	content: "🌐";
	color: white;
}

.app-header .stats {
	display: flex;
	flex-direction: row;
	justify-content: center;
	flex-wrap: wrap;
	padding: 0.2em;
}

.app-header .stats [data-stat] {
	display: flex;
	flex-direction: column;
	justify-content: flex-end;
	max-width: 6em;
	text-align: center;
}

.app-header .stats [data-stat] > span {
	padding: 0.1em 0.6em;
}

.app-header .stats [data-stat] > span:nth-child(1) {
	justify-self: flex-start;
}

.app-header .stats [data-stat] > span:nth-child(2) {
	opacity: 0.3;
	font-size: 0.6em;
}

.twoside {
	display: flex;
	flex-direction: row;
	flex-wrap: wrap;
}

.twoside > div {
	max-width: 100%;
	flex: 1 0 16em;
	padding-bottom: 1em;
}

.twoside > div:nth-child(1) {
	padding-right: 1rem;
}

.editside > .buttons {
	display: flex;
	justify-content: center;
	align-items: center;
}

.app-form > * + * {
	margin-top: 0.3em;
}

.app-form xio-button {
	margin-top: 0.6em;
}

.app-options {
	border: 1px solid #fff1;
	padding: 1em;
}

.app-options > * + * {
	margin-top: 3em;
}

code {
	display: inline-block;
	font-size: 0.8em;
	padding: 0.2em;
	border-radius: 0.3em;
	background: #0002;
	overflow-wrap: anywhere;
	word-break: break-all;
}

code.id {
	opacity: 0.7;
	font-size: 0.5em;
}

.codeblock {
	display: block;
}

.htmlcode { color: #fff6; }
.htmlcode [data-syntax=tag] { color: deepskyblue; }
.htmlcode [data-syntax=attr] { color: skyblue; }
.htmlcode [data-syntax=data] { color: #aaffa0; }
.htmlcode [data-syntax=indent] {
	display: block;
	margin-left: 1em;
}

.app-code > * {
	margin-top: 1em;
}

.app-code code {
	display: block;
	font-size: 0.6em;
	padding: 1em;
	margin: 0.5em 0;
}

.delete-app-button {
	display: block;
	text-align: right;
	--xio-button-hover-color: red;
}

${adminManagerCss}

.app-list {
	margin-bottom: 5rem;
}

.app-registration {
	max-width: 40em;
	margin-bottom: 5em;
}

`;

    function renderXiomeConfig(appId) {
        const h = (syntax, s) => y `<span data-syntax=${syntax}>${s}</span>`;
        const tag = (s) => h("tag", s);
        const attr = (s) => h("attr", s);
        const data = (s) => h("data", s);
        const glue = (s) => h("glue", s);
        const quote = glue(`"`);
        const bundle_link = "https://unpkg.com/xiome/x/xiome.bundle.min.js";
        return y `
		${glue(`<`)}${tag(`script`)} ${attr(`async`)} ${attr(`defer`)} ${attr(`src`)}${glue(`=`)}${quote}${data(bundle_link)}${quote}${glue(`>`)}${glue(`</`)}${tag(`script`)}${glue(`>`)}
		<br/>
		${glue(`<`)}${tag(`xiome-config`)} ${attr(`app`)}${glue(`=`)}${quote}${data(appId)}${quote}${glue(`>`)}${glue(`</`)}${tag(`xiome-config`)}${glue(`>`)}
	`;
    }

    const parseOrigins = (text) => text
        .split("\n")
        .map(line => line.trim().toLowerCase())
        .filter(line => line.length > 0);

    function validateAppFormDraft(formDraft) {
        const problems = [
            ...appDraftValidators.label(formDraft.label),
            ...appDraftValidators.home(formDraft.home),
            ...appDraftValidators.additionalOrigins(formDraft.additionalOrigins),
        ];
        return problems;
    }

    function getEmptyAppFormDraft() {
        return {
            home: "",
            label: "",
            additionalOrigins: [],
        };
    }

    function makeAppForm({ clearOnSubmit, submitButtonText, showAdditionalOrigins, initialValues = getEmptyAppFormDraft(), query, submit, requestUpdate, }) {
        const state = {
            problems: [],
            formDisabled: false,
            draft: initialValues,
            get valid() {
                return !this.formDisabled
                    && this.draft
                    && this.problems.length === 0;
            },
        };
        function getFormElements() {
            return {
                home: query(`.app-form [data-form="home"]`),
                label: query(`.app-form [data-form="label"]`),
                additionalOrigins: query(`.app-form [data-form="additional-origins"]`),
            };
        }
        function setFormValues(draft) {
            const elements = getFormElements();
            elements.home.text = draft.home;
            elements.label.text = draft.label;
            elements.additionalOrigins.text = draft.additionalOrigins.join("\n");
        }
        const refreshAndValidateForm = () => {
            const { home, label, additionalOrigins } = getFormElements();
            state.draft = {
                home: home.value,
                label: label.value,
                additionalOrigins: additionalOrigins.value,
            };
            state.problems = validateAppFormDraft(state.draft);
            requestUpdate();
        };
        function handleFormChange() {
            state.problems = [];
            if (!state.formDisabled)
                refreshAndValidateForm();
        }
        async function handleSubmitClick() {
            refreshAndValidateForm();
            state.formDisabled = true;
            requestUpdate();
            try {
                await submit(state.draft);
                if (clearOnSubmit) {
                    initialValues = getEmptyAppFormDraft();
                    setFormValues(initialValues);
                }
                else {
                    initialValues = state.draft;
                }
            }
            finally {
                state.formDisabled = false;
                requestUpdate();
            }
        }
        function render({ partNamespace }) {
            const { formDisabled, draft, problems } = state;
            const changes = !deepEqual(initialValues, draft);
            const submitButtonDisabled = !changes
                || formDisabled
                || problems.length > 0;
            const exportPartsTextInput = `
			label: xiotextinput-label,
			textinput: xiotextinput-textinput,
			problems: xiotextinput-problems,

			label: ${partNamespace}-appform-xiotextinput-label,
			textinput: ${partNamespace}-appform-xiotextinput-textinput,
			problems: ${partNamespace}-appform-xiotextinput-problems,
		`;
            const renderTextInput = ({ hide, textarea, label, dataForm, initialText, showValidationWhenEmpty, parser, validator, }) => y `
			<xio-text-input
				?textarea=${textarea}
				part="${partNamespace}-appform-xiotextinput"
				exportparts="${exportPartsTextInput}"
				data-form="${dataForm}"
				initial="${initialText}"
				?hidden=${hide}
				?disabled=${formDisabled}
				?hide-validation=${!changes}
				?show-validation-when-empty=${showValidationWhenEmpty}
				.parser=${parser}
				.validator=${validator}
				@valuechange=${handleFormChange}>
					<span part=xio-text-input-label>
						${label}
					</span>
			</xio-text-input>
		`;
            return y `
			<div class=app-form>
				<slot name=create-app-heading></slot>

				${renderTextInput({
            hide: false,
            textarea: false,
            dataForm: "label",
            label: "community name",
            initialText: initialValues.label,
            showValidationWhenEmpty: false,
            parser: undefined,
            validator: appDraftValidators.label,
        })}

				${renderTextInput({
            hide: false,
            textarea: false,
            dataForm: "home",
            label: `website homepage, like "https://chasemoskal.com/"`,
            initialText: initialValues.home,
            showValidationWhenEmpty: false,
            parser: undefined,
            validator: appDraftValidators.home,
        })}

				${renderTextInput({
            hide: !showAdditionalOrigins,
            textarea: true,
            dataForm: "additional-origins",
            label: "(optional) additional domain names",
            initialText: initialValues.additionalOrigins.join("\n"),
            showValidationWhenEmpty: true,
            parser: parseOrigins,
            validator: appDraftValidators.additionalOrigins,
        })}

				<xio-button
					class="create-app-button"
					?disabled=${submitButtonDisabled}
					@press=${handleSubmitClick}>
						${submitButtonText}
				</xio-button>
			</div>
		`;
        }
        return { render };
    }

    function adminManagerControls({ app, state, actions, appEditService, }) {
        const { appId } = app;
        const load = (firstStep = Promise.resolve(undefined)) => (ops.operation({
            promise: firstStep
                .then(() => appEditService.listAdmins({ appId })),
            setOp: admins => actions.setAdmins(admins)
        }));
        async function listAdmins() {
            await load();
        }
        async function assignAdmin() {
            const { email } = state.assignerDraft;
            actions.setAssignerDraft({ email: undefined });
            await load(appEditService.assignAdmin({ appId, email }));
        }
        async function revokeAdmin(userId) {
            await load(appEditService.revokeAdmin({ appId, userId }));
        }
        return { listAdmins, assignAdmin, revokeAdmin };
    }

    function whenOpReady(op, render) {
        return ops.isReady(op)
            ? render(ops.value(op))
            : null;
    }

    function renderOp(op, render, more = null) {
        return y `
		<xio-op .op=${op}>
			${whenOpReady(op, render)}
			${more}
		</xio-op>
	`;
    }

    function adminManagerStateAndActions() {
        const auto = autowatcher();
        const state = auto.state({
            admins: ops.none(),
            assignerDraft: {
                email: undefined,
            },
        });
        const actions = auto.actions({
            setAdmins(op) {
                state.admins = op;
            },
            setAssignerDraft(draft) {
                state.assignerDraft = draft;
            },
        });
        return {
            track: auto.track,
            state,
            actions,
        };
    }

    function makeAdminManager({ app, appEditService, query }) {
        const { state, actions, track } = adminManagerStateAndActions();
        const controls = adminManagerControls({
            app,
            state,
            actions,
            appEditService,
        });
        function handleEmailChange(event) {
            const email = event.detail.value ?? undefined;
            actions.setAssignerDraft({ email });
        }
        function handleAssignButtonPress() {
            const textInput = query(".adminassigner xio-text-input");
            controls.assignAdmin();
            textInput.text = "";
        }
        function renderAdminAssigner() {
            const exportPartsTextInput = `
			label: xiotextinput-label,
			textinput: xiotextinput-textinput,
			problems: xiotextinput-problems,

			label: adminmanager-xiotextinput-label,
			textinput: adminmanager-xiotextinput-textinput,
			problems: adminmanager-xiotextinput-problems,
		`;
            return y `
			<div class=adminassigner>
				<xio-text-input
					part=adminmanager-xiotextinput
					exportparts="${exportPartsTextInput}"
					.validator=${emailValidator}
					@valuechange=${handleEmailChange}>
						email
				</xio-text-input>
				<xio-button
					?disabled=${!state.assignerDraft.email}
					@press=${handleAssignButtonPress}>
						grant
				</xio-button>
			</div>
		`;
        }
        function renderAdminList() {
            return y `
			<div class=adminlist>
				${renderOp(state.admins, admins => y `
					<ul>
						${admins.map(({ email, userId }) => y `
							<li>
								<span>${email}</span>
								<xio-button @press=${() => controls.revokeAdmin(userId)}>
									revoke
								</xio-button>
							</li>
						`)}
					</ul>
				`)}
			</div>
		`;
        }
        function render() {
            return y `
			${renderAdminAssigner()}
			${renderAdminList()}
		`;
        }
        return { render, controls, track };
    }

    function dedupe(arr) {
        return [...new Set(arr)];
    }

    function formDraftToAppDraft(formDraft) {
        return {
            home: formDraft.home,
            label: formDraft.label,
            origins: dedupe([
                new URL(formDraft.home).origin,
                ...formDraft.additionalOrigins,
            ])
        };
    }

    function strongRecordKeeper() {
        return function (makeRecord) {
            const map = new Map();
            return function getRecord(key) {
                let record;
                if (map.has(key))
                    record = map.get(key);
                else {
                    record = makeRecord(key);
                    map.set(key, record);
                }
                return record;
            };
        };
    }

    function originsMinusHome(home, origins) {
        return origins
            .filter(o => o.toLowerCase() !== new URL(home.toLowerCase()).origin)
            .map(o => o.toLowerCase());
    }

    function appDisplayToFormDraft(display) {
        return {
            home: display.home,
            label: display.label,
            additionalOrigins: originsMinusHome(display.home, display.origins),
        };
    }

    var __decorate$j = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XiomeAppManager = class XiomeAppManager extends ComponentWithShare {
        constructor() {
            super(...arguments);
            this.appRegistrationForm = makeAppForm({
                clearOnSubmit: true,
                showAdditionalOrigins: false,
                submitButtonText: "create community",
                requestUpdate: () => this.requestUpdate(),
                query: selector => (this.shadowRoot
                    .querySelector(".app-registration")
                    .querySelector(selector)),
                submit: async (formDraft) => {
                    const appDraft = formDraftToAppDraft(formDraft);
                    await this.share.appsModel.registerApp(appDraft);
                },
            });
            this.getAppState = strongRecordKeeper()(appId => {
                const app = this.share.appsModel.getApp(appId);
                const { actions, getState, onStateChange } = happystate({
                    state: { editMode: false },
                    actions: state => ({
                        toggleEditMode() { state.editMode = !state.editMode; },
                    }),
                });
                onStateChange(() => { this.requestUpdate(); });
                return {
                    toggleEditMode: actions.toggleEditMode,
                    get editMode() { return getState().editMode; },
                    appForm: makeAppForm({
                        clearOnSubmit: false,
                        showAdditionalOrigins: true,
                        submitButtonText: "save changes",
                        initialValues: appDisplayToFormDraft(app),
                        requestUpdate: () => { this.requestUpdate(); },
                        query: selector => (this.shadowRoot
                            .querySelector(`.app[data-app-id="${app.appId}"] .app-options`)
                            .querySelector(selector)),
                        submit: async (formDraft) => {
                            const appDraft = formDraftToAppDraft(formDraft);
                            await this.share.appsModel.updateApp(app.appId, appDraft);
                        },
                    }),
                    adminManager: (() => {
                        const manager = makeAdminManager({
                            app,
                            appEditService: this.share.appsModel.appEditService,
                            query: selector => this.shadowRoot
                                .querySelector(`.app[data-app-id="${app.appId}"] .adminmanager`)
                                .querySelector(selector)
                        });
                        manager.track({
                            watcher: () => { manager.render(); },
                            effect: () => this.requestUpdate(),
                        });
                        manager.controls.listAdmins();
                        return manager;
                    })(),
                };
            });
            this.deleteApp = async (app) => {
                const userIsSure = await this.share.modals.confirm({
                    title: "are you certain?",
                    body: `you really want to delete your community "${app.label}"?`,
                    yes: { label: "delete whole community", vibe: "negative" },
                    no: { label: "nevermind", vibe: "neutral" },
                    focusNthElement: 2,
                });
                if (userIsSure)
                    await this.share.appsModel.deleteApp(app.appId);
            };
        }
        init() {
            this.share.appsModel.loadApps();
        }
        renderAppRegistration() {
            return y `
			<div class=app-registration>
				<slot name="register-app-heading"></slot>
				${this.appRegistrationForm.render({ partNamespace: "appregistration" })}
			</div>
		`;
        }
        renderNoApps() {
            return y `
			<slot name=no-apps></slot>
			${this.renderAppRegistration()}
		`;
        }
        renderAppList(records) {
            return y `
			<slot></slot>
			<div class=app-list>
				${Object.entries(records)
            .map(([appId, record]) => renderOp(record, app => this.renderApp(app)))}
			</div>
			${this.renderAppRegistration()}
		`;
        }
        renderAppBankLinking(app) {
            return null;
            // return html`
            // 	<div>
            // 		<h4>bank link to receive payouts</h4>
            // 		<xiome-bank-connect .appId=${app.appId}></xiome-bank-connect>
            // 	</div>
            // `
        }
        renderAppCode(appId) {
            return y `
			<div class=app-code>
				<h4>connect your website</h4>
				<p>copy-paste this html into your website's &lt;head&gt; section:</p>
				<code class=htmlcode>
					${renderXiomeConfig(appId)}
				</code>
				<p>then head over to the <a part=link href="./components">components page</a></p>
			</div>
		`;
        }
        renderApp(app) {
            const appState = this.getAppState(app.appId);
            return y `
			<div class=app data-app-id=${app.appId}>

				<div class=app-header part=app-header>
					<div class=title>
						<h3>
							<a part=link target=_blank href="${app.home}">
								${app.label}
							</a>
						</h3>
					</div>
					<div class=stats>
						<div data-stat=users>
							<span>${app.stats.users.toLocaleString()}</span>
							<span>users</span>
						</div>
						<div data-stat=monthly-active>
							<span>${app.stats.usersActiveMonthly.toLocaleString()}</span>
							<span>monthly active</span>
						</div>
						<div data-stat=active-last-day>
							<span>${app.stats.usersActiveDaily.toLocaleString()}</span>
							<span>daily active</span>
						</div>
					</div>
				</div>

				<div class=twoside>
					${this.renderAppCode(app.appId)}
					<div class=editside>
						<div class=buttons>
							<xio-button @click=${appState.toggleEditMode}>edit community</xio-button>
						</div>
						<div class=app-options ?hidden=${!appState.editMode}>
							<div class=app-details>
								${appState.appForm.render({ partNamespace: "appeditor" })}
							</div>
							<div class=adminmanager>
								<h4>manage admins</h4>
								${appState.adminManager.render()}
							</div>
							${this.renderAppBankLinking(app)}
							<div class=finalbox>
								<xio-button
									class=delete-app-button
									@press=${() => this.deleteApp(app)}>
										delete community
								</xio-button>
							</div>
						</div>
					</div>
				</div>
			</div>
		`;
        }
        render() {
            const { appRecords } = this.share.appsModel.state;
            return y `
			${renderOp(appRecords, records => Object.values(records).length
            ? this.renderAppList(records)
            : this.renderNoApps())}
		`;
        }
    };
    XiomeAppManager = __decorate$j([
        mixinStyles(styles$j)
    ], XiomeAppManager);

    var styles$i = r$3 `

.avatar {
	display: block;
	width: 3em;
	height: 3em;
	padding: 0.2em;
	border-radius: 0.3em;
}

:host([logged-in]) .avatar {
	color: #fff;
}

svg {
	display: block;
	width: 100%;
	height: 100%;
}

`;

    var __decorate$i = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XiomeMyAvatar = class XiomeMyAvatar extends AutowatcherComponentWithShare {
        render() {
            const accessOp = this.share.accessModel.getAccessOp();
            return renderOp(accessOp, access => y `
			<xio-avatar
				.spec=${access?.user?.profile.avatar}
				part="xio-avatar"
			></xio-avatar>
		`);
        }
    };
    XiomeMyAvatar = __decorate$i([
        mixinStyles(styles$i)
    ], XiomeMyAvatar);

    var styles$h = r$3 `

xio-profile-card {
	--avatar-size: 7em;
}

`;

    var __decorate$h = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XiomeMyAccount = class XiomeMyAccount extends ComponentWithShare {
        constructor() {
            super(...arguments);
            this.saveProfile = async (profileDraft) => {
                await this.share.personalModel.saveProfile(profileDraft);
            };
        }
        render() {
            const { accessOp } = this.share.personalModel.readable;
            return renderOp(accessOp, ({ user }) => y `
			<xio-profile-card
				show-details
				.user=${user}
				.saveProfile=${this.saveProfile}
			></xio-profile-card>
		`);
        }
    };
    XiomeMyAccount = __decorate$h([
        mixinStyles(styles$h)
    ], XiomeMyAccount);

    var styles$g = r$3 `

:host {
	display: block;
}

slot {
	display: block;
}

xio-text-input,
xio-button {
	margin-top: 0.2em;
}

xio-text-input > span {
	opacity: 0.4;
	font-size: 0.7em;
}

.buttonbar {
	margin-top: 0.5em;
	vertical-align: middle;
}

slot[name="legal"] {
	display: inline-block;
	margin: 0 1em;
}

small {
	display: block;
	opacity: 0.5;
	font-size: 0.7em;
}

`;

    var __decorate$g = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XiomeLoginPanel = class XiomeLoginPanel extends ComponentWithShare {
        constructor() {
            super(...arguments);
            this["show-logout"] = false;
            this.sentLoading = ops.none();
            this.emailIsValid = false;
        }
        async sendEmail() {
            const email = this.textInput.value;
            await ops.operation({
                promise: this.share.accessModel.sendLoginLink(email)
                    .then(() => ({ email })),
                setOp: op => this.sentLoading = op,
                errorReason: `failed sending email to "${email}"`,
            });
        }
        resetSentLoading() {
            this.sentLoading = ops.none();
            if (this.textInput)
                this.textInput.text = "";
        }
        logout() {
            this.share.accessModel.logout()
                .then(() => this.sentLoading = ops.none());
        }
        subscribe() {
            return this.share.accessModel.subscribe(() => {
                this.resetSentLoading();
            });
        }
        renderLoggedIn(access) {
            return y `
			<slot>
				<p>Welcome ${access.user.profile.nickname}!</p>
			</slot>
			${this["show-logout"]
            ? y `
					<div part=buttonbar>
						<xio-button class=logout-button @press=${this.logout}>
							Logout
						</xio-button>
					</div>
				`
            : null}
		`;
        }
        handleEmailChange() {
            this.emailIsValid = this.textInput.problems.length === 0;
        }
        renderLegalLink() {
            return y `
			<small>
				<p>
					<a
						part=link
						target=_blank
						href="https://xiome.io/legal">
							policies and terms
					</a>
				</p>
			</small>
		`;
        }
        renderLoggedOut() {
            const { emailIsValid } = this;
            return y `
			<xio-op .op=${this.sentLoading}>
				<div slot=none>
					<slot name=logged-out>
						<p>login with your email address</p>
					</slot>
					<xio-text-input
						.validator=${email()}
						@valuechange=${this.handleEmailChange}
						@enterpress=${this.sendEmail}>
							<span>your email</span>
					</xio-text-input>
					<div class=buttonbar>
						<slot name=legal>
							${this.renderLegalLink()}
						</slot>
						<xio-button
							?disabled=${!emailIsValid}
							@press=${this.sendEmail}>
								send login link
						</xio-button>
					</div>
				</div>
				${ops.isReady(this.sentLoading)
            ? y `
						<p>email sent to ${ops.value(this.sentLoading).email}</p>
						<p>please wait a few minutes for it to arrive</p>
						<xio-button @press=${this.resetSentLoading}>
							restart
						</xio-button>
					`
            : null}
			</xio-op>
		`;
        }
        render() {
            const accessOp = this.share.accessModel.getAccessOp();
            return renderOp(accessOp, access => access?.user
                ? this.renderLoggedIn(access)
                : this.renderLoggedOut());
        }
    };
    __decorate$g([
        e({ type: Boolean, reflect: true })
    ], XiomeLoginPanel.prototype, "show-logout", void 0);
    __decorate$g([
        e()
    ], XiomeLoginPanel.prototype, "sentLoading", void 0);
    __decorate$g([
        i("xio-text-input")
    ], XiomeLoginPanel.prototype, "textInput", void 0);
    __decorate$g([
        e({ type: String })
    ], XiomeLoginPanel.prototype, "emailIsValid", void 0);
    XiomeLoginPanel = __decorate$g([
        mixinStyles(styles$g)
    ], XiomeLoginPanel);

    var styles$f = r$3 `

.creator {
	margin: 1em auto;
}

.privilege {
	display: inline-flex;
	flex-direction: row;
	padding: 0.2em 0.5em;
	margin: 0.2em;
	border: 1px solid;
}

.privilege .icon {
	padding-right: 0.5em;
}

.privilege xio-button {
	--xio-button-padding: 0;
	--xio-button-border: none;
}

.privilege[data-hard] {
	opacity: 0.5;
	border: 1px solid dashed;
}

`;

    var wrenchSvg = b `<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" aria-hidden="true" focusable="false" width="1em" height="1em" style="-ms-transform: rotate(360deg); -webkit-transform: rotate(360deg); transform: rotate(360deg);" preserveAspectRatio="xMidYMid meet" viewBox="0 0 16 16"><g fill="none"><path d="M7 5a4 4 0 0 1 5.445-3.73a.5.5 0 0 1 .173.819L10.708 4L12 5.293l1.91-1.91a.5.5 0 0 1 .82.172a4 4 0 0 1-4.829 5.292L4.897 13.92a1.986 1.986 0 0 1-2.843-2.774l5.051-5.234A4.01 4.01 0 0 1 7 5zm4-3a3 3 0 0 0-2.862 3.903a.5.5 0 0 1-.117.498L2.773 11.84a.986.986 0 0 0 1.41 1.377l5.225-5.293a.5.5 0 0 1 .532-.116a3 3 0 0 0 4.046-3.088l-1.633 1.634a.5.5 0 0 1-.707 0l-2-2a.5.5 0 0 1 0-.707l1.634-1.634A3.045 3.045 0 0 0 11 2z" fill="currentColor"/></g></svg>`;

    var cancelSvg = b `<svg xmlns="http://www.w3.org/2000/svg" width="14" height="16" viewBox="0 0 14 16"><path fill="currentColor" fill-rule="evenodd" d="M7 1C3.14 1 0 4.14 0 8s3.14 7 7 7 7-3.14 7-7-3.14-7-7-7zm0 1.3c1.3 0 2.5.44 3.47 1.17l-8 8A5.755 5.755 0 0 1 1.3 8c0-3.14 2.56-5.7 5.7-5.7zm0 11.41c-1.3 0-2.5-.44-3.47-1.17l8-8c.73.97 1.17 2.17 1.17 3.47 0 3.14-2.56 5.7-5.7 5.7z"/></svg>`;

    function renderPrivilege({ privilege: { hard, label, privilegeId }, onDeleteClick, }) {
        function renderSystemIcon() {
            return y `
			<div class="icon system">
				${wrenchSvg}
			</div>
		`;
        }
        function renderDeleteButton() {
            return y `
			<xio-button
				title="delete privilege"
				class="icon delete"
				@press=${onDeleteClick}>
					${cancelSvg}
			</xio-button>
		`;
        }
        return y `
		<div
			class=privilege
			?data-hard=${hard}>
				${hard ? renderSystemIcon() : renderDeleteButton()}
				<span class=text>${label}</span>
		</div>
	`;
    }

    const validatePermissionsLabel = validator$1(string(), minLength(1), maxLength(32));

    var __decorate$f = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __classPrivateFieldSet$3 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var __classPrivateFieldGet$5 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var _XiomePrivileges_instances, _XiomePrivileges_clearCreatorTextInput, _XiomePrivileges_createPrivilege, _XiomePrivileges_deletePrivilege, _XiomePrivileges_labelDraft, _XiomePrivileges_handleCreatorLabelChange, _XiomePrivileges_renderPrivilegeCreator;
    let XiomePrivileges = class XiomePrivileges extends ComponentWithShare {
        constructor() {
            super(...arguments);
            _XiomePrivileges_instances.add(this);
            _XiomePrivileges_labelDraft.set(this, void 0);
            _XiomePrivileges_handleCreatorLabelChange.set(this, (event) => {
                __classPrivateFieldSet$3(this, _XiomePrivileges_labelDraft, event.detail.value, "f");
                this.requestUpdate();
            });
        }
        get model() {
            return this.share.permissionsModel;
        }
        init() {
            this.model.initialize();
        }
        render() {
            const { readable: { permissionsDisplay } } = this.model;
            const allowed = this.model.getUserCanCustomizePermissions();
            return renderOp(permissionsDisplay, permissions => allowed ? y `
			<div class=privileges>
				${__classPrivateFieldGet$5(this, _XiomePrivileges_instances, "m", _XiomePrivileges_renderPrivilegeCreator).call(this)}
				${permissions.privileges.map(privilege => renderPrivilege({
            privilege,
            onDeleteClick: () => __classPrivateFieldGet$5(this, _XiomePrivileges_instances, "m", _XiomePrivileges_deletePrivilege).call(this, privilege)
        }))}
			</div>
		` : y `
			<p>you are not permitted to customize privileges</p>
		`);
        }
    };
    _XiomePrivileges_labelDraft = new WeakMap(), _XiomePrivileges_handleCreatorLabelChange = new WeakMap(), _XiomePrivileges_instances = new WeakSet(), _XiomePrivileges_clearCreatorTextInput = function _XiomePrivileges_clearCreatorTextInput() {
        const input = this.shadowRoot.querySelector(".creator xio-text-input");
        input.text = "";
        input.dispatchValueChange();
    }, _XiomePrivileges_createPrivilege = async function _XiomePrivileges_createPrivilege() {
        const label = __classPrivateFieldGet$5(this, _XiomePrivileges_labelDraft, "f");
        __classPrivateFieldGet$5(this, _XiomePrivileges_instances, "m", _XiomePrivileges_clearCreatorTextInput).call(this);
        await this.model.createPrivilege({ label });
    }, _XiomePrivileges_deletePrivilege = async function _XiomePrivileges_deletePrivilege({ privilegeId, label }) {
        const confirm = await this.share.modals.confirm({
            title: y `delete privilege ${label}?`,
            body: y `are you sure you want to delete this privilege?`,
        });
        if (confirm)
            await this.model.deletePrivilege({ privilegeId });
    }, _XiomePrivileges_renderPrivilegeCreator = function _XiomePrivileges_renderPrivilegeCreator() {
        const isCreateButtonDisabled = !__classPrivateFieldGet$5(this, _XiomePrivileges_labelDraft, "f");
        return y `
			<div class=creator>
				<xio-text-input
					.validator=${validatePermissionsLabel}
					@valuechange=${__classPrivateFieldGet$5(this, _XiomePrivileges_handleCreatorLabelChange, "f")}
					@enterpress=${__classPrivateFieldGet$5(this, _XiomePrivileges_instances, "m", _XiomePrivileges_createPrivilege)}>
						privilege label
				</xio-text-input>
				<xio-button
					?disabled=${isCreateButtonDisabled}
					@press=${__classPrivateFieldGet$5(this, _XiomePrivileges_instances, "m", _XiomePrivileges_createPrivilege)}>
						create privilege
				</xio-button>
			</div>
		`;
    };
    XiomePrivileges = __decorate$f([
        mixinStyles(styles$f)
    ], XiomePrivileges);

    var styles$e = r$3 `

* {
	ZZoutline: 1px solid #0f02;
}

:host {
	display: block;
	width: 100%;
}

.container {
	display: flex;
	flex-direction: row;
	flex-wrap: wrap;
	gap: 0.5em;
}

.container > * {
	flex: 3 1 10em;
	display: flex;
	flex-direction: column;
}

.roles {
	flex: 1 3 10em;
}

.roles xio-button[data-hard] {
	opacity: 0.6;
}

.container > * > p {
	padding: 0 0.5em;
}

[part=plate] {
	flex: 1 1 auto;
	padding: 0.5em;
	background: #fff2;
}

[part=plate] xio-button {
	display: inline-block;
	margin: 0.2em 0.1em;
	--xio-button-disabled-opacity: 0.6;
}

[part=plate] xio-button[data-hard] {
	opacity: 0.6;
}

[part=plate] xio-button[data-selected]::part(button) {
	border: 1px solid lime;
	background: yellow;
	color: black;
}

[part=plate] xio-button > div {
	display: flex;
	flex-direction: row;
}

[part=plate] xio-button .icon {
	margin-right: 0.2em;
}

[part=plate] xio-button svg {
	width: 0.8em;
	height: 0.8em;
}

.buttonbar {
	text-align: right;
	background: #0002;
}

.buttonbar [data-button=delete] {
	--xio-button-hover-color: red;
	--xio-button-hover-background: transparent;
}

.buttonbar [data-button=new] {
	--xio-button-hover-color: lime;
	--xio-button-hover-background: transparent;
}

`;

    var lockSvg = b `<svg xmlns="http://www.w3.org/2000/svg" width="1em" height="1em" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-lock"><rect x="3" y="11" width="18" height="11" rx="2" ry="2"></rect><path d="M7 11V7a5 5 0 0 1 10 0v4"></path></svg>`;

    var __decorate$e = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XiomePermissions = class XiomePermissions extends ComponentWithShare {
        constructor() {
            super(...arguments);
            this.clickRole = (role) => () => {
                this.roleSelected = role;
            };
            this.clickDeleteRole = async () => {
                const { modals, permissionsModel } = this.share;
                const role = this.roleSelected;
                const confirmed = await modals.confirm({
                    title: "Delete role?",
                    body: `Are you sure you want to permanently delete the role "${role.label}"`,
                    yes: { vibe: "negative", label: "Delete role" },
                    no: { vibe: "neutral", label: "Nevermind" },
                    focusNthElement: 2,
                });
                if (confirmed)
                    await permissionsModel.deleteRole({ roleId: role.roleId });
            };
            this.clickNewRole = async () => {
                const { modals, permissionsModel } = this.share;
                const result = await modals.prompt({
                    title: "Create a new role",
                    input: {
                        label: "Role name",
                        validator: roleLabelValidator,
                    },
                    yes: { vibe: "positive", label: "Create role" }
                });
                if (result)
                    await permissionsModel.createRole({ label: result.value });
            };
            this.clickAvailablePrivilege = (privilegeId) => async () => {
                const { roleSelected } = this;
                if (roleSelected)
                    await this.share.permissionsModel.assignPrivilege({
                        privilegeId,
                        roleId: roleSelected.roleId,
                    });
            };
            this.clickAssignedPrivilege = (privilegeId) => async () => {
                const { roleSelected } = this;
                if (roleSelected)
                    await this.share.permissionsModel.unassignPrivilege({
                        privilegeId,
                        roleId: roleSelected.roleId,
                    });
            };
        }
        init() {
            this.share.permissionsModel.initialize();
        }
        getAssignedPrivileges(permissions) {
            const { roleSelected } = this;
            if (!roleSelected)
                return [];
            const assignedPrivilegeIds = permissions.rolesHavePrivileges
                .filter(({ roleId }) => roleId === roleSelected.roleId)
                .map(({ privilegeId }) => privilegeId);
            return permissions.privileges
                .filter(({ privilegeId }) => assignedPrivilegeIds.includes(privilegeId))
                .map(privilege => {
                const { active, immutable } = permissions.rolesHavePrivileges.find(rp => rp.roleId === roleSelected.roleId &&
                    rp.privilegeId === privilege.privilegeId);
                return { ...privilege, active, immutable };
            });
        }
        renderPrivilege({ privilegeId, label, hard, immutable, onPrivilegeClick, }) {
            return y `
			<xio-button
				title="${privilegeId}"
				?disabled=${immutable}
				?data-hard=${hard}
				?data-soft=${!hard}
				?data-immutable=${immutable}
				@press=${onPrivilegeClick}>
					<div>
						${hard
            ? y `<div class=icon>${wrenchSvg}</div>`
            : null}
						${immutable
            ? y `<div class=icon>${lockSvg}</div>`
            : null}
						${label}
					</div>
			</xio-button>
		`;
        }
        renderPermissions(permissions) {
            const assignedPrivileges = this.getAssignedPrivileges(permissions);
            const activePrivileges = assignedPrivileges.filter(p => p.active);
            const availablePrivileges = this.roleSelected
                ? [
                    ...permissions.privileges
                        .filter(privilege => {
                        const assigned = assignedPrivileges
                            .find(priv => priv.privilegeId === privilege.privilegeId);
                        return !assigned;
                    })
                        .map(privilege => ({ ...privilege, immutable: false })),
                    ...assignedPrivileges
                        .filter(privilege => !privilege.active)
                ]
                : [];
            return y `
			<div class=container>
				<div class=roles>
					<p>roles</p>
					<div part=plate>
						${permissions.roles.map(role => y `
							<xio-button
								title="${role.roleId}"
								?data-selected=${this.roleSelected &&
            role.roleId === this.roleSelected.roleId}
								?data-hard=${role.hard}
								?disabled=${this.roleSelected &&
            role.roleId === this.roleSelected.roleId}
								@click=${this.clickRole(role)}>
								<div>
									${role.hard
            ? y `<div class=icon>${wrenchSvg}</div>`
            : null}
									${role.label}
								</div>
							</xio-button>
						`)}
					</div>
					<div part=plate class=buttonbar>
						${this.roleSelected
            ? y `
								<xio-button
									data-button=delete
									?disabled=${this.roleSelected.hard}
									@press=${this.clickDeleteRole}>
										delete role
								</xio-button>
							`
            : null}
						<xio-button data-button=new @press=${this.clickNewRole}>
							new role
						</xio-button>
					</div>
				</div>

				<div class=assigned>
					<p>
						privileges assigned
						${this.roleSelected
            ? ` to "${this.roleSelected.label}"`
            : null}
					</p>
					<div part=plate>
						${activePrivileges.map(privilege => this.renderPrivilege({
            ...privilege,
            onPrivilegeClick: this.clickAssignedPrivilege(privilege.privilegeId)
        }))}
					</div>
				</div>

				<div class=available>
					<p>privileges available</p>
					<div part=plate>
						${availablePrivileges.map(privilege => this.renderPrivilege({
            ...privilege,
            onPrivilegeClick: this.clickAvailablePrivilege(privilege.privilegeId),
        }))}
					</div>
				</div>
			</div>
		`;
        }
        render() {
            const { permissionsModel } = this.share;
            const { getUserCanCustomizePermissions, readable } = permissionsModel;
            return getUserCanCustomizePermissions()
                ? renderOp(readable.permissionsDisplay, this.renderPermissions.bind(this))
                : y `
				<p>you are not privileged to customize permissions</p>
			`;
        }
    };
    __decorate$e([
        e()
    ], XiomePermissions.prototype, "roleSelected", void 0);
    XiomePermissions = __decorate$e([
        mixinStyles(styles$e)
    ], XiomePermissions);

    function integrateAuthComponents({ models, modals }) {
        const { accessModel, appsModel, personalModel, permissionsModel } = models;
        return {
            XiomeMyAvatar: mixinSnapstateSubscriptions(accessModel.subscribe)(mixinShare({
                accessModel,
            })(XiomeMyAvatar)),
            XiomeLoginPanel: mixinSnapstateSubscriptions(accessModel.subscribe)(mixinShare({
                accessModel,
            })(XiomeLoginPanel)),
            XiomeAppManager: mixinSnapstateSubscriptions(appsModel.subscribe)(mixinShare({
                modals,
                appsModel,
            })(XiomeAppManager)),
            XiomeMyAccount: mixinSnapstateSubscriptions(personalModel.subscribe)(mixinShare({
                personalModel,
            })(XiomeMyAccount)),
            XiomePermissions: mixinSnapstateSubscriptions(permissionsModel.subscribe)(mixinShare({
                modals,
                permissionsModel,
            })(XiomePermissions)),
            XiomePrivileges: mixinSnapstateSubscriptions(permissionsModel.subscribe)(mixinShare({
                modals,
                permissionsModel,
            })(XiomePrivileges)),
        };
    }

    function formatDate(milliseconds) {
        const d = new Date(milliseconds);
        const twoDigit = (n) => n.toString().padStart(2, "0");
        const year = twoDigit(d.getFullYear());
        const month = twoDigit(d.getMonth() + 1);
        const day = twoDigit(d.getDate());
        const hours24 = d.getHours();
        let hours = d.getHours();
        hours %= 12;
        hours = hours ? hours : 12;
        const minutes = twoDigit(d.getMinutes());
        const ampm = hours24 >= 12 ? "pm" : "am";
        const timezoneOffsetMinutes = -d.getTimezoneOffset();
        const timezoneOffset = (timezoneOffsetMinutes / 60).toFixed(timezoneOffsetMinutes % 60 === 0
            ? 0
            : 1);
        const timezoneUtc = timezoneOffsetMinutes === 0
            ? "UTC"
            : `UTC${timezoneOffsetMinutes < 0 ? "" : "+"}${timezoneOffset}`;
        const timezoneName = Intl.DateTimeFormat().resolvedOptions().timeZone;
        const timezone = `${timezoneName} ${timezoneUtc}`;
        const date = `${year}-${month}-${day}`;
        const time = `${hours}:${minutes} ${ampm}`;
        const zone = `${timezone}`;
        const full = `${date} ${time} (${zone})`;
        return {
            date,
            time,
            zone,
            full,
        };
    }

    var styles$d = r$3 `

:host {
	display: block;
}

`;

    var __decorate$d = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __classPrivateFieldSet$2 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var __classPrivateFieldGet$4 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var _XiomeVideoHosting_linkFailed, _XiomeVideoHosting_apiKeyDraft, _XiomeVideoHosting_handleInputChange, _XiomeVideoHosting_handleLinkClick, _XiomeVideoHosting_handleUnlinkClick, _XiomeVideoHosting_renderWhenUnlinked, _XiomeVideoHosting_renderWhenLinked;
    let XiomeVideoHosting = class XiomeVideoHosting extends ComponentWithShare {
        constructor() {
            super(...arguments);
            _XiomeVideoHosting_linkFailed.set(this, false);
            _XiomeVideoHosting_apiKeyDraft.set(this, "");
            _XiomeVideoHosting_handleInputChange.set(this, ({ detail: { value } }) => {
                __classPrivateFieldSet$2(this, _XiomeVideoHosting_apiKeyDraft, value, "f");
                this.requestUpdate();
            });
            _XiomeVideoHosting_handleLinkClick.set(this, async () => {
                const apiKey = __classPrivateFieldGet$4(this, _XiomeVideoHosting_apiKeyDraft, "f");
                if (apiKey) {
                    __classPrivateFieldSet$2(this, _XiomeVideoHosting_linkFailed, false, "f");
                    __classPrivateFieldSet$2(this, _XiomeVideoHosting_apiKeyDraft, "", "f");
                    const link = await this.share.dacastModel.linkAccount({ apiKey });
                    __classPrivateFieldSet$2(this, _XiomeVideoHosting_linkFailed, !link, "f");
                }
            });
            _XiomeVideoHosting_handleUnlinkClick.set(this, async () => {
                __classPrivateFieldSet$2(this, _XiomeVideoHosting_apiKeyDraft, "", "f");
                await this.share.dacastModel.unlinkAccount();
            });
            _XiomeVideoHosting_renderWhenUnlinked.set(this, () => {
                return y `
			<h2>link your dacast account</h2>
			<div class=linkbox>
				<xio-text-input
					placeholder="api key"
					.validator=${validateDacastApiKeyAllowingMock}
					@enterpress=${__classPrivateFieldGet$4(this, _XiomeVideoHosting_handleLinkClick, "f")}
					@valuechange=${__classPrivateFieldGet$4(this, _XiomeVideoHosting_handleInputChange, "f")}>
				</xio-text-input>
				<xio-button
					?disabled=${!__classPrivateFieldGet$4(this, _XiomeVideoHosting_apiKeyDraft, "f")}
					@press=${__classPrivateFieldGet$4(this, _XiomeVideoHosting_handleLinkClick, "f")}>
						link
				</xio-button>
			</div>
			${__classPrivateFieldGet$4(this, _XiomeVideoHosting_linkFailed, "f")
                ? y `<p class=failed>dacast rejected the api link</p>`
                : null}
			<div class=helpbox>
				<p>how to find your dacast api key</p>
				<ul>
					<li>create a <a part=link target=_blank href="https://dacast.com/">dacast</a> account</li>
					<li>if you have a trial account, you must email support and ask them to activate your account's "api access"</li>
					<li>generate an api key in your <a part=link target=_blank href="https://app.dacast.com/settings/integrations">dacast integrations settings</a></li>
				</ul>
			</div>
		`;
            });
            _XiomeVideoHosting_renderWhenLinked.set(this, (linkedAccount) => {
                return y `
			<h2>your dacast account is linked</h2>
			<p>linked on ${formatDate(linkedAccount.time).full}</p>
			<xio-button @press=${__classPrivateFieldGet$4(this, _XiomeVideoHosting_handleUnlinkClick, "f")}>unlink</xio-button>
		`;
            });
        }
        get state() {
            return this.share.dacastModel.state;
        }
        async init() {
            await this.share.dacastModel.initialize();
        }
        render() {
            return renderOp(this.state.accessOp, access => y `
			<div class=dacastbox>
				${access.permit.privileges.includes(videoPrivileges["moderate videos"])
            ? renderOp(this.state.linkedAccountOp, linkedAccount => linkedAccount
                ? __classPrivateFieldGet$4(this, _XiomeVideoHosting_renderWhenLinked, "f").call(this, linkedAccount)
                : __classPrivateFieldGet$4(this, _XiomeVideoHosting_renderWhenUnlinked, "f").call(this))
            : y `<slot name=forbidden>you don't have permission to edit video hosting settings</slot>`}
			</div>
		`);
        }
    };
    _XiomeVideoHosting_linkFailed = new WeakMap(), _XiomeVideoHosting_apiKeyDraft = new WeakMap(), _XiomeVideoHosting_handleInputChange = new WeakMap(), _XiomeVideoHosting_handleLinkClick = new WeakMap(), _XiomeVideoHosting_handleUnlinkClick = new WeakMap(), _XiomeVideoHosting_renderWhenUnlinked = new WeakMap(), _XiomeVideoHosting_renderWhenLinked = new WeakMap();
    XiomeVideoHosting = __decorate$d([
        mixinStyles(styles$d)
    ], XiomeVideoHosting);

    var styles$c = r$3 `

:host {
	display: block;
}

iframe {
	width: 100%;
	height: 100%;
	border: none;
}

`;

    function renderView({ view, onDeleteClick, getPrivilegeDisplay }) {
        return y `
		<div class="view">
			<p>label: ${view.label}</p>
			<p>provider: ${view.provider}</p>
			<p>type: ${view.type}</p>
			<p>id: ${view.id}</p>
			<p>privileges:</p>
			<ul>
				${view.privileges.map(getPrivilegeDisplay)
        .map(privilege => privilege ? y `
						<li>${privilege.label}</li>
					` : null)}
			</ul>
			<xio-button @press=${onDeleteClick}>delete view</xio-button>
		</div>
	`;
    }

    function renderViewCreator({ isContentSelected, isCreateButtonDisabled, privilegesOp, catalogOp, queryAll, onCatalogSelect, onPrivilegesSelect, onCreateClick, }) {
        const onCatalogSelectChange = (event) => {
            const target = event.target;
            onCatalogSelect(parseInt(target.value));
        };
        const onPrivilegesSelectChange = () => {
            onPrivilegesSelect(queryAll(".create-privileges select option")
                .filter(option => option.selected)
                .map(option => option.value));
        };
        function renderContentSelector() {
            const catalog = ops.value(catalogOp);
            return y `
			<div class=create-content>
				${catalog.length ? y `
					<p>select content for this view</p>
					<select @change=${onCatalogSelectChange}>
						<option disabled selected>(select video content)</option>
						${catalog.map(({ provider, type, title }, index) => y `
							<option value=${index}>
								${`${provider} ${type} ${title}`}
							</option>
						`)}
					</select>
				` : y `
					<p>no available video content (are your video accounts linked?)</p>
				`}
			</div>
		`;
        }
        function renderPrivilegeSelector() {
            const privileges = ops.value(privilegesOp);
            return y `
			<div class=create-privileges>
				<p>select which privileges have access</p>
				<select multiple @change=${onPrivilegesSelectChange}>
					${privileges.map(privilege => y `
						<option value="${privilege.privilegeId}">${privilege.label}</option>
					`)}
				</select>
			</div>
		`;
        }
        return renderOp(ops.combine(catalogOp, privilegesOp), () => y `
		<div class=create-view>
			<p>create view</p>
			${renderContentSelector()}
			${isContentSelected
        ? renderPrivilegeSelector()
        : null}
			<xio-button
				class=create-button
				?disabled=${isCreateButtonDisabled}
				@press=${onCreateClick}>
					create view
			</xio-button>
		</div>
	`);
    }

    function videoControls({ queryAll, contentModel: model, requestUpdate, }) {
        const { readable, writable, subscribe } = snapstate({
            open: false,
            selectedContent: undefined,
            selectedPrivileges: [],
        });
        const toggleControls = () => {
            writable.open = !writable.open;
        };
        function render(label) {
            const currentView = model.getView(label);
            const otherViews = model.views
                .filter(view => view.label !== label);
            return y `
			<h2>
				<span>video display controls</span>
				<xio-button @press=${toggleControls}>
					${readable.open ? "close" : "open"}
				</xio-button>
			</h2>
			${readable.open ? y `
				<p>this view <em>"${label}"</em></p>
				${currentView
            ? renderView({
                view: currentView,
                onDeleteClick: () => model.deleteView(label),
                getPrivilegeDisplay: id => model.getPrivilege(id),
            })
            : renderViewCreator({
                queryAll,
                catalogOp: model.state.catalogOp,
                privilegesOp: model.state.privilegesOp,
                isContentSelected: readable.selectedContent !== undefined,
                isCreateButtonDisabled: readable.selectedContent === undefined
                    || readable.selectedPrivileges.length === 0,
                onCatalogSelect: index => {
                    writable.selectedContent = index;
                },
                onPrivilegesSelect: privileges => {
                    writable.selectedPrivileges = privileges;
                },
                onCreateClick: () => {
                    const { selectedContent, selectedPrivileges } = readable;
                    writable.selectedContent = undefined;
                    writable.selectedPrivileges = [];
                    const content = model.catalog[selectedContent];
                    model.setView({
                        label,
                        privileges: selectedPrivileges,
                        reference: {
                            id: content.id,
                            type: content.type,
                            provider: content.provider,
                        },
                    });
                }
            })}
				${otherViews.length
            ? y `
						<p>all other views</p>
						<div class="otherviews">
							${otherViews.map(view => renderView({
                view,
                onDeleteClick: () => model.deleteView(view.label),
                getPrivilegeDisplay: id => model.getPrivilege(id),
            }))}
						</div>
					`
            : null}
			` : null}
		`;
        }
        return {
            render,
            subscribe: () => subscribe(requestUpdate),
        };
    }

    const dacastIframeRegex = /"(https:\/\/iframe\.dacast\.com\/(\S+)\/(\S+))"/i;
    function parseDacastIframeSrc(code) {
        const parts = code.match(dacastIframeRegex);
        if (!parts)
            throw new Error(`failed to parse dacast iframe embed code: make-dacast-iframe likely needs to be updated`);
        const [, , resource, id] = parts;
        const fixedResource = resource === "playlists"
            ? "playlist"
            : resource;
        return `https://iframe.dacast.com/${fixedResource}/${id}`;
    }

    var __decorate$c = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __classPrivateFieldGet$3 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var _XiomeVideoDisplay_instances, _XiomeVideoDisplay_videoControls, _XiomeVideoDisplay_embeds, _XiomeVideoDisplay_renderShow;
    let XiomeVideoDisplay = class XiomeVideoDisplay extends ComponentWithShare {
        constructor() {
            super(...arguments);
            _XiomeVideoDisplay_instances.add(this);
            this.label = "default";
            _XiomeVideoDisplay_videoControls.set(this, (() => {
                const controls = videoControls({
                    contentModel: this.model,
                    requestUpdate: () => this.requestUpdate(),
                    queryAll: s => Array.from(this.shadowRoot.querySelectorAll(s)),
                });
                this.addSubscription(controls.subscribe);
                return controls;
            })());
            _XiomeVideoDisplay_embeds.set(this, (() => {
                const map = new Map();
                return {
                    obtain(details) {
                        if (details.provider !== "dacast")
                            throw new Error(`unsupported video provider "${details.provider}"`);
                        let div = map.get(details.id);
                        if (!div) {
                            div = document.createElement("div");
                            div.setAttribute("data-id", details.id);
                            const iframe = document.createElement("iframe");
                            iframe.src = parseDacastIframeSrc(details.embed);
                            iframe.setAttribute("part", "iframe");
                            div.appendChild(iframe);
                            map.set(details.id, div);
                        }
                        return div;
                    },
                };
            })());
        }
        get model() {
            return this.share.contentModel;
        }
        async init() {
            await this.model.initialize(this.label);
        }
        render() {
            return renderOp(this.model.state.accessOp, () => y `
			${this.model.allowance.canModerateVideos
            ? __classPrivateFieldGet$3(this, _XiomeVideoDisplay_videoControls, "f").render(this.label)
            : null}
			${__classPrivateFieldGet$3(this, _XiomeVideoDisplay_instances, "m", _XiomeVideoDisplay_renderShow).call(this)}
		`);
        }
    };
    _XiomeVideoDisplay_videoControls = new WeakMap(), _XiomeVideoDisplay_embeds = new WeakMap(), _XiomeVideoDisplay_instances = new WeakSet(), _XiomeVideoDisplay_renderShow = function _XiomeVideoDisplay_renderShow() {
        const show = this.model.getShow(this.label);
        return show?.details ? y `
			<slot name=title>${show.details.title}</slot>
			${__classPrivateFieldGet$3(this, _XiomeVideoDisplay_embeds, "f").obtain(show.details) ?? "(embed missing)"}
			<slot></slot>
		` : y `
			<slot name=no-show></slot>
		`;
    };
    __decorate$c([
        e({ type: String })
    ], XiomeVideoDisplay.prototype, "label", void 0);
    XiomeVideoDisplay = __decorate$c([
        mixinStyles(styles$c)
    ], XiomeVideoDisplay);

    function integrateVideoComponents({ models }) {
        const { dacastModel, contentModel } = models.videoModels;
        return {
            XiomeVideoHosting: (mixinSnapstateSubscriptions(dacastModel.subscribe)(mixinShare({
                dacastModel,
            })(XiomeVideoHosting))),
            XiomeVideoDisplay: (mixinSnapstateSubscriptions(contentModel.subscribe)(mixinShare({
                contentModel,
            })(XiomeVideoDisplay))),
        };
    }

    var styles$b = r$3 `

button {
	font-size: unset;
	color: unset;
	background: transparent;
}

.container {
	position: relative;
	display: flex;
	flex-direction: row;
	user-select: none;
	cursor: pointer;
	border: none;
}

:host {
	display: inline-block;
	position: relative;
}

:host::before {
	opacity: 0;
	content: "copied!";
	display: block;
	position: absolute;
	top: -0.5em;
	right: -0.5em;
	background: #0a0c;
	padding: 0 0.5em;
	border-radius: 1em;
	pointer-events: none;
	z-index: 1;
}

:host(:not([copied]))::before {
	transition: opacity 500ms ease;
}

:host([copied])::before {
	opacity: 1;
}

.content {
	display: flex;
	justify-content: row;
}

.label::after {
	content: ":";
	margin-right: 0.2em;
}

.id {
	opacity: 0.75;
	max-width: var(--id-max-width, 6em);
	overflow: hidden;
	text-overflow: ellipsis;
}

.copy svg {
	width: 1em;
	height: 1em;
}

.copy {
	opacity: 0.4;
	transition: opacity 100ms linear;
}

.container:hover .copy {
	opacity: 0.9;
}

.container:active .copy {
	opacity: 1;
}

`;

    var clipboardIcon = b `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-clipboard"><path d="M16 4h2a2 2 0 012 2v14a2 2 0 01-2 2H6a2 2 0 01-2-2V6a2 2 0 012-2h2"/><rect x="8" y="2" width="8" height="4" rx="1" ry="1"/></svg>`;

    var __decorate$b = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XioId = class XioId extends AutowatcherComponent {
        constructor() {
            super(...arguments);
            this.copied = false;
        }
        async copy() {
            try {
                await navigator.clipboard.writeText(this.id);
                const activeElement = document.activeElement;
                if (activeElement)
                    activeElement.blur();
                if (this.copyTimeout) {
                    clearTimeout(this.copyTimeout);
                }
                this.copied = true;
                this.copyTimeout = setTimeout(() => {
                    this.copied = false;
                    this.copyTimeout = undefined;
                }, 1000);
            }
            catch (error) {
                console.error("failed to copy");
            }
        }
        render() {
            return y `
			<button class=container @click=${this.copy}>
				<div class=id>${this.id}</div>
				<div class=copy>
					${clipboardIcon}
				</div>
			</button>
		`;
        }
    };
    __decorate$b([
        e()
    ], XioId.prototype, "id", void 0);
    __decorate$b([
        e({ type: Boolean, reflect: true })
    ], XioId.prototype, "copied", void 0);
    XioId = __decorate$b([
        mixinFocusable,
        mixinStyles(styles$b)
    ], XioId);

    var styles$a = r$3 `

svg, span {
	vertical-align: middle;
}

svg {
	width: 1.2em;
	height: 1.2em;
}

slot[name=loading] svg {
	animation:
		spin 10s linear infinite,
		fade 500ms ease infinite alternate;
}

slot[name=error] * {
	color: var(--loading-error-color, red);
}

@keyframes spin {
	from { transform: rotate(0deg); }
	to { transform: rotate(360deg); }
}

@keyframes fade {
	from { opacity: 1.0; }
	to { opacity: 0.5; }
}

`;

    var warningSvg = b `<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" aria-hidden="true" focusable="false" width="1em" height="1em" style="-ms-transform: rotate(360deg); -webkit-transform: rotate(360deg); transform: rotate(360deg);" preserveAspectRatio="xMidYMid meet" viewBox="0 0 20 20"><g fill="none"><path d="M10 7a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0v-4A.5.5 0 0 1 10 7z" fill="#626262"/><path d="M10 14.5a.75.75 0 1 0 0-1.5a.75.75 0 0 0 0 1.5z" fill="#626262"/><path d="M8.686 2.852a1.5 1.5 0 0 1 2.628 0l6.56 11.925A1.5 1.5 0 0 1 16.558 17H3.44a1.5 1.5 0 0 1-1.314-2.223L8.686 2.852zm1.752.482a.5.5 0 0 0-.876 0L3.003 15.26a.5.5 0 0 0 .438.741H16.56a.5.5 0 0 0 .438-.74L10.438 3.333z" fill="currentColor"/></g></svg>`;

    var spinnerIcon = b `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-loader"><line x1="12" y1="2" x2="12" y2="6"></line><line x1="12" y1="18" x2="12" y2="22"></line><line x1="4.93" y1="4.93" x2="7.76" y2="7.76"></line><line x1="16.24" y1="16.24" x2="19.07" y2="19.07"></line><line x1="2" y1="12" x2="6" y2="12"></line><line x1="18" y1="12" x2="22" y2="12"></line><line x1="4.93" y1="19.07" x2="7.76" y2="16.24"></line><line x1="16.24" y1="7.76" x2="19.07" y2="4.93"></line></svg>`;

    var __decorate$a = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XioOp = class XioOp extends AutowatcherComponent {
        constructor() {
            super(...arguments);
            this.mode = "none";
            // @property({type: Object})
            // op: Op<xPayload> = ops.none()
            this._op = ops.none();
            this.errorIcon = warningSvg;
            this.loadingIcon = spinnerIcon;
            this["loading-message"] = "loading...";
            this["error-message"] = "error";
        }
        get op() {
            return this._op;
        }
        set op(op) {
            const old = this._op;
            this._op = op;
            this.mode = ops.mode(op);
            this.requestUpdate("op", old);
        }
        render() {
            return ops.select(this.op, {
                none: () => y `
				<slot name=none></slot>
			`,
                loading: () => y `
				<slot name=loading>
					${this.loadingIcon}
					<span>${this["loading-message"]}</span>
				</slot>
			`,
                error: reason => y `
				<slot name=error>
					${this.errorIcon}
					<span>${reason ?? this["fallbackErrorMessage"]}</span>
				</slot>
			`,
                ready: () => y `
				<slot></slot>
			`,
            });
        }
    };
    __decorate$a([
        e({ type: String, reflect: true })
    ], XioOp.prototype, "mode", void 0);
    __decorate$a([
        e({ type: Object })
    ], XioOp.prototype, "op", null);
    __decorate$a([
        e()
    ], XioOp.prototype, "errorIcon", void 0);
    __decorate$a([
        e()
    ], XioOp.prototype, "loadingIcon", void 0);
    __decorate$a([
        e({ type: String })
    ], XioOp.prototype, "loading-message", void 0);
    __decorate$a([
        e({ type: String })
    ], XioOp.prototype, "error-message", void 0);
    XioOp = __decorate$a([
        mixinStyles(styles$a)
    ], XioOp);

    var styles$9 = r$3 `

:host {
	display: inline-block;
	width: var(--avatar-size, 3em);
	height: var(--avatar-size, 3em);
	border-radius: var(--avatar-border-radius, 0.3em);
	overflow: hidden;
	user-select: none;
}

.avatar {
	display: block;
	width: 100%;
	height: 100%;
	color: var(--xio-avatar-color, #444);
	background: var(--xio-avatar-background, #888);
}

.avatar[data-logged-in] {
	color: #fff;
}

svg, img {
	pointer-events: none;
	display: block;
	width: 100%;
	height: 100%;
	object-fit: cover;
}

`;

    var svgSilhouette = b `<svg xmlns="http://www.w3.org/2000/svg" width="12" height="16" viewBox="0 0 12 16"><path fill="currentColor" fill-rule="evenodd" d="M12 14.002a.998.998 0 0 1-.998.998H1.001A1 1 0 0 1 0 13.999V13c0-2.633 4-4 4-4s.229-.409 0-1c-.841-.62-.944-1.59-1-4 .173-2.413 1.867-3 3-3s2.827.586 3 3c-.056 2.41-.159 3.38-1 4-.229.59 0 1 0 1s4 1.367 4 4v1.002z"/></svg>`;

    function cap(x, min, max) {
        return x < min
            ? min
            : x > max
                ? max
                : x;
    }
    function hsl(xHue, xSaturation, xLightness) {
        const hue = xHue % 360;
        const saturation = cap(xSaturation, 0, 100);
        const lightness = cap(xLightness, 0, 100);
        return `hsl(${hue}, ${saturation}%, ${lightness}%)`;
    }
    function getNiceColors(value) {
        const base = Math.ceil(value * 360);
        const split_a = (base - 30) % 360;
        const split_b = (base + 30) % 360;
        return {
            color1: hsl(base, 100, 90),
            color2: hsl(split_a, 50, 60),
            color3: hsl(split_b, 50, 40),
        };
    }

    var __decorate$9 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XioAvatar = class XioAvatar extends Component {
        renderBlankAvatar(spec) {
            return y `
			<div class=avatar>
				${svgSilhouette}
			</div>
		`;
        }
        renderSimpleAvatar({ value }) {
            const { color1, color2, color3 } = getNiceColors(value);
            const style = `color: ${color1}; background: linear-gradient(to bottom right, ${color2}, ${color3});`;
            return y `
			<div class=avatar style=${style}>
				${svgSilhouette}
			</div>
		`;
        }
        renderImageAvatar({ link }) {
            return y `
			<div class=avatar>
				<img src="${link}" alt=""/>
			</div>
		`;
        }
        render() {
            const { spec = { type: "blank" } } = this;
            switch (spec.type) {
                case "blank":
                    return this.renderBlankAvatar(spec);
                case "simple":
                    return this.renderSimpleAvatar(spec);
                case "image":
                    return this.renderImageAvatar(spec);
                default:
                    return y `avatar missing`;
            }
        }
    };
    __decorate$9([
        e({ type: Object })
    ], XioAvatar.prototype, "spec", void 0);
    XioAvatar = __decorate$9([
        mixinStyles(styles$9)
    ], XioAvatar);

    var styles$8 = r$3 `

:host {
	display: inline-block;
	--_padding: var(--xio-button-padding, 0.2em 0.5em);
	--_background: var(--xio-button-background, transparent);
	--_border: var(--xio-button-border, 1px solid);
	--_border-radius: var(--xio-button-border-radius, 0);
	--_opacity: var(--xio-button-opacity, 0.7);
	--_hover-opacity: var(--xio-button-hover-opacity, 1);
	--_hover-color: var(--xio-button-hover-color, inherit);
	--_hover-background: var(--xio-button-hover-background, var(--_background));
	--_disabled-opacity: var(--xio-button-disabled-opacity, 0.2);
	--_disabled-border-style: var(--xio-button--disabled-border-style, dashed);
}

button {
	cursor: pointer;
	font: inherit;
	color: inherit;
	padding: var(--_padding);
	font-weight: bold;
	background: var(--_background);
	border: var(--_border);
	border-radius: var(--_border-radius);
	opacity: var(--_opacity);
}

button:not([disabled]):hover,
button:not([disabled]):focus {
	opacity: var(--_hover-opacity);
	color: var(--_hover-color);
	background: var(--_hover-background);
}

button[disabled] {
	cursor: default;
	opacity: var(--_disabled-opacity);
	border-style: var(--_disabled-border-style);
}

`;

    class PressEvent extends CustomEvent {
        constructor(button) {
            super("press", { detail: button });
        }
    }

    var __decorate$8 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XioButton = class XioButton extends AutowatcherComponent {
        constructor() {
            super(...arguments);
            this["disabled"] = false;
            this.handleClick = () => {
                const event = new PressEvent(this);
                if (this.onpress)
                    this.onpress(event);
                if (!this.disabled)
                    this.dispatchEvent(event);
                event.stopPropagation();
            };
        }
        focus() {
            this.shadowRoot.querySelector("button").focus();
        }
        render() {
            return y `
			<button
				part=button
				@click=${this.handleClick}
				?disabled=${this.disabled}>
					<slot></slot>
			</button>
		`;
        }
    };
    __decorate$8([
        e({ type: Boolean, reflect: true })
    ], XioButton.prototype, "disabled", void 0);
    __decorate$8([
        e({ type: Function })
    ], XioButton.prototype, "onpress", void 0);
    XioButton = __decorate$8([
        mixinFocusable,
        mixinStyles(styles$8)
    ], XioButton);

    var styles$7 = r$3 `

.example {
	color: #f90;
}

`;

    var __decorate$7 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XioExample = class XioExample extends AutowatcherComponent {
        render() {
            return y `
			<div class=example>
				<p>xio-example</p>
				<slot></slot>
			</div>
		`;
        }
    };
    XioExample = __decorate$7([
        mixinStyles(styles$7)
    ], XioExample);

    var styles$6 = r$3 `

* {box-sizing: border-box;}

:host {
	display: inline-block;
	vertical-align: middle;
	color: white;
}

:host([disabled]) {
	opacity: 0.25;
}

button {
	display: inline-block;
	border: unset;
	color: unset;
	background: unset;
}

button {
	position: relative;
	width: 1.5em;
	height: 1.5em;
	background: #3338;
	box-shadow: inset 1px 2px 3px #000a;
	border-radius: 0.5em;
	border: 1px solid #fff4;
}

button[data-mode="ready"] {
	cursor: pointer;
}

button[data-mode="ready"]:hover,
button[data-mode="ready"]:focus {
	background: #5558;
	border: 1px solid #fff8;
}

button > span {
	display: inline-block;
}

button svg {
	width: 100%;
	height: 100%;
}

button .error {
	display: inline-block;
	position: absolute;
	top: 80%;
	left: 0;
	width: max-content;
	max-width: 16em;
	color: red;
	background: #200a;
	text-shadow: 1px 2px 3px #0008;
	text-align: left;
	padding: 0.2em 0.5em;
	border-radius: 0.5em;
	pointer-events: none;
}

button[data-mode="loading"] svg,
button[data-mode="error"] svg {
	padding: 10%;
}

button[data-mode="error"] svg {
	color: red;
	animation: fade 1s ease infinite alternate;
}

button[data-mode="loading"] svg {
	animation:
		spin 10s linear infinite,
		fade 500ms ease infinite alternate;
}

@keyframes spin {
	from { transform: rotate(0deg); }
	to { transform: rotate(360deg); }
}

@keyframes fade {
	from { opacity: 1.0; }
	to { opacity: 0.5; }
}

`;

    class CheckEvent extends CustomEvent {
        constructor(checkbox) {
            super("press", { detail: checkbox });
        }
    }

    var checkIcon = b `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-check"><polyline points="20 6 9 17 4 12"></polyline></svg>`;

    var __decorate$6 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XioCheckbox = class XioCheckbox extends AutowatcherComponent {
        constructor() {
            super(...arguments);
            this.loading = ops.none();
            this["initially-checked"] = false;
            this["error-message"] = "error";
            this.disabled = false;
            this.handleClick = () => {
                if (!this.disabled) {
                    this.toggle();
                    const activeElement = document.activeElement;
                    if (activeElement)
                        activeElement.blur();
                }
            };
        }
        init() {
            this.loading = ops.ready(this["initially-checked"]);
        }
        get checked() {
            return ops.value(this.loading);
        }
        set checked(value) {
            this.loading = ops.ready(value);
        }
        async toggle(previousChecked = ops.value(this.loading), dispatchEvent = true) {
            const checked = !previousChecked;
            const isNotDisabled = !this.disabled;
            const isLoadingDone = ops.isReady(this.loading);
            return (isNotDisabled && isLoadingDone)
                ? ops.operation({
                    setOp: op => this.loading = op,
                    errorReason: this["error-message"],
                    promise: (async () => {
                        await (this.save ?? (async () => { }))(checked);
                        if (dispatchEvent) {
                            const event = new CheckEvent(this);
                            if (this.oncheck)
                                this.oncheck(event);
                            this.dispatchEvent(event);
                            event.stopPropagation();
                        }
                        return checked;
                    })(),
                })
                : checked;
        }
        render() {
            return y `
			<button
				data-mode="${ops.mode(this.loading)}"
				?data-checked=${ops.value(this.loading)}
				@click=${this.handleClick}>
					${ops.select(this.loading, {
            none: () => null,
            loading: () => spinnerIcon,
            error: reason => y `${warningSvg}<div class=error>${reason}</div>`,
            ready: checked => checked
                ? checkIcon
                : null,
        })}
			</button>
		`;
        }
    };
    __decorate$6([
        e()
    ], XioCheckbox.prototype, "loading", void 0);
    __decorate$6([
        e({ type: Boolean })
    ], XioCheckbox.prototype, "initially-checked", void 0);
    __decorate$6([
        e({ type: String })
    ], XioCheckbox.prototype, "error-message", void 0);
    __decorate$6([
        e({ type: Boolean, reflect: true })
    ], XioCheckbox.prototype, "disabled", void 0);
    __decorate$6([
        e({ type: Function })
    ], XioCheckbox.prototype, "oncheck", void 0);
    __decorate$6([
        e({ type: Function })
    ], XioCheckbox.prototype, "save", void 0);
    XioCheckbox = __decorate$6([
        mixinFocusable,
        mixinStyles(styles$6)
    ], XioCheckbox);

    var styles$5 = r$3 `

/* * { outline: 1px solid #f002; } */

:host {
	display: block;
	width: 100%;
	max-width: 48rem;
	--height: var(--xio-text-input-height, 5rem);
	--pad: var(--xio-text-input-pad, 0.2em);
	--font: var(--xio-text-input-font, inherit);
	--color: var(--xio-text-input-color, inherit);
	--label-font: var(--xio-text-input-label-font, inherit);
	--label-color: var(--xio-text-input-label-color, inherit);
	--problems-font: var(--xio-text-input-problems-font, inherit);
	--problems-color: var(--xio-text-input-problems-color);
	--background: var(--xio-text-input-background, transparent);
	--valid-color: var(--xio-text-input-valid-color, #00ff8c);
	--invalid-color: var(--xio-text-input-invalid-color, #ff6100);
	--border: var(--xio-text-input-border, 1px solid);
	--border-radius: var(--xio-text-input-border-radius, 0.3em);
}

:host([hidden]) {
	display: none;
}

label {
	font: var(--label-font);
	color: var(--label-color);
}

slot {
	display: block;
	padding: 0 var(--pad);
}

.flexy {
	display: flex;
	flex-direction: row;
	flex-wrap: wrap;
}

.flexy > * {
	flex: 0 0 auto;
}

:host([textarea]) .flexy {
	flex-direction: column;
}

[part="inputbox"] {
	display: block;
	/* max-width: var(--width); */
	position: relative;
	flex: 1 1 auto;
}

:host([textarea]) [part="inputbox"] {
	flex: 1 1 auto;
	/* max-width: unset; */
}

[part="inputbox"] svg {
	position: absolute;
	display: block;
	top: var(--pad);
	right: var(--pad);
	width: 1.2em;
	height: 1.2em;
	pointer-events: none;
}

.container[data-valid] [part="inputbox"] svg {
	color: var(--valid-color);
}

.container:not([data-valid]) [part="inputbox"] svg {
	color: var(--invalid-color);
}

#textinput {
	min-width: 1em;
	width: 100%;
	font: var(--font);
	padding: var(--pad);
	padding-right: calc(1em + calc(2 * var(--pad)));
	margin: 0;
	color: var(--color);
	background: var(--background);
	border: var(--border);
	border-radius: var(--border-radius);
	text-overflow: ellipsis;
}

textarea {
	min-height: var(--height);
}

[part="problems"] {
	display: flex;
	font: var(--problems-font);
	padding: 0 calc(2 * var(--pad));
	flex-direction: column;
	justify-content: flex-start;
	flex: 1 1 auto;
	/* min-width: 12rem; */
	list-style: none;
	color: var(--problems-color, var(--invalid-color));
}

[part="problems"] > li {
	margin-top: 0.2em;
}

`;

    function noopParser(text) {
        return text;
    }

    class EnterPressEvent extends CustomEvent {
        constructor() {
            super("enterpress", {
                bubbles: true,
                composed: true,
                detail: undefined,
            });
        }
    }

    class ValueChangeEvent extends CustomEvent {
        constructor(value) {
            super("valuechange", {
                bubbles: true,
                composed: true,
                detail: { value },
            });
        }
    }

    var svgCircleCheck = b `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-check-circle"><path d="M22 11.08V12a10 10 0 11-5.93-9.14"/><path d="M22 4L12 14.01l-3-3"/></svg>`;

    var __decorate$5 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XioTextInput = class XioTextInput extends AutowatcherComponent {
        constructor() {
            super(...arguments);
            this["initial"] = "";
            this["readonly"] = false;
            this["textarea"] = false;
            this["hide-validation"] = false;
            this["show-validation-when-empty"] = false;
            this["disabled"] = false;
            this["placeholder"] = "";
            this["hidden"] = false;
            this.parser = noopParser;
            this.problems = [];
            this.onvaluechange = () => { };
            this.onenterpress = () => { };
            this.draft = "";
            this.lastDraft = "";
            this.dispatchValueChange = () => {
                const { draft, lastDraft } = this;
                if (draft !== lastDraft)
                    this.dispatchEvent(new ValueChangeEvent(this.value));
                this.lastDraft = draft;
            };
            this.dispatchEnterPress = () => {
                this.dispatchEvent(new EnterPressEvent());
            };
            this.updateFromRawInput = () => {
                this.draft = this.input.value;
            };
            this.handleInputKeyUp = debounce(250, (event) => {
                this.updateFromRawInput();
                if (!this.textarea && event.key === "Enter") {
                    this.dispatchEnterPress();
                }
                else {
                    this.dispatchValueChange();
                }
            });
            this.handleInputChange = () => {
                this.updateFromRawInput();
                this.dispatchValueChange();
            };
        }
        get input() {
            return this.shadowRoot
                ? this.shadowRoot.querySelector("#textinput")
                : undefined;
        }
        focus() {
            if (this.input)
                this.input.focus();
        }
        get value() {
            const { draft } = this;
            const parsed = (this.parser ?? noopParser)(draft);
            this.problems = this.validator ? this.validator(parsed) : [];
            return this.problems.length === 0
                ? parsed
                : undefined;
        }
        get text() {
            return this.draft;
        }
        set text(value) {
            const { input } = this;
            if (input) {
                this.input.value = value;
                this.updateFromRawInput();
            }
            else {
                this.initial = value;
            }
        }
        init() {
            this.draft = this.initial;
            this.lastDraft = this.initial;
            this.addEventListener("valuechange", this.onvaluechange);
            this.addEventListener("enterpress", this.onenterpress);
        }
        render() {
            const { readonly, disabled, problems, draft, placeholder, textarea, validator, handleInputKeyUp, handleInputChange, } = this;
            const valid = problems.length === 0;
            const showValidation = !this["hide-validation"] && !readonly && validator && (this["show-validation-when-empty"]
                ? true
                : draft.length !== 0);
            const showProblems = showValidation && !valid;
            const icon = showValidation
                ? valid
                    ? svgCircleCheck
                    : warningSvg
                : null;
            return y `
			<div class=container ?data-valid=${valid}>
				<label for=textinput part=label><slot></slot></label>
				<div class=flexy>
					<div part=inputbox>
						${showValidation ? icon : null}
						${textarea ? y `
							<textarea
								id=textinput
								part=textinput
								.value="${draft}"
								?disabled=${disabled}
								?readonly=${readonly}
								tabindex=${readonly ? "-1" : "0"}
								placeholder=${placeholder}
								@keyup=${handleInputKeyUp}
								@change=${handleInputChange}
							></textarea>
						` : y `
							<input
								id=textinput
								type=text
								part=textinput
								.value="${draft}"
								?disabled=${disabled}
								?readonly=${readonly}
								tabindex=${readonly ? "-1" : "0"}
								placeholder=${placeholder}
								@keyup=${handleInputKeyUp}
								@change=${handleInputChange}
								/>
						`}
					</div>
					<ol part=problems>
						${showProblems
            ? problems.map(problem => y `
								<li>${problem}</li>
							`)
            : null}
					</ol>
				</div>
			</div>
		`;
        }
    };
    __decorate$5([
        e({ type: String, reflect: true })
    ], XioTextInput.prototype, "initial", void 0);
    __decorate$5([
        e({ type: Boolean, reflect: true })
    ], XioTextInput.prototype, "readonly", void 0);
    __decorate$5([
        e({ type: Boolean, reflect: true })
    ], XioTextInput.prototype, "textarea", void 0);
    __decorate$5([
        e({ type: Boolean, reflect: true })
    ], XioTextInput.prototype, "hide-validation", void 0);
    __decorate$5([
        e({ type: Boolean, reflect: true })
    ], XioTextInput.prototype, "show-validation-when-empty", void 0);
    __decorate$5([
        e({ type: Boolean, reflect: true })
    ], XioTextInput.prototype, "disabled", void 0);
    __decorate$5([
        e({ type: String, reflect: true })
    ], XioTextInput.prototype, "placeholder", void 0);
    __decorate$5([
        e({ type: Boolean, reflect: true })
    ], XioTextInput.prototype, "hidden", void 0);
    __decorate$5([
        e({ type: Function })
    ], XioTextInput.prototype, "parser", void 0);
    __decorate$5([
        e({ type: Function })
    ], XioTextInput.prototype, "validator", void 0);
    __decorate$5([
        e({ type: Object })
    ], XioTextInput.prototype, "problems", void 0);
    __decorate$5([
        e({ type: String })
    ], XioTextInput.prototype, "text", null);
    __decorate$5([
        e({ type: String })
    ], XioTextInput.prototype, "draft", void 0);
    XioTextInput = __decorate$5([
        mixinFocusable,
        mixinStyles(styles$5)
    ], XioTextInput);

    var styles$4 = r$3 `

:host {
	display: inline-block;
	--local-avatar-size: var(--avatar-size, 3em);
}

.container {
	display: flex;
	flex-direction: row;
	flex-wrap: wrap;
}

xio-avatar {
	flex: 0 0 auto;
	margin-right: 0.4em;
	--avatar-size: var(--local-avatar-size);
}

.box {
	display: flex;
	flex-direction: row;
	flex-wrap: wrap;
	flex: 1;
	flex-basis: 8em;
}

[part="nameplate"] {
	flex: 1 1 auto;
	margin-right: 1em;
	margin-bottom: 0.5em;
}

[part="details"] {
	margin-bottom: 0.5em;
}

.buttonbar {
	margin-top: 0.5em;
}

.cardplate > * {
	display: block;
}

.tags {
	list-style: none;
	font-size: 0.6em;
	cursor: default;
}

.tags > li {
	display: inline-block;
	margin: 0 0.1em;
	padding: 0 0.25em;
	border: 1px solid;
	border-radius: 1em;
}

[data-tag=staff] {
	color: var(--cobalt-tagcolor-staff, lime);
}

[data-tag=banned] {
	color: var(--cobalt-tagcolor-banned, red);
}

p[data-field=tagline] {
	opacity: 0.7;
	font-size: 0.7em;
	font-style: italic;
}

xio-text-input > span {
	opacity: 0.4;
	font-size: 0.7rem;
}

xio-text-input + xio-text-input {
	margin-top: 0.1em;
}

[part="details"] {
	font-size: 0.7em;
	list-style: none;
}

[part="details"] > li {
	margin-top: 0.2em;
}

[part="details"] > li > :first-child {
	font-weight: bold;
}

[part="details"] code {
	font-size: 0.6em;
	padding: 0.2em;
	border-radius: 0.3em;
	word-break: break-all;
	background: #0002;
}

.roles {
	font-size: 0.6em;
	list-style: none;
	padding: 0;
	margin-top: 0.3rem;
}

.roles li {
	display: inline-block;
	border: 1px solid;
	border-radius: 1em;
	padding: 0.1em 0.3em;
	line-height: 0.8em;
}

`;

    function renderText({ field, text, input }) {
        return input
            ? y `
			<xio-text-input
				data-field="${field}"
				text="${text}"
				part="xiotextinput"
				exportparts="${`
					label: xiotextinput-label,
					textinput: xiotextinput-textinput,
					problems: xiotextinput-problems,
				`}"
				show-validation-when-empty
				?readonly=${input.readonly}
				?hide-validation=${!input.draftIsChanged}
				.validator=${input.validator}
				@valuechange=${input.onvaluechange}>
					<span>${input.label}</span>
			</xio-text-input>
		`
            : y `
			<p part="textfield" data-field="${field}">${text}</p>
		`;
    }

    function renderRoles(user) {
        return y `
		<ul class=roles>
			${user.roles.map(role => y `
				<li
					data-role-label="${role.label}"
					data-role-id="${role.roleId}">
						${role.label}
				</li>
			`)}
		</ul>
	`;
    }

    function select(selector, context = document) {
        return context.querySelector(selector);
    }

    function renderDetails(user) {
        return y `
		<ul part=details>
			<li>
				<span>joined:</span>
				<span>${formatDate(user.stats.joined).date}</span>
			</li>
			<li>
				<span>user id:</span>
				<span><xio-id id="${user.userId}"></xio-id></span>
			</li>
		</ul>
	`;
    }

    function makeProfileDraft(profile) {
        return {
            tagline: profile.tagline,
            nickname: profile.nickname,
        };
    }

    var __decorate$4 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __classPrivateFieldGet$2 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var _XioProfileCard_state, _XioProfileCard_actions;
    let XioProfileCard = class XioProfileCard extends AutowatcherComponent {
        constructor() {
            super(...arguments);
            _XioProfileCard_state.set(this, this.auto.state({
                busy: ops.ready(undefined),
            }));
            _XioProfileCard_actions.set(this, this.auto.actions({
                setBusy: (op) => {
                    __classPrivateFieldGet$2(this, _XioProfileCard_state, "f").busy = op;
                },
            }));
            this.profileDraft = undefined;
            this.problems = [];
            this.handleChange = debounce(200, () => {
                if (!this.user)
                    return;
                const { profileDraft, isChanged } = this.generateNewProfileDraftFromInputs();
                this.profileDraft = isChanged ? profileDraft : undefined;
            });
            this.handleSave = async () => {
                await ops.operation({
                    promise: this.saveProfile(this.profileDraft)
                        .finally(() => {
                        this.profileDraft = null;
                    }),
                    setOp: op => __classPrivateFieldGet$2(this, _XioProfileCard_actions, "f").setBusy(op),
                });
            };
        }
        init() {
            __classPrivateFieldGet$2(this, _XioProfileCard_actions, "f").setBusy(ops.ready(undefined));
        }
        get draftIsChanged() {
            return !!this.profileDraft;
        }
        get readonly() {
            return !this.saveProfile;
        }
        getTextInputField(name) {
            return select(`xio-text-input[data-field="${name}"]`, this.shadowRoot);
        }
        generateNewProfileDraftFromInputs() {
            const { profile } = this.user;
            const profileDraft = makeProfileDraft(profile);
            const nicknameInput = this.getTextInputField("nickname");
            if (!nicknameInput)
                return { profileDraft, isChanged: false };
            const taglineInput = this.getTextInputField("tagline");
            profileDraft.nickname = nicknameInput.value;
            profileDraft.tagline = taglineInput.value;
            this.problems = [...nicknameInput.problems, ...taglineInput.problems];
            const isChanged = !deepEqual(makeProfileDraft(profile), profileDraft);
            return { profileDraft, isChanged };
        }
        render() {
            const { user, draftIsChanged } = this;
            if (!user)
                return null;
            const avatarSpec = user.profile.avatar;
            return renderOp(__classPrivateFieldGet$2(this, _XioProfileCard_state, "f").busy, () => y `
			<div class=container ?data-readonly=${this.readonly}>
				<xio-avatar part=avatar .spec=${avatarSpec}></xio-avatar>
				<div class=box>
					<div part=nameplate>
						${renderText({
            field: "nickname",
            text: this.profileDraft
                ? this.profileDraft.nickname
                : user.profile.nickname,
            input: this.readonly
                ? undefined
                : {
                    label: "nickname",
                    readonly: false,
                    draftIsChanged,
                    validator: profileValidators.nickname,
                    onvaluechange: this.handleChange,
                }
        })}
						${renderText({
            field: "tagline",
            text: this.profileDraft
                ? this.profileDraft.tagline
                : user.profile.tagline,
            input: this.readonly
                ? undefined
                : {
                    label: "tagline",
                    readonly: false,
                    draftIsChanged,
                    validator: profileValidators.tagline,
                    onvaluechange: this.handleChange,
                }
        })}
						${renderRoles(user)}
						${this.readonly ? null : y `
							<div class=buttonbar>
								<xio-button
									?disabled=${!this.profileDraft || this.problems.length > 0}
									@press=${this.handleSave}>
										<slot name=save-button>save profile</slot>
								</xio-button>
							</div>
						`}
					</div>
					${this["show-details"]
            ? renderDetails(user)
            : null}
				</div>
			</div>
		`);
        }
    };
    _XioProfileCard_state = new WeakMap(), _XioProfileCard_actions = new WeakMap();
    __decorate$4([
        e({ type: Boolean })
    ], XioProfileCard.prototype, "show-details", void 0);
    __decorate$4([
        e({ type: Object })
    ], XioProfileCard.prototype, "user", void 0);
    __decorate$4([
        e({ type: Object })
    ], XioProfileCard.prototype, "saveProfile", void 0);
    __decorate$4([
        e({ type: Object })
    ], XioProfileCard.prototype, "profileDraft", void 0);
    XioProfileCard = __decorate$4([
        mixinStyles(styles$4)
    ], XioProfileCard);

    function integrateXioComponents() {
        return {
            XioId,
            XioOp,
            XioAvatar,
            XioButton,
            XioExample,
            XioCheckbox,
            XioTextInput,
            XioProfileCard,
        };
    }

    var styles$3 = r$3 `

:host {
	display: block;
}

`;

    var __decorate$3 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XiomeExample = class XiomeExample extends ComponentWithShare {
        get state() {
            return this.share.exampleModel.state;
        }
        render() {
            return renderOp(this.state.accessOp, access => y `
			<p>Example Component</p>
			${access?.user
            ? y `<p>Welcome, ${access.user.profile.nickname}</p>`
            : y `<p>User is not logged in.</p>`}
		`);
        }
    };
    XiomeExample = __decorate$3([
        mixinStyles(styles$3)
    ], XiomeExample);

    function integrateExampleComponents({ models, modals }) {
        const { exampleModel } = models;
        return {
            XiomeExample: (mixinSnapstateSubscriptions(exampleModel.subscribe)(mixinShare({
                modals,
                exampleModel,
            })(XiomeExample))),
        };
    }

    var votingUnitCss = r$3 `

[data-vote] {
	display: block;
	border: none;
	font: inherit;
	background: transparent;
	color: inherit;
}

[data-vote] {
	cursor: pointer;
	opacity: 0.6;
	user-select: none;
}

[data-vote="report"] {
	opacity: 0.4;
}

[data-vote]:not([disabled]):hover,
[data-vote]:not([disabled]):focus {
	opacity: 1;
}

[data-vote][disabled] {
	cursor: default;
}

[data-vote="like"][data-active] {
	color: var(--like-color);
}

[data-vote="report"][data-active] {
	color: var(--report-color);
}

[data-vote] > span {
	vertical-align: middle;
}

[data-vote] > span:nth-child(2) {
	font-size: 0.9em;
}

[data-vote] svg {
	width: 1.4em;
	height: 1.4em;
	position: relative;
	top: 0.1em;
}

`;

    var postCss = r$3 `

${votingUnitCss}

.post {
	display: grid;
	grid-template-rows: auto;
	grid-template-columns: auto 1fr auto;
	grid-template-areas:
		".... tophat    ...."
		"bar1 bubble    bar2"
		".... buttonbar ....";
}

a {
	opacity: 0.5;
	color: inherit;
	text-decoration: none;
}

a:hover, a:focus {
	opacity: 0.8;
	text-decoration: underline;
}

/*
** normal questions
*/

.post .tophat { grid-area: tophat }
.post .bar1 { grid-area: bar1 }
.post .bar2 { grid-area: bar2 }
.post .bubble { grid-area: bubble }
.post .buttonbar { grid-area: buttonbar }

.post .bar {
	padding: 0.5em;
	padding-top: 0.2em;
	display: flex;
	flex-direction: column;
}

.post .bar > * + * {
	margin-top: 0.1em;
}

.post .bar.bar2 {
	padding-top: 0;
}

.post .metabar {
	font-size: 0.7em;
	padding: 0 1.5em;
	opacity: 0.6;
}

.post .tophat xio-profile-card {
	width: 100%;
}

.post .bubble {
	flex: 1 1 auto;
	/* padding-left: 1em; */
}

.post .bubble xio-text-input {
	--xio-text-input-border-radius: 0 1em 1em 1em;
	--xio-text-input-pad: 0.5em;
}

.post .bubble .textbox p {
	border-radius: 0 1em 1em 1em;
	border: 1px solid;
	padding: 0.5em;
	min-height: 4em;
}

.post [part="bubble"] {
	white-space: pre-wrap;
}

/*
** question editor
*/

.intro {
	margin-bottom: 1em;
}

.intro .heading {
	font-size: 2em;
}

.post.editor .buttonbar {
	text-align: right;
	padding: 0.5em;
}

.post.editor {
	padding-bottom: 2em;
	margin-bottom: 2em;
	border-bottom: 1px dashed;
}

`;

    var styles$2 = r$3 `

/* * {
	outline: 1px solid #f002;
} */

:host {
	display: block;
	max-width: 42em;
	--like-color: var(--xiome-questions-like-color, cyan);
	--report-color: var(--xiome-questions-report-color, #f90);
}

.questions-moderation-panel {
	padding: 1em;
	border: 1px dotted;
	margin-bottom: 1em;
}

.questions-moderation-panel .purge-button {
	--xio-button-hover-color: red;
}

slot[name="empty"] {
	display: block;
	margin-top: 1em;
}

[part="questions-list"] {
	list-style: none;

}

[part="questions-list"] > li {
	margin-top: 2em;
}

${postCss}

[part="answers-list"] {
	list-style: none;
	margin-left: 3.4em;
	margin-right: 1em;
	margin-bottom: 1em;
}

[part="answers-list"] > li {
	
	margin-top: 1em;
}

@media (max-width: 420px) {
	[part="answers-list"] {
		margin-left: 1em;
		margin-right: 0;
	}
	[part="answers-list"] > li {
		border-left: 1px solid;
	}
}

/*
** question editor
*/

.editor .intro {
	margin-bottom: 1em;
}

.editor .intro .heading {
	font-size: 2em;
}

.editor .buttonbar {
	text-align: right;
	padding: 0.5em;
}

.question-editor {
	padding-bottom: 2em;
	margin-bottom: 2em;
	border-bottom: 1px dashed;
}

/*
** answer editor
*/

.answer-editor {
	border: 1px dashed;
	margin-left: 5em;
	margin-top: 1em;
}

.answer-editor .intro {
	padding: 1em;
}

`;

    function sortQuestions(questions, myUserId) {
        const myQuestions = [];
        const otherQuestions = [];
        for (const question of questions) {
            const isMine = myUserId && question.authorUserId === myUserId;
            if (isMine)
                myQuestions.push(question);
            else
                otherQuestions.push(question);
        }
        const sort = (a, b) => {
            const promote = { a: -1, b: 1 };
            if (a.likes > b.likes)
                return promote.a;
            if (a.likes < b.likes)
                return promote.b;
            if (a.reports < b.reports)
                return promote.a;
            if (a.reports > b.reports)
                return promote.b;
            if (a.timePosted > b.timePosted)
                return promote.a;
            if (a.timePosted < b.timePosted)
                return promote.b;
            return 0;
        };
        return [
            ...myQuestions.sort(sort),
            ...otherQuestions.sort(sort),
        ];
    }

    function renderBubble({ content, editable, timePosted, handleValueChange = () => { }, }) {
        const { date, time } = formatDate(timePosted);
        return y `
		<div class=bubble ?data-editable=${editable}>
			<div class=textbox>
				${editable
        ? y `
						<xio-text-input
							exportparts="textinput: bubble"
							textarea
							.validator=${validatePostContent}
							@valuechange=${handleValueChange}
						></xio-text-input>
					`
        : y `
						<p part=bubble>${content}</p>
					`}
			</div>
			<div class=metabar>
				<p>${`${date} ${time}`}</p>
			</div>
		</div>
	`;
    }

    function linkClick(handler) {
        return (event) => {
            event.preventDefault();
            handler();
        };
    }

    var PostType;
    (function (PostType) {
        PostType[PostType["Question"] = 0] = "Question";
        PostType[PostType["Answer"] = 1] = "Answer";
        PostType[PostType["Editor"] = 2] = "Editor";
    })(PostType || (PostType = {}));

    var heartSvg = b `<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" aria-hidden="true" focusable="false" width="1em" height="1em" style="-ms-transform: rotate(360deg); -webkit-transform: rotate(360deg); transform: rotate(360deg);" preserveAspectRatio="xMidYMid meet" viewBox="0 0 24 24"><path fill-rule="evenodd" d="M6.736 4C4.657 4 2.5 5.88 2.5 8.514c0 3.107 2.324 5.96 4.861 8.12a29.66 29.66 0 0 0 4.566 3.175l.073.041l.073-.04c.271-.153.661-.38 1.13-.674c.94-.588 2.19-1.441 3.436-2.502c2.537-2.16 4.861-5.013 4.861-8.12C21.5 5.88 19.343 4 17.264 4c-2.106 0-3.801 1.389-4.553 3.643a.75.75 0 0 1-1.422 0C10.537 5.389 8.841 4 6.736 4zM12 20.703l.343.667a.75.75 0 0 1-.686 0l.343-.667zM1 8.513C1 5.053 3.829 2.5 6.736 2.5C9.03 2.5 10.881 3.726 12 5.605C13.12 3.726 14.97 2.5 17.264 2.5C20.17 2.5 23 5.052 23 8.514c0 3.818-2.801 7.06-5.389 9.262a31.146 31.146 0 0 1-5.233 3.576l-.025.013l-.007.003l-.002.001l-.344-.666l-.343.667l-.003-.002l-.007-.003l-.025-.013A29.308 29.308 0 0 1 10 20.408a31.147 31.147 0 0 1-3.611-2.632C3.8 15.573 1 12.332 1 8.514z" fill="currentColor"/></svg>`;

    var heartFillSvg = b `<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" aria-hidden="true" focusable="false" width="1em" height="1em" style="-ms-transform: rotate(360deg); -webkit-transform: rotate(360deg); transform: rotate(360deg);" preserveAspectRatio="xMidYMid meet" viewBox="0 0 24 24"><path d="M14 20.408c-.492.308-.903.546-1.192.709c-.153.086-.308.17-.463.252h-.002a.75.75 0 0 1-.686 0a16.709 16.709 0 0 1-.465-.252a31.147 31.147 0 0 1-4.803-3.34C3.8 15.572 1 12.331 1 8.513C1 5.052 3.829 2.5 6.736 2.5C9.03 2.5 10.881 3.726 12 5.605C13.12 3.726 14.97 2.5 17.264 2.5C20.17 2.5 23 5.052 23 8.514c0 3.818-2.801 7.06-5.389 9.262A31.146 31.146 0 0 1 14 20.408z" fill="currentColor"/></svg>`;

    function renderVotingUnit({ title, dataVote, voteCount, voteCasted, icon, castVote, }) {
        const handleClick = castVote
            ? voteCasted
                ? () => castVote(false)
                : () => castVote(true)
            : () => { };
        return y `
		<button
			tabindex=0
			data-vote="${dataVote}"
			?data-active=${voteCasted}
			?disabled=${!castVote}
			title="${title}"
			@click=${handleClick}>
				<span>${icon}</span>
				<span>${voteCount}</span>
		</button>
	`;
    }

    function renderLiking(liking) {
        return renderVotingUnit({
            dataVote: "like",
            icon: liking.liked
                ? heartFillSvg
                : heartSvg,
            title: liking.liked
                ? "unlike this post"
                : "like this post",
            voteCount: liking.likes,
            voteCasted: liking.liked,
            castVote: liking.castLikeVote
                ? status => liking.castLikeVote(status)
                : undefined,
        });
    }

    var warningFillSvg = b `<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" aria-hidden="true" focusable="false" width="1em" height="1em" style="-ms-transform: rotate(360deg); -webkit-transform: rotate(360deg); transform: rotate(360deg);" preserveAspectRatio="xMidYMid meet" viewBox="0 0 20 20"><g fill="none"><path d="M8.686 2.852L2.127 14.777A1.5 1.5 0 0 0 3.441 17H16.56a1.5 1.5 0 0 0 1.314-2.223L11.314 2.852a1.5 1.5 0 0 0-2.628 0zM10 6.75a.75.75 0 0 1 .75.75v4a.75.75 0 0 1-1.5 0v-4a.75.75 0 0 1 .75-.75zm.75 7a.75.75 0 1 1-1.5 0a.75.75 0 0 1 1.5 0z" fill="currentColor"/></g></svg>`;

    function renderReporting(reporting) {
        return renderVotingUnit({
            dataVote: "report",
            icon: reporting.reported
                ? warningFillSvg
                : warningSvg,
            title: reporting.reported
                ? "unreport this post"
                : "report this post",
            voteCount: reporting.reports,
            voteCasted: reporting.reported,
            castVote: reporting.castReportVote
                ? status => reporting.castReportVote(status)
                : undefined,
        });
    }

    function renderPostStructure({ postOptions, bar1, bubble, bar2, buttonBar }) {
        return y `
		<div class=post data-post-id="${postOptions.postId ?? ""}">
			<div class=tophat>
				<xio-profile-card .user=${postOptions.author} show-details></xio-profile-card>
			</div>
			<div class="bar bar1">
				${bar1}
			</div>
			${bubble}
			<div class="bar bar2">
				${bar2}
			</div>
			${buttonBar
        ? y `
					<div class=buttonbar>
						${buttonBar}
					</div>
				`
        : null}
		</div>
	`;
    }

    function renderPost(options) {
        const { author, content, timePosted, ...specificOptions } = options;
        switch (specificOptions.type) {
            case PostType.Question: {
                const { liking, reporting, deletePost, toggleAnswerEditor } = specificOptions;
                return renderPostStructure({
                    postOptions: options,
                    bar1: y `
					${renderLiking(liking)}
				`,
                    bubble: renderBubble({
                        content,
                        timePosted,
                        editable: false,
                        handleValueChange: undefined,
                    }),
                    bar2: y `
					${renderReporting(reporting)}
					${deletePost
                    ? y `<a href="#" @click=${linkClick(deletePost)}>delete</a>`
                    : null}
					${toggleAnswerEditor
                    ? y `<a href="#" @click=${linkClick(toggleAnswerEditor)}>answer</a>`
                    : null}
				`,
                    buttonBar: undefined,
                });
            }
            case PostType.Answer: {
                const { liking, reporting, deletePost } = specificOptions;
                return renderPostStructure({
                    postOptions: options,
                    bar1: y `
					${renderLiking(liking)}
				`,
                    bubble: renderBubble({
                        content,
                        timePosted,
                        editable: false,
                        handleValueChange: undefined,
                    }),
                    bar2: y `
					${renderReporting(reporting)}
					${deletePost
                    ? y `<a href="#" @click=${linkClick(deletePost)}>delete</a>`
                    : null}
				`,
                    buttonBar: undefined,
                });
            }
            case PostType.Editor: {
                const { isPostable, postButtonText, changeDraftContent, submitPost } = specificOptions;
                return renderPostStructure({
                    postOptions: options,
                    bar1: null,
                    bubble: renderBubble({
                        content,
                        timePosted,
                        editable: true,
                        handleValueChange: changeDraftContent,
                    }),
                    bar2: null,
                    buttonBar: y `
					<xio-button
						?disabled=${!isPostable}
						@click=${submitPost}
							>${postButtonText}</xio-button>
				`,
                });
            }
        }
    }

    function renderAnswer({ answer, questionId, modals, boardModel, canDeleteAnswer, }) {
        const permissions = boardModel.getPermissions();
        const canLike = permissions["like questions"];
        const canReport = permissions["report questions"];
        return renderPost({
            type: PostType.Answer,
            postId: answer.answerId,
            author: boardModel.getUser(answer.authorUserId),
            content: answer.content,
            timePosted: answer.timePosted,
            liking: {
                liked: answer.liked,
                likes: answer.likes,
                castLikeVote: canLike
                    ? like => boardModel.likeAnswer(questionId, answer.answerId, like)
                    : undefined,
            },
            reporting: {
                reported: answer.reported,
                reports: answer.reports,
                castReportVote: canReport
                    ? async (report) => {
                        const confirmed = report
                            ? await modals.confirm({
                                title: "Report answer?",
                                body: "Are you sure you want to submit a report against this answer?",
                                yes: { vibe: "negative", label: "Submit report" },
                                no: { vibe: "neutral", label: "Nevermind" },
                                focusNthElement: 2,
                            })
                            : true;
                        if (confirmed)
                            await boardModel.reportAnswer(questionId, answer.answerId, report);
                    }
                    : undefined,
            },
            deletePost: canDeleteAnswer(answer)
                ? async () => {
                    const confirmed = await modals.confirm({
                        title: "Delete answer?",
                        body: "Are you sure you want to delete this answer? This cannot be undone.",
                        yes: { vibe: "negative", label: "Delete answer" },
                        no: { vibe: "neutral", label: "Nevermind" },
                        focusNthElement: 2,
                    });
                    if (confirmed)
                        await boardModel.archiveAnswer(questionId, answer.answerId, true);
                }
                : undefined,
        });
    }

    function renderAnswerEditor({ now, boardModel, answerEditor, }) {
        const answerEditorState = answerEditor.getState();
        return y `
		<div class="editor answer-editor">
			<div class=intro>
				<p class=heading>Post your answer</p>
			</div>
			${renderPost({
        author: boardModel.getAccess().user,
        type: PostType.Editor,
        timePosted: now,
        postButtonText: "post answer",
        content: answerEditorState.draftText,
        isPostable: answerEditorState.isPostable,
        submitPost: answerEditor.submitAnswer,
        changeDraftContent: answerEditor.actions.handleValueChange,
    })}
		</div>
	`;
    }

    function renderQuestion({ now, question, modals, boardModel, answerEditor, }) {
        const { questionId } = question;
        const access = boardModel.getAccess();
        const permissions = boardModel.getPermissions();
        const author = boardModel.getUser(question.authorUserId);
        const questionAuthorities = (() => {
            const isAuthor = (access && access.user)
                ? access.user.userId === author.userId
                : false;
            const canDelete = permissions["moderate questions"] || isAuthor;
            const canAnswer = permissions["answer questions"];
            const canLike = permissions["like questions"];
            const canReport = permissions["report questions"];
            return { canDelete, canAnswer, canLike, canReport };
        })();
        const handleDelete = async () => {
            const confirmed = await modals.confirm({
                title: "Delete question?",
                body: "Are you sure you want to delete this question? This cannot be undone.",
                yes: { vibe: "negative", label: "Delete question" },
                no: { vibe: "neutral", label: "Nevermind" },
                focusNthElement: 2,
            });
            if (confirmed)
                await boardModel.archiveQuestion(questionId, true);
        };
        const handleLike = (like) => {
            boardModel.likeQuestion(questionId, like);
        };
        const handleReport = async (report) => {
            const confirmed = report
                ? await modals.confirm({
                    title: "Report question?",
                    body: "Are you sure you want to submit a report against this question?",
                    yes: { vibe: "negative", label: "Submit report" },
                    no: { vibe: "neutral", label: "Nevermind" },
                    focusNthElement: 2,
                })
                : true;
            if (confirmed)
                await boardModel.reportQuestion(questionId, report);
        };
        const answerEditorState = answerEditor.getState();
        const canDeleteAnswer = (answer) => {
            const isAuthor = access?.user?.userId === answer.authorUserId;
            return permissions["moderate questions"] || isAuthor;
        };
        return y `
		${renderPost({
        type: PostType.Question,
        author,
        content: question.content,
        deletePost: questionAuthorities.canDelete
            ? handleDelete
            : undefined,
        liking: {
            liked: question.liked,
            likes: question.likes,
            castLikeVote: questionAuthorities.canLike
                ? handleLike
                : undefined,
        },
        postId: question.questionId,
        reporting: {
            reported: question.reported,
            reports: question.reports,
            castReportVote: questionAuthorities.canReport
                ? handleReport
                : undefined,
        },
        timePosted: question.timePosted,
        toggleAnswerEditor: questionAuthorities.canAnswer
            ? answerEditor.actions.toggleEditMode
            : undefined,
    })}
		${answerEditorState.editMode
        ? renderAnswerEditor({ now, boardModel, answerEditor })
        : null}
		${question.answers.length ? y `
			<ol part=answers-list>
				${question.answers.map(answer => y `
					<li part=answer data-answer-id="${answer.answerId}">
						${renderAnswer({
        answer,
        modals,
        questionId,
        boardModel,
        canDeleteAnswer,
    })}
					</li>
				`)}
			</ol>
		` : null}
	`;
    }

    function makeEditorState() {
        return happystate({
            state: {
                draftText: "",
                isPostable: false,
            },
            actions: state => ({
                handleValueChange(event) {
                    state.draftText = event.detail.value;
                    state.isPostable = !!state.draftText;
                },
            }),
        });
    }

    function makeQuestionEditor({ requestUpdate, getTextInput, getBoardModel, }) {
        const { actions, getState, onStateChange } = makeEditorState();
        onStateChange(requestUpdate);
        const resetEditor = () => {
            const input = getTextInput();
            input.text = "";
        };
        return {
            actions,
            getState,
            getTextInput,
            submitQuestion: async () => {
                const { draftText } = getState();
                resetEditor();
                await getBoardModel().postQuestion({
                    content: draftText,
                });
            },
        };
    }

    /*
     * happyCombine is tricky to use because typescript types are tricky.
     * example:
     *     const happy = happyCombine(happy1)(happy2)(happy3).combine()
     * do take note:
     *  - you must end the curried chain with `.combine()`
     *
     *
     */
    function happyCombine(happy1) {
        return function (happy2) {
            const getHappy = () => ({
                actions: { ...happy1.actions, ...happy2.actions },
                getState: () => ({ ...happy1.getState(), ...happy2.getState() }),
                clearStateListeners: () => {
                    happy1.clearStateListeners();
                    happy2.clearStateListeners();
                },
                onStateChange: (listener) => {
                    const subscriptions = [
                        happy1.onStateChange(listener),
                        happy2.onStateChange(listener),
                    ];
                    return () => subscriptions.forEach(unsubscribe => unsubscribe());
                },
            });
            function more(happy3) {
                return happyCombine(getHappy())(happy3);
            }
            more.combine = getHappy;
            return more;
        };
    }

    function makeAnswerEditorGetter({ requestUpdate, getBoardModel, getTextInput, }) {
        const getRecord = strongRecordKeeper()(questionId => {
            const editorHappy = makeEditorState();
            const answerHappy = happystate({
                state: {
                    editMode: false,
                },
                actions: state => ({
                    toggleEditMode() {
                        state.editMode = !state.editMode;
                    },
                }),
            });
            const happy = happyCombine(editorHappy)(answerHappy).combine();
            happy.onStateChange(requestUpdate);
            const resetEditor = () => {
                const input = getTextInput(questionId);
                input.text = "";
            };
            return {
                ...happy,
                submitAnswer: async () => {
                    const { draftText } = happy.getState();
                    resetEditor();
                    happy.actions.toggleEditMode();
                    await getBoardModel().postAnswer(questionId, { content: draftText });
                },
            };
        });
        return (questionId) => getRecord(questionId);
    }

    function renderModerationPanel({ modals, board, archiveBoard }) {
        const handlePressPurgeButton = async () => {
            const confirmed = await modals.confirm({
                title: `Purge questions?`,
                body: `Are you sure you want to delete all the questions on the board "${board}"? This cannot be undone.`,
                yes: { vibe: "negative", label: "Purge all" },
                no: { vibe: "neutral", label: "Nevermind" },
                focusNthElement: 2,
            });
            if (confirmed)
                await archiveBoard();
        };
        return y `
		<div class=questions-moderation-panel>
			<h3>moderate questions board "${board}"</h3>
			<xio-button
				class=purge-button
				@press=${handlePressPurgeButton}>
					Purge all questions
			</xio-button>
		</div>
	`;
    }

    function renderQuestionEditor({ now, access, postingOp, questionEditor, }) {
        const author = access?.user;
        const editorState = questionEditor.getState();
        return renderOp(postingOp, () => y `
		<div class="editor question-editor" part=question-editor>
			<div class=intro>
				<p class=heading>Post a new question</p>
			</div>
			${renderPost({
        type: PostType.Editor,
        author,
        content: editorState.draftText,
        isPostable: editorState.isPostable,
        timePosted: now,
        postButtonText: "post question",
        submitPost: questionEditor.submitQuestion,
        changeDraftContent: questionEditor.actions.handleValueChange,
    })}
		</div>
	`);
    }

    var __decorate$2 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __classPrivateFieldGet$1 = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var __classPrivateFieldSet$1 = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var _XiomeQuestions_boardModel, _XiomeQuestions_questionEditor, _XiomeQuestions_getAnswerEditor, _XiomeQuestions_now;
    let BaseComponent = class BaseComponent extends ComponentWithShare {
    };
    BaseComponent = __decorate$2([
        mixinTicker(1000)
    ], BaseComponent);
    let XiomeQuestions = class XiomeQuestions extends BaseComponent {
        constructor() {
            super(...arguments);
            _XiomeQuestions_boardModel.set(this, void 0);
            _XiomeQuestions_questionEditor.set(this, makeQuestionEditor({
                getBoardModel: () => __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f"),
                requestUpdate: () => this.requestUpdate(),
                getTextInput: () => (this.shadowRoot.querySelector(".question-editor xio-text-input")),
            }));
            _XiomeQuestions_getAnswerEditor.set(this, makeAnswerEditorGetter({
                getBoardModel: () => __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f"),
                requestUpdate: () => this.requestUpdate(),
                getTextInput: (questionId) => (this.shadowRoot.querySelector(`[part="questions-list"] li[data-question-id="${questionId}"] xio-text-input`)),
            }));
            _XiomeQuestions_now.set(this, Date.now());
            this.board = "default";
        }
        tick() {
            __classPrivateFieldSet$1(this, _XiomeQuestions_now, Date.now(), "f");
        }
        init() {
            __classPrivateFieldSet$1(this, _XiomeQuestions_boardModel, this.share.questionsModel.makeBoardModel(this.board), "f");
            __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").loadQuestions();
        }
        renderQuestionsBoard() {
            const now = __classPrivateFieldGet$1(this, _XiomeQuestions_now, "f");
            const { modals } = this.share;
            const questionEditor = __classPrivateFieldGet$1(this, _XiomeQuestions_questionEditor, "f");
            const { archiveBoard } = __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f");
            const access = __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").getAccess();
            const board = __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").getBoardName();
            const boardOp = __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").getBoardOp();
            const postingOp = __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").getPostingOp();
            const permissions = __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").getPermissions();
            const questions = sortQuestions(__classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").getQuestions(), access?.user?.userId);
            return renderOp(boardOp, () => y `

			${permissions["moderate questions"]
            ? renderModerationPanel({ modals, board, archiveBoard })
            : null}

			${permissions["post questions"]
            ? renderQuestionEditor({ now, access, postingOp, questionEditor })
            : null}

			${questions.length > 0
            ? y `
					<ol part=questions-list>
						${questions.map(question => y `
							<li data-question-id="${question.questionId}">
								${renderQuestion({
                question,
                now: __classPrivateFieldGet$1(this, _XiomeQuestions_now, "f"),
                modals: this.share.modals,
                boardModel: __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f"),
                answerEditor: __classPrivateFieldGet$1(this, _XiomeQuestions_getAnswerEditor, "f").call(this, question.questionId),
            })}
							</li>
						`)}
					</ol>
				`
            : y `<slot name=empty><p>Be the first to post a question!</p></slot>`}
		`);
        }
        render() {
            return __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f") && __classPrivateFieldGet$1(this, _XiomeQuestions_boardModel, "f").getBoardOp()
                ? this.renderQuestionsBoard()
                : null;
        }
    };
    _XiomeQuestions_boardModel = new WeakMap(), _XiomeQuestions_questionEditor = new WeakMap(), _XiomeQuestions_getAnswerEditor = new WeakMap(), _XiomeQuestions_now = new WeakMap();
    __decorate$2([
        e({ type: String, reflect: true })
    ], XiomeQuestions.prototype, "board", void 0);
    XiomeQuestions = __decorate$2([
        mixinStyles(styles$2)
    ], XiomeQuestions);

    function integrateQuestionsComponents({ models, modals }) {
        const { questionsModel } = models;
        return {
            XiomeQuestions: mixinHappy(questionsModel.onStateChange)(mixinShare({
                modals,
                questionsModel,
            })(XiomeQuestions)),
        };
    }

    var styles$1 = r$3 `

:host {
	display: block;
}

`;

    function generateVimeoUrl(vimeoId) {
        return `https://vimeo.com/${vimeoId}`;
    }

    function parseVimeoId(field) {
        const vimeoId = isUrl(field)
            ? new URL(field).pathname.slice(1)
            : field;
        const problems = validateVimeoId(vimeoId);
        if (!problems.length)
            return vimeoId;
        else
            throw new Error("invalid vimeo id");
    }

    var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __classPrivateFieldGet = (undefined && undefined.__classPrivateFieldGet) || function (receiver, state, kind, f) {
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    };
    var __classPrivateFieldSet = (undefined && undefined.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
        if (kind === "m") throw new TypeError("Private method is not writable");
        if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    };
    var _XiomeLivestream_instances, _XiomeLivestream_renderStreamViewer, _XiomeLivestream_getCurrentVimeoId, _XiomeLivestream_changed, _XiomeLivestream_vimeoIdDraft, _XiomeLivestream_vimeoFieldOnValueChange, _XiomeLivestream_vimeoIdDraftIsDifferent, _XiomeLivestream_submitVimeoId, _XiomeLivestream_renderModBox, _XiomeLivestream_renderForRights;
    let XiomeLivestream = class XiomeLivestream extends ComponentWithShare {
        constructor() {
            super(...arguments);
            _XiomeLivestream_instances.add(this);
            this.label = "default";
            _XiomeLivestream_renderStreamViewer.set(this, () => {
                const showOp = this.share.livestreamModel.getShow(this.label);
                return renderOp(showOp, show => y `
			<p>
				${show.vimeoId
                ? `label "${show.label}", vimeo id ${show.vimeoId}`
                : `no vimeo id for show "${show.label}"`}
			</p>
		`);
            });
            _XiomeLivestream_changed.set(this, false);
            _XiomeLivestream_vimeoIdDraft.set(this, void 0);
            _XiomeLivestream_renderModBox.set(this, () => {
                const showOp = this.share.livestreamModel.getShow(this.label);
                const currentVimeoId = __classPrivateFieldGet(this, _XiomeLivestream_instances, "m", _XiomeLivestream_getCurrentVimeoId).call(this);
                const currentVimeoUrl = currentVimeoId.length
                    ? generateVimeoUrl(currentVimeoId)
                    : "";
                const draftIsDifferent = __classPrivateFieldGet(this, _XiomeLivestream_instances, "m", _XiomeLivestream_vimeoIdDraftIsDifferent).call(this);
                return renderOp(showOp, () => y `
			<div class=modbox>
				<xio-text-input
					class=vimeo-field
					.initial="${currentVimeoUrl}"
					show-validation-when-empty
					?hide-validation=${!draftIsDifferent}
					.validator=${validateVimeoField}
					?disabled=${!ops.isReady(showOp)}
					@valuechange=${__classPrivateFieldGet(this, _XiomeLivestream_instances, "m", _XiomeLivestream_vimeoFieldOnValueChange)}>
						vimeo url
				</xio-text-input>
				<xio-button
					?disabled=${!draftIsDifferent}
					@press=${__classPrivateFieldGet(this, _XiomeLivestream_instances, "m", _XiomeLivestream_submitVimeoId)}>
						set livestream
				</xio-button>
			</div>
		`);
            });
            _XiomeLivestream_renderForRights.set(this, () => {
                switch (this.share.livestreamModel.state.rights) {
                    case LivestreamRights.Forbidden: return y `
				<p>you are forbidden lol</p>
			`;
                    case LivestreamRights.Viewer: return y `
				<p>you are viewer</p>
				${__classPrivateFieldGet(this, _XiomeLivestream_renderStreamViewer, "f").call(this)}
			`;
                    case LivestreamRights.Moderator: return y `
				<p>you are moderator</p>
				${__classPrivateFieldGet(this, _XiomeLivestream_renderModBox, "f").call(this)}
				${__classPrivateFieldGet(this, _XiomeLivestream_renderStreamViewer, "f").call(this)}
			`;
                }
            });
        }
        get state() {
            return this.share.livestreamModel.state;
        }
        updated(props) {
            if (props.has("label") && props.get("label") !== this.label)
                this.share.livestreamModel.loadShow(this.label);
        }
        render() {
            return renderOp(this.state.accessOp, __classPrivateFieldGet(this, _XiomeLivestream_renderForRights, "f"));
        }
    };
    _XiomeLivestream_renderStreamViewer = new WeakMap(), _XiomeLivestream_changed = new WeakMap(), _XiomeLivestream_vimeoIdDraft = new WeakMap(), _XiomeLivestream_renderModBox = new WeakMap(), _XiomeLivestream_renderForRights = new WeakMap(), _XiomeLivestream_instances = new WeakSet(), _XiomeLivestream_getCurrentVimeoId = function _XiomeLivestream_getCurrentVimeoId() {
        const showOp = this.share.livestreamModel.getShow(this.label);
        const show = ops.value(showOp);
        return (show && show.vimeoId)
            ? show.vimeoId
            : "";
    }, _XiomeLivestream_vimeoFieldOnValueChange = function _XiomeLivestream_vimeoFieldOnValueChange({ detail: { value } }) {
        __classPrivateFieldSet(this, _XiomeLivestream_changed, true, "f");
        __classPrivateFieldSet(this, _XiomeLivestream_vimeoIdDraft, value
            ? parseVimeoId(value)
            : "", "f");
        this.requestUpdate();
    }, _XiomeLivestream_vimeoIdDraftIsDifferent = function _XiomeLivestream_vimeoIdDraftIsDifferent() {
        const showOp = this.share.livestreamModel.getShow(this.label);
        return __classPrivateFieldGet(this, _XiomeLivestream_changed, "f") && ops.isReady(showOp)
            ? __classPrivateFieldGet(this, _XiomeLivestream_vimeoIdDraft, "f") !== __classPrivateFieldGet(this, _XiomeLivestream_instances, "m", _XiomeLivestream_getCurrentVimeoId).call(this)
            : false;
    }, _XiomeLivestream_submitVimeoId = function _XiomeLivestream_submitVimeoId() {
        __classPrivateFieldSet(this, _XiomeLivestream_changed, false, "f");
        const vimeoId = __classPrivateFieldGet(this, _XiomeLivestream_vimeoIdDraft, "f");
        this.share.livestreamModel.commitShow({
            vimeoId,
            label: this.label,
        });
    };
    __decorate$1([
        e({ type: String, reflect: true })
    ], XiomeLivestream.prototype, "label", void 0);
    XiomeLivestream = __decorate$1([
        mixinStyles(styles$1)
    ], XiomeLivestream);

    function integrateLivestreamComponents({ models, modals }) {
        const { livestreamModel } = models;
        return {
            XiomeLivestream: (mixinSnapstateSubscriptions(livestreamModel.subscribe)(mixinShare({ modals, livestreamModel })(XiomeLivestream))),
        };
    }

    var styles = r$3 `

.container {
	display: block;
	width: 100%;
	max-width: 36em;
	padding: 0.5em !important;
	border: 1px solid;
}

.results {
	margin-top: 0.5em;
}

.userlist {
	list-style: none;
}

.userlist > li {
	display: flex;
	flex-direction: column;
	flex-wrap: wrap;
	background: #0002;
	padding: 0.5em;
	margin-top: 0.5em;
}

.userinfo {
	display: flex;
	flex-direction: row;
}

.userinfo xio-profile-card {
	flex: 1 1 auto;
}

.userinfo .controls {
	margin-left: 1em;
}

.userinfo .controls .edit[data-edit-mode] {
	color: lime;
	--xio-button-hover-color: lime;
}

.editwidget > div {
	margin-top: 1em;
}

.editwidget header + * {
	margin-top: 0.25em;
}

.editwidget header small {
	font-size: inherit;
	opacity: 0.6;
}

.editwidget ul {
	list-style: none;
}

.editwidget .allprivileges li {
	display: inline-block;
	font-size: 0.75em;
	padding: 0.1em 0.3em;
	margin: 0.2em 0.1em;
	border: 1px solid;
	border-radius: 1em;
}

`;

    function makeUserStates({ getUserResultsOp, rerender, }) {
        const states = new Map();
        function cleanupObsoleteStates() {
            const userIdsPendingRemovalFromState = [];
            const userResultsOp = getUserResultsOp();
            if (ops.ready(userResultsOp)) {
                const userResults = ops.value(userResultsOp);
                for (const stateUserId of states.keys()) {
                    const userIsGone = !userResults.find(({ user }) => user.userId === stateUserId);
                    if (userIsGone)
                        userIdsPendingRemovalFromState.push(stateUserId);
                }
            }
            for (const obsoleteUserId of userIdsPendingRemovalFromState)
                states.delete(obsoleteUserId);
        }
        function obtainStateForUser(userId) {
            let state = states.get(userId);
            if (!state) {
                const newState = {
                    editMode: false,
                    toggleEditMode() {
                        newState.editMode = !newState.editMode;
                        rerender();
                    },
                };
                state = newState;
                states.set(userId, state);
            }
            return state;
        }
        return {
            cleanupObsoleteStates,
            obtainStateForUser,
        };
    }

    function sortAssignableFirst(roles) {
        const assignable = roles.filter(role => role.assignable);
        const notAssignable = roles.filter(role => !role.assignable);
        return [...assignable, ...notAssignable];
    }
    function renderRoleButton(role, onClick) {
        return y `
		<xio-button
			?disabled=${!role.assignable}
			title=${role.roleId}
			data-role-id=${role.roleId}
			@press=${() => onClick(role)}>
				${role.label}
		</xio-button>
	`;
    }
    function renderPrivilegeUserHas(privilege) {
        return y `
		<li
			title="${privilege.privilegeId}"
			data-privilege-id="${privilege.privilegeId}">
				${privilege.label}
		</li>
	`;
    }
    function renderEditWidget({ permissions, administrativeModel, updateLocalUserResultsCache, userResult: { user: { userId }, roleIds }, blur, search, }) {
        const rolesAssigned = sortAssignableFirst(roleIds
            .map(id => permissions.roles.find(role => role.roleId === id))
            .filter(id => !!id));
        const rolesAvailable = sortAssignableFirst(permissions.roles
            .filter(role => !roleIds.includes(role.roleId))
            .filter(id => !!id));
        const privilegesUserHas = permissions.privileges
            .filter(privilege => {
            const rolesWithThisPrivilege = permissions.rolesHavePrivileges
                .filter(r => r.active && r.privilegeId === privilege.privilegeId);
            const rolesUserHas = rolesWithThisPrivilege
                .filter(r => roleIds.includes(r.roleId));
            return rolesUserHas.length > 0;
        });
        async function ifChangingSelfThenReauthorize() {
            if (userId === administrativeModel.getAccess()?.user?.userId)
                await administrativeModel.reauthorize();
        }
        async function clickToAssign({ roleId }) {
            await administrativeModel.assignRoleToUser({
                userId,
                roleId,
                isPublic: true,
                timeframeEnd: undefined,
                timeframeStart: undefined,
            });
            updateLocalUserResultsCache.assignRole(userId, roleId);
            blur();
            await ifChangingSelfThenReauthorize();
            await search();
        }
        async function clickToRevoke({ roleId }) {
            await administrativeModel.revokeRoleFromUser({
                userId,
                roleId,
            });
            updateLocalUserResultsCache.revokeRole(userId, roleId);
            blur();
            await ifChangingSelfThenReauthorize();
            await search();
        }
        return y `
		<div class=editwidget>
			<div class=available>
				<header>roles available <small>(click to assign)</small></header>
				<div>
					${rolesAvailable.map(role => renderRoleButton(role, clickToAssign))}
				</div>
			</div>
			<div class=assigned>
				<header>roles assigned <small>(click to revoke)</small></header>
				<div>
					${rolesAssigned.map(role => renderRoleButton(role, clickToRevoke))}
				</div>
			</div>
			<div class=allprivileges>
				<header>user has these privileges</header>
				<ul>
					${privilegesUserHas.map(renderPrivilegeUserHas)}
				</ul>
			</div>
		</div>
	`;
    }

    var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    let XiomeManageUsers = class XiomeManageUsers extends ComponentWithShare {
        constructor() {
            super(...arguments);
            this.userResults = ops.ready([]);
            this.updateLocalUserResultsCache = {
                assignRole: (userId, roleId) => {
                    if (!ops.ready(this.userResults))
                        throw new Error("error updating user cache");
                    const userResults = ops.value(this.userResults);
                    this.userResults = ops.ready(userResults.map(result => result.user.userId === userId
                        ? { user: result.user, roleIds: [...result.roleIds, roleId] }
                        : result));
                },
                revokeRole: (userId, roleId) => {
                    if (!ops.ready(this.userResults))
                        throw new Error("error updating user cache");
                    const userResults = ops.value(this.userResults);
                    this.userResults = ops.ready(userResults.map(result => result.user.userId === userId
                        ? { user: result.user, roleIds: result.roleIds.filter(id => id !== roleId) }
                        : result));
                },
            };
            this.userStates = makeUserStates({
                getUserResultsOp: () => this.userResults,
                rerender: () => this.requestUpdate(),
            });
            this.searchText = "";
            this.debouncedSearchUsers = debounce(1000, async (term) => term
                ? await this.share.administrativeModel.searchUsers({ term })
                : []);
            this.searchChange = async (event) => {
                this.searchText = event.detail.value ?? "";
                return this.search();
            };
        }
        init() {
            this.share.administrativeModel.initialize();
        }
        async search() {
            const { searchText } = this;
            this.userResults = ops.ready([]);
            if (searchText)
                await ops.operation({
                    promise: this.debouncedSearchUsers(searchText),
                    setOp: op => this.userResults = op,
                });
            else
                this.userResults = ops.ready([]);
            this.userStates.cleanupObsoleteStates();
        }
        render() {
            const { permissionsOp } = this.share.administrativeModel.state;
            const allowed = this.share.administrativeModel
                .isAllowed("administrate user roles");
            const renderUser = (userResult) => {
                const { user } = userResult;
                const state = this.userStates.obtainStateForUser(user.userId);
                return y `
				<li>
					<div class=userinfo>
						<xio-profile-card
							.user=${user}
							show-details
						></xio-profile-card>
						<div class=controls>
							<xio-button
								class=edit
								?data-edit-mode=${state.editMode}
								@press=${state.toggleEditMode}>
									${wrenchSvg}
							</xio-button>
						</div>
					</div>
					${state.editMode
                ? renderOp(permissionsOp, permissions => renderEditWidget({
                    userResult,
                    permissions,
                    administrativeModel: this.share.administrativeModel,
                    updateLocalUserResultsCache: this.updateLocalUserResultsCache,
                    blur: () => {
                        const activeElement = document.activeElement;
                        if (activeElement)
                            activeElement.blur();
                    },
                    search: () => this.search(),
                }))
                : null}
				</li>
			`;
            };
            return y `
			<div class=container>

				${allowed ? y `
					<xio-text-input
						.text=${this.searchText}
						placeholder="search for users"
						.validator=${validateUserSearchTerm}
						@valuechange=${this.searchChange}
					></xio-text-input>

					<div class=results>
						${renderOp(this.userResults, results => results.length > 0
            ? y `
								<ol class=userlist>
									${results.map(renderUser)}
								</ol>
							`
            : y `
								<div class=noresults>
									no results
								</div>
							`)}
					</div>
				` : y `
					<p>you are not permitted to administrate user roles</p>
				`}

			</div>
		`;
        }
    };
    __decorate([
        e()
    ], XiomeManageUsers.prototype, "userResults", void 0);
    __decorate([
        e({ type: String })
    ], XiomeManageUsers.prototype, "searchText", void 0);
    XiomeManageUsers = __decorate([
        mixinStyles(styles)
    ], XiomeManageUsers);

    function integrateAdministrativeComponents({ models, modals }) {
        const { administrativeModel } = models;
        return {
            XiomeManageUsers: (mixinSnapstateSubscriptions(administrativeModel.subscribe)(mixinShare({
                modals,
                administrativeModel,
            })(XiomeManageUsers))),
        };
    }

    function getComponents(options) {
        return themeComponents(theme, {
            ...integrateXioComponents(),
            ...integrateExampleComponents(options),
            ...integrateAuthComponents(options),
            ...integrateLivestreamComponents(options),
            ...integrateQuestionsComponents(options),
            ...integrateAdministrativeComponents(options),
            ...integrateVideoComponents(options),
            // // TODO reactivate store
            // ...xiomeStoreComponents(options),
        });
    }

    function trapFocus(element, focusNth) {
        const blueprint = `
		[focusable]:not([disabled])
		[tabindex]:not([disabled])
		a[href]:not([disabled])
		button:not([disabled])
		textarea:not([disabled])
		input[type="text"]:not([disabled])
		input[type="radio"]:not([disabled])
		input[type="checkbox"]:not([disabled])
		select:not([disabled])
	`;
        const selector = blueprint
            .split("\n")
            .map(l => l.trim())
            .filter(l => l.length)
            .join(", ");
        const focusable = Array.from(element.querySelectorAll(selector));
        if (focusable.length === 0)
            throw new Error("cannot trap focus: no focusable element");
        const firstFocusable = focusable[0];
        const lastFocusable = focusable[focusable.length - 1];
        const nthFocusable = focusable[focusNth - 1];
        setTimeout(() => nthFocusable.focus(), 0);
        element.onkeydown = event => {
            if (event.key === "Tab") {
                if (event.shiftKey) {
                    if (document.activeElement === firstFocusable) {
                        lastFocusable.focus();
                        event.preventDefault();
                    }
                }
                else {
                    if (document.activeElement === lastFocusable) {
                        firstFocusable.focus();
                        event.preventDefault();
                    }
                }
            }
        };
    }

    function preparePrompt(popup) {
        return async ({ title, input, body = null, yes = { label: "okay", vibe: "positive" }, no = { label: "nevermind", vibe: "neutral" }, focusNthElement = 1, blanketClickMeansCancel = true, }) => new Promise(resolve => {
            const xioTextInput = document.createElement("xio-text-input");
            xioTextInput.textarea = input.textarea ?? false;
            xioTextInput.parser = input.parser;
            xioTextInput.validator = input.validator;
            xioTextInput.textContent = input.label;
            function hasProblems() {
                void xioTextInput.value;
                return xioTextInput.problems.length !== 0;
            }
            function getCurrentValue() {
                return hasProblems()
                    ? undefined
                    : { value: xioTextInput.value };
            }
            function prepareFinishingMoves(controls) {
                return {
                    yes: () => {
                        const currentValue = getCurrentValue();
                        if (!hasProblems()) {
                            controls.close();
                            resolve(currentValue);
                        }
                    },
                    no: () => {
                        controls.close();
                        resolve(undefined);
                    },
                };
            }
            const { controls, modal } = popup({
                focusNthElement,
                onBlanketClick: blanketClickMeansCancel
                    ? controls => {
                        controls.close();
                        resolve(undefined);
                    }
                    : () => { },
                renderContent: controls => {
                    const finish = prepareFinishingMoves(controls);
                    return y `
					<div data-confirm>
						${typeof title == "string" ? y `<h2>${title}</h2>` : title}
						${typeof body == "string" ? y `<p>${body}</p>` : body}
						${xioTextInput}
						<div data-buttons>
							<xio-button
								data-button=yes
								data-vibe=${yes.vibe}
								?disabled=${hasProblems()}
								@press=${finish.yes}>
									${yes.label}
							</xio-button>
							<xio-button
								data-button=no
								data-vibe=${no.vibe}
								@press=${finish.no}>
									${no.label}
							</xio-button>
						</div>
					</div>
				`;
                },
            });
            const finish = prepareFinishingMoves(controls);
            xioTextInput.onvaluechange = () => controls.rerender();
            xioTextInput.onenterpress = () => finish.yes();
            modal.addEventListener("keyup", ({ key }) => {
                if (key === "Escape")
                    finish.no();
            });
        });
    }

    function prepareConfirm(popup) {
        return async ({ title, body = null, yes = { label: "yes", vibe: "positive" }, no = { label: "no", vibe: "neutral" }, focusNthElement = 1, blanketClickMeansNo = true, }) => new Promise(resolve => {
            popup({
                focusNthElement,
                onBlanketClick: blanketClickMeansNo
                    ? controls => {
                        controls.close();
                        resolve(false);
                    }
                    : () => { },
                renderContent: controls => {
                    const onYes = () => {
                        controls.close();
                        resolve(true);
                    };
                    const onNo = () => {
                        controls.close();
                        resolve(false);
                    };
                    return y `
					<div data-confirm>
						${typeof title == "string" ? y `<h2>${title}</h2>` : title}
						${typeof body == "string" ? y `<p>${body}</p>` : body}
						<div data-buttons>
							<xio-button
								focusable
								data-button=yes
								data-vibe="${yes.vibe}"
								@press=${onYes}>
									${yes.label}
							</xio-button>
							${no ? y `
								<xio-button
									focusable
									data-button=no
									data-vibe="${no.vibe}"
									@press=${onNo}>
										${no.label}
								</xio-button>
							` : null}
						</div>
					</div>
				`;
                },
            });
        });
    }

    var modalSystemStyles = r$3 `

*:focus {
	outline: var(--focus-outline);
}

[data-modal-system] {
	position: absolute;
	top: 0;
	left: 0;
	right: 0;
	display: block;
}

[data-modal-system] [data-blanket] {
	z-index: 101;
	position: fixed;
	top: 0;
	left: 0;
	right: 0;
	bottom: 0;
	display: block;
	background: #0008;
	backdrop-filter: blur(10px);
}

[data-modal-system] [data-plate] {
	z-index: 102;
	position: absolute;
	top: 0;
	left: 0;
	right: 0;

	width: 96%;
	max-width: 32em;
	min-height: 1em;
	margin: 2em auto;

	background: linear-gradient(to bottom right, #fff, #ccc);
	color: #444;
}

[data-modal-system] [data-content]:focus {
	outline: unset;
}

[data-modal-system] [data-confirm] {
	padding: 1em;
}

[data-modal-system] [data-buttons] {
	margin-top: 1em;
	text-align: right;
}

[data-modal-system] [data-button] {
	--xio-button-hover-color: white;
	--xio-button-hover-background: #666;
}

[data-modal-system] [data-vibe=positive] {
	color: green;
	--xio-button-hover-color: white;
	--xio-button-hover-background: green;
}

[data-modal-system] [data-vibe=negative] {
	color: #a00;
	--xio-button-hover-color: white;
	--xio-button-hover-background: #a00;
}

`;

    function prepareModalSystemRendering(listModals) {
        const style = document.createElement("style");
        A(modalSystemStyles, style);
        const element = document.createElement("div");
        element.setAttribute("data-modal-system", "");
        function rerender() {
            A(y `
			${style}
			${listModals().map(modal => modal.render())}
		`, element);
        }
        return { element, rerender };
    }

    function prepareAlert(confirm) {
        return async ({ title, body = null, button = { label: "ok", vibe: "neutral" }, focusNthElement = 1, blanketClickMeansNo = true, }) => {
            await confirm({
                title,
                body,
                blanketClickMeansNo,
                yes: button,
                no: null,
            });
        };
    }

    function setupModalSystem() {
        let count = 0;
        const modals = new Map();
        const listModals = () => Array.from(modals.values());
        const { rerender, element } = prepareModalSystemRendering(listModals);
        function popup({ focusNthElement, renderContent, onBlanketClick }) {
            const id = count++;
            const controls = {
                rerender,
                close: () => {
                    modals.delete(id);
                    rerender();
                },
            };
            const top = window.pageYOffset
                ?? document.documentElement.scrollTop
                ?? document.body.scrollTop
                ?? 0;
            const handleBlanketClick = () => onBlanketClick(controls);
            modals.set(id, {
                render() {
                    return y `
					<div data-modal="${id}">
						<div data-blanket @click=${handleBlanketClick}></div>
						<div data-plate style="top: ${top}px">
							${renderContent(controls)}
						</div>
					</div>
				`;
                },
            });
            rerender();
            const modal = element.querySelector(`[data-modal="${id}"]`);
            trapFocus(modal, focusNthElement);
            return { controls, modal };
        }
        const confirm = prepareConfirm(popup);
        return {
            modalsElement: element,
            modals: {
                popup,
                confirm,
                alert: prepareAlert(confirm),
                prompt: preparePrompt(popup),
            },
        };
    }

    function parseQuery(query = location.search) {
        const parsed = {};
        query = query.startsWith("?") ? query.slice(1) : query;
        query = query.startsWith("#") ? query.slice(1) : query;
        const parts = query.split("&");
        for (const part of parts) {
            const [key, ...rest] = part.split("=");
            const value = rest.join("=");
            parsed[decodeURIComponent(key)] = decodeURIComponent(value);
        }
        return parsed;
    }

    async function loginWithLinkTokenOrUseExistingLogin({ link, accessModel, onDone, onError, }) {
        const { hash } = new URL(link);
        const { login } = parseQuery(hash);
        if (login) {
            try {
                await accessModel.login(login);
            }
            catch (error) {
                onError(error);
            }
            onDone();
        }
        else
            await accessModel.useExistingLogin();
    }

    async function assembleAndInitializeFrontend({ appId, popups, storage, authMediator, remote, }) {
        const { modals, modalsElement } = setupModalSystem();
        const models = await assembleModels({
            appId,
            remote,
            popups,
            storage,
            authMediator,
        });
        const components = getComponents({ models, modals });
        await loginWithLinkTokenOrUseExistingLogin({
            accessModel: models.accessModel,
            link: window.location.toString(),
            onError: error => {
                if (error instanceof AccessLoginExpiredError)
                    modals.alert({
                        title: "expired login link",
                        body: "this login link has expired, please try again",
                    });
                else
                    modals.alert({
                        title: "invalid login link",
                        body: "something is wrong with this login link, please try again",
                    });
            },
            onDone: () => {
                window.location.hash = "";
            },
        });
        return { components, models, modals, modalsElement };
    }

    void async function demos() {
        const connection = await mockConnectApp({
            appWindowLink: window.location.href,
            origins: [window.location.origin],
            storage: memoryFlexStorage(),
        });
        await demoQuestions(connection);
        connection.remote = addMockLatency({
            remote: connection.remote,
            latency: { min: 200, max: 800 },
        });
        const frontend = await assembleAndInitializeFrontend(connection);
        applyMockHacks({ connection, frontend });
        const xiome = { ...connection, ...frontend };
        document.body.prepend(xiome.modalsElement);
        await xiome.models.accessModel.sendLoginLink("creative@xiome.io");
        const demoComponents = {};
        for (const [key, value] of Object.entries(xiome.components))
            demoComponents[`Demo${key}`] = value;
        registerComponents(demoComponents);
    }();

})();
//# sourceMappingURL=demos.bundle.js.map
